/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConnectorProperties.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector;

import java.io.Serializable;

import javax.resource.ResourceException;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterAssociation;

import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * This class holds all properties for the connector. It implements the
 * interface ResourceAdapterAssociation which is used to associate its instances
 * to our JOnAS MQ connector.
 * 
 * @author Kien THAI
 * 
 */
public abstract class ConnectorProperties implements ResourceAdapterAssociation,
		Serializable {

	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	//TODO To verify to use this or the other
	private transient ResourceAdapter ra = null;
	//private ResourceAdapter ra = null;

	/**
	 * Transport type of queue manager connection
	 */
	private String transportType = Constants.CLIENT_MODE;

	/**
	 * Queue manager host name or IP address
	 */
	private String hostName = Constants.DEFAULT_HOST;

	/**
	 * Queue manager port
	 */
	private String port = Constants.DEFAULT_PORT;

	/**
	 * Server connection channel of queue manager
	 */
	private String channel = Constants.DEFAULT_CHANNEL;

	/**
	 * Queue manager name
	 */
	private String queueManager;

	/**
	 * A string that holds property name-value paires for the queue connection
	 * factories
	 */
	private String queueConnectionFactoryProperties = null;

	/**
	 * A string that holds property name-value paires for the topic connection
	 * factories
	 */
	private String topicConnectionFactoryProperties = null;

	/**
	 * A string that holds property name-value paires for the connection
	 * factories
	 */
	private String connectionFactoryProperties = null;

	/**
	 * Non-managed Connection Factory JNDI Name 
	 */
	private String jConnectionFactoryJndiName = Constants.JCF_JNDI_NAME;
	
	/**
	 * Non-managed Topic Connection Factory JNDI Name
	 */
	private String jTopicConnectionFactoryJndiName = Constants.JTCF_JNDI_NAME;
	
	/**
	 * Non-managed Queue Connection Factory Name
	 */
	private String jQueueConnectionFactoryJndiName = Constants.JQCF_JNDI_NAME;

	/**
	 * Authentication mode for Queue Manager JNDI namespace
	 */
	private String authenticationMode = Constants.NONE_AUTHENTICATION_MODE;

	/**
	 * To use in case, channel is secured
	 */
	private String jndiUserName = null;
	private String jndiPassword = null;
	
	/**
	 * Name of file that stores destination configuration
	 */
	private String configFileName = Constants.DEFAULT_CONFIG_FILENAME;
	
	/**
	 * MBean name of this instance of connector
	 */
	private String mBeanName = Constants.DEFAULT_MBEAN_NAME;

	public ResourceAdapter getResourceAdapter() {
		logger.debug("ConnectorProperties#getResourceAdapter()");
		return ra;
	}

	public void setResourceAdapter(ResourceAdapter ra) throws ResourceException {
		logger.debug("ConnectorProperties#setResourceAdapter(" + ra + ")");
		logger.info(toString()+"#setResourceAdapter(" + ra + ")");
		
		if (this.ra != null)
			throw new ResourceException("Can not change resource adapter "
					+ "association.");

		if (!(ra instanceof JonasMQConnector))
			throw new ResourceException("Provided resource adapter is not "
					+ "MQ Connector: " + ra.getClass().getName());

		this.ra = (JonasMQConnector) ra;
	}
	
	abstract public String toString();

	public String getChannel() {
		logger.debug("ConnectorProperties#getChannel()");
		return this.channel;
	}

	public void setChannel(String channel) {
		logger.debug("ConnectorProperties#setChannel(" + channel + ")");
		this.channel = channel;
	}

	public String getHostName() {
		logger.debug("ConnectorProperties#getHostName()");
		return this.hostName;
	}

	public void setHostName(String host) {
		logger.debug("ConnectorProperties#setHostName(" + host + ")");
		this.hostName = host;
	}

	public Integer getPort() throws ResourceException {
		logger.debug("ConnectorProperties#getPort()");
		try {
			return new Integer(this.port);
		} catch (Exception e) {
			throw ExceptionUtils.newResourceException
			(this.port + " is not a valid port number", e);
		}
	}
	
	public String getPortAsString() {
		logger.debug("ConnectorProperties#getPortAsString()");
		return this.port;
	}

	public void setPort(String port) {
		logger.debug("ConnectorProperties#setPort(" + port + ")");
		this.port = port;
	}

	public String getTransportType() {
		logger.debug("ConnectorProperties#getTransportType()");
		return this.transportType;
	}

	public void setTransportType(String transport) {
		logger.debug("ConnectorProperties#setTransportType(" + transport
				+ ")");
		this.transportType = transport;
	}

	public String getConnectionFactoryProperties() {
		logger.debug("ConnectorProperties#getConnectionFactoryProperties()");
		return connectionFactoryProperties;
	}

	public void setConnectionFactoryProperties(
			String connectionFactoryProperties) {
		logger.debug("ConnectorProperties#setConnectionFactoryProperties" + "("
				+ connectionFactoryProperties + ")");
		this.connectionFactoryProperties = connectionFactoryProperties;
	}

	public String getQueueManager() {
		logger.debug("ConnectorProperties#getQueueManager()");
		return this.queueManager;
	}

	public void setQueueManager(String qManager) {
		logger.debug("ConnectorProperties#setQueueManager(" + qManager
				+ ")");
		this.queueManager = qManager;
	}

	public String getJndiPassword() {
		logger.debug("ConnectorProperties#getJndiPassword()");
		return jndiPassword;
	}

	public void setJndiPassword(String password) {
		logger.debug("ConnectorProperties#setJndiPassword(" + password + ")");
		this.jndiPassword = password;
	}

	public String getJndiUserName() {
		logger.debug("ConnectorProperties#getJndiUserName()");
		return jndiUserName;
	}

	public void setJndiUserName(String userName) {
		logger.debug("ConnectorProperties#setUserName(" + userName + ")");
		this.jndiUserName = userName;
	}

	public String getAuthenticationMode() {
		logger.debug("ConnectorProperties#getAuthenticationMode()");
		return authenticationMode;
	}

	public void setAuthenticationMode(String authenticationMode) {
		logger.debug("ConnectorProperties#setAuthenticationMode("
				+ authenticationMode + ")");
		this.authenticationMode = authenticationMode;
	}

	public String getQueueConnectionFactoryProperties() {
		logger
				.debug("ConnectorProperties#getQueueConnectionFactoryProperties()");
		return queueConnectionFactoryProperties;
	}

	public void setQueueConnectionFactoryProperties(
			String queueConnectionFactoryProperties) {
		logger.debug("ConnectorProperties#setQueueConnectionFactoryProperties"
				+ "(" + queueConnectionFactoryProperties + ")");
		this.queueConnectionFactoryProperties = queueConnectionFactoryProperties;
	}

	public String getTopicConnectionFactoryProperties() {
		logger
				.debug("ConnectorProperties#getTopicConnectionFactoryProperties()");
		return topicConnectionFactoryProperties;
	}

	public void setTopicConnectionFactoryProperties(
			String topicConnectionFactoryProperties) {
		logger.debug("ConnectorProperties#setTopicConnectionFactoryProperties"
				+ "(" + topicConnectionFactoryProperties + ")");
		this.topicConnectionFactoryProperties = topicConnectionFactoryProperties;
	}

	public String getJConnectionFactoryJndiName() {
		return jConnectionFactoryJndiName;
	}

	public void setJConnectionFactoryJndiName(String connectionFactoryJndiName) {
		jConnectionFactoryJndiName = connectionFactoryJndiName;
	}

	public String getJQueueConnectionFactoryJndiName() {
		return jQueueConnectionFactoryJndiName;
	}

	public void setJQueueConnectionFactoryJndiName(
			String queueConnectionFactoryJndiName) {
		jQueueConnectionFactoryJndiName = queueConnectionFactoryJndiName;
	}

	public String getJTopicConnectionFactoryJndiName() {
		return jTopicConnectionFactoryJndiName;
	}

	public void setJTopicConnectionFactoryJndiName(
			String topicConnectionFactoryJndiName) {
		jTopicConnectionFactoryJndiName = topicConnectionFactoryJndiName;
	}

	public String getConfigFileName() {
		return configFileName;
	}

	public void setConfigFileName(String configFileName) {
		this.configFileName = configFileName;
	}

	public String getMBeanName() {
		return mBeanName;
	}

	public void setMBeanName(String beanName) {
		mBeanName = beanName;
	}

}
