/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JonasMQConnector.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector;

import java.util.ArrayList;

import javax.jms.XAConnection;
import javax.jms.XASession;
import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.resource.spi.work.WorkManager;
import javax.transaction.xa.XAResource;

import com.bull.jonasmqconnector.admin.JmsAdmin;
import com.bull.jonasmqconnector.inbound.ActivationSpecImpl;
import com.bull.jonasmqconnector.inbound.InboundConsumer;
import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * This is the main class of the connector. This acts as an interface to manage
 * the connector. Through this class, JOnAS can perform the following tasks: -
 * Establish the properties for the connector by the setter methods - Start
 * the connector by the method start - Stop the connector by the
 * method stop - Activate an endpoint by the method endpointActivation -
 * Deactivate an endpoint by the method endpointDeactivation - Recover
 * pending transactions by the method getXAResources.
 * 
 * @author Kien THAI
 * 
 */
public class JonasMQConnector extends ConnectorProperties implements
		ResourceAdapter {
	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	//private transient static JonasMQConnector instance = null;

	private transient BootstrapContext context = null;

	private transient JmsAdmin jmsAdmin = null;

	/**
	 * true if the adapter has been started.
	 */
	private boolean started = false;

	/**
	 * true if the adapter has been stopped.
	 */
	private boolean stopped = false;

	/**
	 * The connector is bootstrapped. Set up properties for the connector
	 * 
	 * @param context
	 *            BootstrapContext used to retrieve WorkManager reference
	 * @exception ResourceAdapterInternalException
	 *                If the JMS Admin object can not be created
	 */
	public void start(BootstrapContext context)
			throws ResourceAdapterInternalException {
		logger.debug("JonasMQConnector#start(" + context + ")");
		if (started)
			throw new ResourceAdapterInternalException(
					"The JMQ connector already started");
		if (stopped)
			throw new ResourceAdapterInternalException(
					"The JMQ connector already stopped.");

		//this.instance = this;
		this.context = context;

		try {
			jmsAdmin = new JmsAdmin(this);
		} catch (ResourceAdapterInternalException ne) {
			throw ExceptionUtils.newResourceAdapterInternalException(
					"Can not create JMS Admin for MQ", ne);
		}

		started = true;
		stopped = false;
	}

	public WorkManager getWorkManager() {
		logger.debug("JonasMQConnector#getWorkManager()");
		return context.getWorkManager();
	}

	public void stop() {
		logger.debug("JonasMQConnector#stop()");

		if (!started || stopped) {
			logger.warn("JonasMQConnector#stop() : " + "connector not active.");
			return;
		}

		try {
			this.jmsAdmin.stop();
		} catch (ResourceAdapterInternalException raie) {
			logger.warn("Can not stop JMS Admin for WSMQ" + raie);
		}

		stopped = true;
	}

	/**
	 * An MDB is deployed. Setup message delivery for the message endpoint
	 * 
	 * @param mef
	 *            a message endpoint factory instance for the deployed MDB
	 * @param spec
	 *            an activation spec for the deployed MDB
	 * @exception ResourceException
	 *                If the message consumer can not be created
	 */
	public void endpointActivation(MessageEndpointFactory mef,
			ActivationSpec spec) throws ResourceException {
		logger.debug("JonasMQConnector#endpointActivation(" + mef + "," + spec
				+ ")");

		if (!started || stopped)
			throw new ResourceException(
					"JonasMQConnector#endpointActivation : "
							+ "connector not active.");

		try {
			jmsAdmin.newConsumer(mef, spec);
		} catch (ResourceException e) {
			throw ExceptionUtils.newResourceException(
					"Can not start message consumer", e);
		}
	}

	/**
	 * An MDB is undeployed. Stop delivering messages to the message endpoint
	 * and perform clean tasks
	 * 
	 * @param mef
	 *            a message endpoint factory instance for the undeployed MDB
	 * @param spec
	 *            an activation spec for the undeployed MDB
	 */
	public void endpointDeactivation(MessageEndpointFactory mef,
			ActivationSpec spec) {
		logger.debug("JonasMQConnector#endpointDeactivation(" + mef + ","
				+ spec + ")");

		if (!started || stopped) {
			logger.warn("JonasMQConnector#endpointDeactivation : "
					+ "connector not active.");
			return;
		}
		try {
			jmsAdmin.stopConsumer(mef, spec);
		} catch (ResourceException re) {
			logger.warn("JonasMQConnector#endpointDeactivation : "
					+ "Can not deactivate endpoint : " + re.getMessage());
		}

	}

	/**
	 * Called during crash recovery in order to complete the pending
	 * transactions for message redelivery
	 * 
	 * @param specs
	 *            Array of activation spec instances correspond to the deployed
	 *            endpoints that was active prior to the crash of JOnAS
	 * @return Array of XA resources correspond to given activation spec
	 *         instances
	 */
	public XAResource[] getXAResources(ActivationSpec[] specs)
			throws ResourceException {
		logger.debug("JonasMQConnector#getXAResources(" + specs + ")");

		if (!started || stopped)
			throw new ResourceException(
					"JonasMQConnector#endpointActivation : "
							+ "connector not active.");

		ArrayList xAResources = new ArrayList();
		for (int i = 0; i < specs.length; i++) {
			ActivationSpecImpl tmpSpec = null;
			if (specs[i] instanceof ActivationSpecImpl) {
				tmpSpec = (ActivationSpecImpl) specs[i];
			} else {
				continue;
			}

			XAConnection xacon = null;
			XASession xasess = null;
			InboundConsumer consumer = null;
			try {
				consumer = new InboundConsumer(this, tmpSpec);
				xacon = (XAConnection) consumer.getConnection();
				xasess = xacon.createXASession();
				XAResource xaRes = xasess.getXAResource();
				xAResources.add(xaRes);
				logger.info("JonasMQConnector#getXAResources : "
						+ "Added XA Resource : " + xaRes);
			} catch (Exception e) {
				logger.warn("JonasMQConnector#getXAResources : " + e);
			}

		}
		if (xAResources.size() != 0)
			return (XAResource[]) xAResources.toArray();
		return null;
	}

	public JmsAdmin getJmsAdmin() throws ResourceException {
		logger.debug("JonasMQConnector#getJmsAdmin()");
		if (jmsAdmin == null)
			throw new ResourceException("JMS Admin instance is null");
		else
			return jmsAdmin;
	}

	public String toString() {
		String result = "JonasMQConnector#";
		if (getTransportType().equals(Constants.CLIENT_MODE)) {
			result = result 
					+ getHostName() + ":" 
					+ String.valueOf(getPortAsString())
					+ "/" + getChannel();

		} else if (getTransportType().equals(Constants.BINDING_MODE)) {
			result = result + getQueueManager();
		}

		return result;
	}
}