/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AdminDestination.java 10252 2007-04-23 11:27:02Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.admin;

import java.util.Hashtable;

import javax.jms.Destination;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;

import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.JavaBeanExecutor;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * This class acts as administration interface for a destination.
 * 
 * There are two types of property for a destination:
 * - JMS properties for queues and topics.
 * - Physic properties for queues only.
 * 
 * @author Kien THAI
 *
 */
public class AdminDestination implements AdminDestinationMBean {

	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	private Destination destination = null;

	private Hashtable propertiesTable = null;

	private String jndiName = null;

	private boolean isTopic;

	private AdminQm adminQm = null;

	private String queueName = null;
	
	private InitialContext ctx = null;

	public AdminDestination(Destination dest, Hashtable properties,
			String name, boolean isTopic, AdminQm qm)
			throws ResourceException {
		logger.debug("AdminDestination#AdminDestination(" + dest + "," + name
				+ "," + isTopic + "," + qm + ")");
		this.destination = dest;
		this.propertiesTable = properties;
		this.jndiName = name;
		this.isTopic = isTopic;
		this.adminQm = qm;

		if (!isTopic)
			queueName = ((com.ibm.mq.jms.MQQueue) this.destination)
					.getBaseQueueName();
		
		// Create JOnAS JNDI Context
		try {
			ctx = new InitialContext();
		} catch (NamingException ne) {
			throw ExceptionUtils.newResourceException(
					"Can not create JOnAS JNDI context", ne);
		}
	}

	/*************************************************************************
	 *  JMS DESTINATION PROPERTIES
	 *************************************************************************/
	
	
	public String getProperty(String propertyName) throws Exception {
		logger.debug("AdminDestination#getProperty(" + propertyName + ")");
		return JavaBeanExecutor.getProperty(destination, propertyName)
				.toString();
	}

	public void setProperty(String propertyName, String propertyValue)
			throws Exception {
		logger.debug("AdminDestination#setProperty(" + propertyName
				+ propertyValue + ")");
		
		JavaBeanExecutor.setProperty(destination, propertyName, propertyValue);
		propertiesTable.put(propertyName, propertyValue);
		ctx.rebind(jndiName, destination);
	}

	public Destination getJmsDestination() {
		logger.debug("AdminDestination#getJmsDestination()");
		return this.destination;
	}

	/*************************************************************************
	 *  PHYSIC QUEUE ATTRIBUTS
	 *************************************************************************/
	
	
	public int getCurrentDepth() throws Exception {
		logger.debug("AdminDestination#getCurrentDepth()");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getCurrentDepth(queueName);
	}

	public int getInputCount() throws Exception {
		logger.debug("AdminDestination#getInputCount()");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getInputCount(queueName);
	}

	public int getOutputCount() throws Exception {
		logger.debug("AdminDestination#getOutputCount()");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getOutputCount(queueName);
	}

	public int getMaxQueueDepth() throws Exception {
		logger.debug("AdminDestination#getMaxQueueDepth()");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getMaxQueueDepth(queueName);
	}

	public void setMaxQueueDepth(int maxDepth) throws Exception {
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		adminQm.setMaxQueueDepth(queueName, maxDepth);
	}

	public int getDepthHighLimit() throws Exception {
		logger.debug("AdminDestination#getDepthHighLimit()");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getDepthHighLimit(queueName);
	}

	public void setDepthHighLimit(int depthHighLimit) throws Exception {
		logger.debug("AdminDestination#setDepthHighLimit("+depthHighLimit+")");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		adminQm.setDepthHighLimit(queueName, depthHighLimit);
	}

	public int getServiceIntervalEvent() throws Exception {
		logger.debug("AdminDestination#getServiceIntervalEvent()");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getServiceIntervalEvent(queueName);
	}

	public void setServiceIntervalEvent(int type) throws Exception {
		logger.debug("AdminDestination#setServiceIntervalEvent("+type+")");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		adminQm.setServiceIntervalEvent(queueName, type);
	}

	public int getServiceInterval() throws Exception {
		logger.debug("AdminDestination#getServiceInterval()");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getServiceInterval(queueName);
	}

	public void setServiceInterval(int interval) throws Exception {
		logger.debug("AdminDestination#setServiceInterval("+interval+")");
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		adminQm.setServiceInterval(queueName, interval);
	}

	public boolean getDepthHighEvent() throws Exception {
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getDepthHighEvent(queueName);
	}

	public void setDepthHighEvent(boolean enable) throws Exception {
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		adminQm.setDepthHighEvent(queueName, enable);
	}

	public boolean getDefPersistence() throws Exception {
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		return adminQm.getDefPersistence(queueName);
	}

	public void setDefPersistence(boolean enable) throws Exception {
		if (isTopic)
			throw new Exception("a topic does not have this attribut");
		adminQm.setDefPersistence(queueName, enable);
	}

	/**
	 * this method will clear all messages in a queue
	 */
	public void clearQueue() throws Exception {
		logger.debug("AdminDestination#clearQueue()");
		if (isTopic)
			throw new Exception("a topic does not have this function");
		adminQm.clearQueue(queueName);
	}

	/*************************************************************************
	 *  JMS PROPERTIES GETTER
	 *************************************************************************/
	
	public Hashtable getPropertiesTable() {
		return propertiesTable;
	}

	public String getJndiName() {
		return this.jndiName;
	}

	public boolean getIsTopic() {
		return isTopic;
	}

}