/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AdminQm.java 10252 2007-04-23 11:27:02Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.admin;

import javax.jms.MessageConsumer;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.naming.InitialContext;

import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;
import com.ibm.mq.MQMessage;
import com.ibm.mq.jms.MQQueue;
import com.ibm.mq.pcf.CMQC;
import com.ibm.mq.pcf.CMQCFC;
import com.ibm.mq.pcf.MQCFH;
import com.ibm.mq.pcf.MQCFIN;
import com.ibm.mq.pcf.MQCFST;
import com.ibm.mq.pcf.PCFMessage;
import com.ibm.mq.pcf.PCFMessageAgent;
import com.ibm.mq.pcf.PCFParameter;

/**
 * An util class used for administer QueueManager
 * 
 * @author Kien THAI
 * 
 */
public class AdminQm {
	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();
	
	private PCFMessageAgent agent = null;

	private String jQcfJndiName;
	
	private boolean isEventMonitoring = false;
	
	private QueueConnection qconnection = null;
	
	private QueueSession qsession = null;
	
	/**
	 * The client mode for the queue manager connection
	 */
	private static final String CLIENT_MODE = "client";

	/**
	 * The binding mode for the queue manager connection
	 */
	private static final String BINDING_MODE = "binding";

	public AdminQm(String jQcfJndiName) {
		logger.debug("AdminQm#AdminQm("+jQcfJndiName+")");
		this.jQcfJndiName = jQcfJndiName;
	}
	
	/**
	 * This method starts administer queue manager.
	 * 
	 * @param transportType
	 * @param hostName
	 * @param channel
	 * @param port
	 * @param queueManager
	 * @throws Exception
	 */
	public void start (
			String transportType,
			String hostName,
			String channel,
			String port,
			String queueManager
			) throws Exception {
		logger.debug("AdminQm#start("+transportType+","+hostName+","+
				channel+","+port+","+queueManager+")");
		
		agent = initAgent(
					transportType,
					hostName,
					channel,
					port,
					queueManager);
		
		// TODO To implement onMessage function in QueueEventListener class
		// in order to monitor queues in this queue manager
		//monitoring();
	}
	
	public void stop() throws Exception {
		logger.debug("AdminQm#stop()");
		
		if (isEventMonitoring) {
			qsession.close();
			qconnection.close();
		}
		
		agent.disconnect();
	}

	/***************************************************************************
	 * QUEUE MANAGER ADMINISTRATION METHODS
	 **************************************************************************/
	
	
	/**
	 * Enable a queue manager to generate queue performance events
	 * 
	 * @param enable
	 * @throws Exception
	 */
	public void enablePerformanceEvent(boolean enable) throws Exception {
		PCFParameter[] parameters = null;
		

		parameters = new PCFParameter[] { new MQCFIN(
				CMQC.MQIA_PERFORMANCE_EVENT, this.parseBoolean(enable)), };

		try {
			MQMessage[] pcfResponses = agent.send(CMQCFC.MQCMD_CHANGE_Q_MGR,
					parameters);
			MQCFH cfh = new MQCFH(pcfResponses[0]);
			int iReasonCode = cfh.reason;
			if (iReasonCode != 0)
				throw new Exception("Can not change queue, reason code: "
						+ iReasonCode);
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not send PCF message", e);
		} finally {
		}
	}

	/**
	 * Check if a queue manager enable performance events or not
	 * 
	 * @return
	 */
	public boolean isPerformanceEventEnabled() throws Exception {
		int performanceEventEnabled = -1;

		try {
			int[] attrs = { CMQC.MQIA_PERFORMANCE_EVENT };
			PCFMessage[] responses = null;
			try {
				PCFMessage message = new PCFMessage(CMQCFC.MQCMD_INQUIRE_Q_MGR);
				message.addParameter(CMQCFC.MQIACF_Q_MGR_ATTRS, attrs);
				responses = agent.send(message);
			} catch (Exception e) {
				throw ExceptionUtils
						.newException("Can not send PCF message", e);
			} finally {
			}

			performanceEventEnabled = ((Integer) responses[0]
					.getParameterValue(CMQC.MQIA_PERFORMANCE_EVENT)).intValue();
			return this.parseInt(performanceEventEnabled);
		} catch (Exception e) {
			throw ExceptionUtils.newException(
					"Can not know if performance event is enabled or not", e);
		}
	}

	/**
	 * Start listening to performance event queue
	 * 
	 * @throws Exception
	 */
	private void monitoring() throws Exception {
		
		com.ibm.mq.jms.MQQueue perfEventQueue = new MQQueue();
		
		// TODO To parameterize performance event queue name
		perfEventQueue.setBaseQueueName("SYSTEM.ADMIN.PERFM.EVENT");
		
		InitialContext ctx = new InitialContext();
		
		QueueConnectionFactory qcf = (QueueConnectionFactory)ctx.lookup
		(jQcfJndiName);
		
		qconnection = qcf.createQueueConnection();
		
		qsession = 
			qconnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
		MessageConsumer mc = qsession.createConsumer(
				perfEventQueue);
		
		mc.setMessageListener(new QueueEventListener());
		
		logger.info("Queue Event Listener: Waiting for Event messages ...");
		
		qconnection.start();
		isEventMonitoring = true;
	}
	
	
	
	/***************************************************************************
	 * QUEUE ADMINISTRATION METHODS
	 **************************************************************************/
	
	public int getCurrentDepth(String queueName) throws Exception {
		logger.debug("AdminDestination#getCurrentDepth()");
		return inquireQueue(CMQC.MQIA_CURRENT_Q_DEPTH, queueName);
	}

	public int getInputCount(String queueName) throws Exception {
		logger.debug("AdminDestination#getInputCount()");
		return inquireQueue(CMQC.MQIA_OPEN_INPUT_COUNT, queueName);
	}

	public int getOutputCount(String queueName) throws Exception {
		logger.debug("AdminDestination#getOutputCount()");
		return inquireQueue(CMQC.MQIA_OPEN_OUTPUT_COUNT, queueName);
	}

	public int getMaxQueueDepth(String queueName) throws Exception {
		logger.debug("AdminDestination#getMaxQueueDepth()");
		return inquireQueue(CMQC.MQIA_MAX_Q_DEPTH, queueName);
	}

	public void setMaxQueueDepth(String queueName, int maxDepth) throws Exception {
		changeQueue(new MQCFIN(CMQC.MQIA_MAX_Q_DEPTH, maxDepth), queueName);
	}

	public int getDepthHighLimit(String queueName) throws Exception {
		logger.debug("AdminDestination#getDepthHighLimit()");
		return inquireQueue(CMQC.MQIA_Q_DEPTH_HIGH_LIMIT, queueName);
	}

	public void setDepthHighLimit(String queueName, int depthHighLimit) 
	throws Exception {
		changeQueue(
				new MQCFIN(CMQC.MQIA_Q_DEPTH_HIGH_LIMIT, depthHighLimit), 
				queueName);
	}

	public int getServiceIntervalEvent(String queueName) throws Exception {
		logger.debug("AdminDestination#getServiceIntervalEvent()");
		return inquireQueue(CMQC.MQIA_Q_SERVICE_INTERVAL_EVENT, queueName);
	}

	public void setServiceIntervalEvent(String queueName, int type) 
	throws Exception {
		changeQueue
		(new MQCFIN(CMQC.MQIA_Q_SERVICE_INTERVAL_EVENT, type), queueName);
	}

	public int getServiceInterval(String queueName) throws Exception {
		logger.debug("AdminDestination#getServiceInterval()");
		return inquireQueue(CMQC.MQIA_Q_SERVICE_INTERVAL, queueName);
	}

	public void setServiceInterval(String queueName, int interval) 
	throws Exception {
		changeQueue(new MQCFIN(CMQC.MQIA_Q_SERVICE_INTERVAL, interval), queueName);
	}

	public boolean getDepthHighEvent(String queueName) throws Exception {
		return this.parseInt(
				inquireQueue(CMQC.MQIA_Q_DEPTH_HIGH_EVENT, queueName));
	}

	public void setDepthHighEvent(String queueName, boolean enable) 
	throws Exception {
		changeQueue(new MQCFIN(CMQC.MQIA_Q_DEPTH_HIGH_EVENT, this
				.parseBoolean(enable)), queueName);
	}

	public boolean getDefPersistence(String queueName) throws Exception {
		return this.parseInt
		(inquireQueue(CMQC.MQIA_DEF_PERSISTENCE, queueName));
	}

	public void setDefPersistence(String queueName, boolean enable) 
	throws Exception {
		changeQueue(new MQCFIN(CMQC.MQIA_DEF_PERSISTENCE, this
				.parseBoolean(enable)), queueName);
	}

	public void clearQueue(String queueName) throws Exception {
		logger.debug("AdminDestination#clearQueue()");

		PCFMessage[] responses = null;
		try {
			PCFMessage message = new PCFMessage(CMQCFC.MQCMD_CLEAR_Q);
			message.addParameter(CMQC.MQCA_Q_NAME, queueName);

			responses = agent.send(message);
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not send PCF message", e);
		}

		int reason = responses[0].getReason();
		if (reason != 0)
			throw new Exception("PCF Command failed, Reason: " + reason);
	}

	/***************************************************************************
	 * PRIVATE FUNCTIONS
	 **************************************************************************/

	/**
	 * Execute a PCF Command to inquire queue information
	 * 
	 * @param PCF
	 *            Command
	 * @param Attributs
	 * @return Result PCF Messages
	 * @throws Exception
	 *             if PCF Command can not be executed
	 */
	private int inquireQueue(int attr, String queueName) throws Exception {

		PCFMessage[] responses = null;

		int result = -1;

		int[] attrs = { attr };

		try {
			PCFMessage message = new PCFMessage(CMQCFC.MQCMD_INQUIRE_Q);
			message.addParameter(CMQC.MQCA_Q_NAME, queueName);
			message.addParameter(CMQCFC.MQIACF_Q_ATTRS, attrs);
			responses = agent.send(message);

			result = ((Integer) responses[0].getParameterValue(attr))
					.intValue();
			return result;
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not send PCF message", e);
		} 	
	}

	/**
	 * Execute a PCF Command to change a queue property
	 * 
	 * @param parameters
	 *            Queue properties
	 * @return The result
	 * @throws Exception
	 */
	private void changeQueue
	(PCFParameter parameter, String queueName) throws Exception {

		try {

			PCFParameter[] parameters = new PCFParameter[] {
					new MQCFST(CMQC.MQCA_Q_NAME, queueName),
					new MQCFIN(CMQC.MQIA_Q_TYPE, CMQC.MQQT_LOCAL), parameter };
			MQMessage[] pcfResponses = agent.send(CMQCFC.MQCMD_CHANGE_Q,
					parameters);
			MQCFH cfh = new MQCFH(pcfResponses[0]);
			int iReasonCode = cfh.reason;
			if (iReasonCode != 0)
				throw new Exception("Can not change queue, reason code: "
						+ iReasonCode);
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not send PCF message", e);
		} 	
	}

	/**
	 * Initial PCF Message Agent
	 * 
	 * @return
	 * @throws Exception
	 */
	private PCFMessageAgent initAgent(
			String transportType,
			String hostName,
			String channel,
			String port,
			String queueManager)
			throws Exception {

		PCFMessageAgent agent = null;

		try {
			if (transportType.equals(CLIENT_MODE))
				agent = new PCFMessageAgent(
						hostName, 
						(new Integer(port)).intValue(), 
						channel);
			
			else if (transportType.equals(BINDING_MODE))
				agent = new PCFMessageAgent(queueManager);
			else
				throw new Exception(
						"Unknown Transport Type for QueueManager Connection");
			return agent;
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not create PCFAgent", e);
		}
	}

	private int parseBoolean(boolean b) {
		return b ? 1 : 0;
	}

	private boolean parseInt(int i) throws Exception {
		if (i == 0)
			return false;
		else if (i == 1)
			return true;
		else
			throw new Exception("Can not parse this int value to boolean");
	}
}