/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JMXServer.java 10252 2007-04-23 11:27:02Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.admin;

import java.util.List;
import java.util.ListIterator;
import java.util.Set;

import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;
import javax.management.RuntimeOperationsException;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * An MBeanServer wrapper class for facilitate registering and unregistering the
 * MBeans of this connector
 * 
 * @author Kien THAI
 * 
 */
public class JMXServer {
	
	private static final String CONNECTOR_TYPE = "JonasMQConnector";
	
	private static final String DESTINATION_TYPE = "MQDestination";
	
	private static final String CONSUMER_TYPE = "MDB";
	
	/**
	 * Logger for the class.
	 */
	private static Logger logger = LogUtils.getLogger();
	
	private MBeanServer jmxserver = null;

	private String domain = null;
	
	private String connectorMBeanName = null;

	public JMXServer(Object connectorMBean, String mBeanName) throws Exception {
		
		logger.debug("JMXServer#JMXServer("+connectorMBean+","+mBeanName+")");
		connectorMBeanName = mBeanName;

		/*
		 * Retrieve the MBean server. 
		 * it may not be the first one, especially if the JVM mbean server is set 
		 * Look for the mbean server that holds the J2EE server
		 */

		// Default mbean server and domain if no J2EEServer is found
		List mbeanServers = MBeanServerFactory.findMBeanServer(null);
		if (mbeanServers != null && mbeanServers.size() > 0) {
			jmxserver = (MBeanServer) mbeanServers.get(0);
		}
		this.domain = jmxserver.getDefaultDomain();
		
		// Look for a J2EEServer. if found override the default.
		if (mbeanServers != null && mbeanServers.size() > 0) {
			ObjectName j2eeServerON = ObjectName.getInstance("*:j2eeType=J2EEServer,*");
			ListIterator it = mbeanServers.listIterator();
			while (it.hasNext()) {
				MBeanServer mbs = (MBeanServer) it.next();
				Set j2eeServers = mbs.queryNames(j2eeServerON, null);
				if (!j2eeServers.isEmpty()) {
					// retrieve the domain from the J2EEServer domain key.
					this.domain = ((ObjectName)j2eeServers.iterator().next()).getDomain();
					jmxserver = mbs;
					break;
				}
			}
		}
		
		registerConnector(connectorMBean, mBeanName);
	}

	
	
	/************************************************************************
	 * CONNECTOR MBEAN
	 ************************************************************************/
	
	
	public void registerConnector(Object bean, String name) throws Exception {
		logger.debug("JMXServer#registerConnector("+bean+","+name+")");
		
		registerMBean(bean, CONNECTOR_TYPE, buildConnectorNameSuffix(name));
	}
	
	public String buildConnectorNameSuffix(String name) {
		
		StringBuffer suffix = new StringBuffer();
		if (name != null)
			suffix.append("," + "name=" + name);
		
		return suffix.toString();
	}

	public void unregisterConnector(String name) throws Exception {
		logger.debug("JMXServer#unregisterConnector("+name+")");
		
		unregisterMBean(CONNECTOR_TYPE, buildConnectorNameSuffix(name));
	}
	
	
	
	/************************************************************************
	 * CONSUMER MBEAN
	 ************************************************************************/
	
	
	public void registerConsumer(Object bean, 
			String name, String destinationName, Integer key) 
	throws Exception {
		
		logger.debug("JMXServer#registerConsumer("+bean+","
				+name+","+destinationName+","+key+")");
		
		String suffix = buildConsumerNameSuffix(name, destinationName, key);
		
		registerMBean(bean, CONSUMER_TYPE, suffix);
	}
	
	private String buildConsumerNameSuffix
	(String name, String destinationName, Integer key) {

		StringBuffer suffix = new StringBuffer();
		String name2 = name + "#" + key;

		if (name2 != null)
			suffix.append("," + "name=" + name2);
		
		if (destinationName != null)
			suffix.append("," + DESTINATION_TYPE + "=" + destinationName);
		
		if (connectorMBeanName != null)
			suffix.append("," + CONNECTOR_TYPE + "=" + connectorMBeanName);

		return suffix.toString();
	}
	
	public String buildConsumerObjectName
	(String name, String destinationName, Integer key) {
		StringBuffer strbuf = new StringBuffer();
		String suffix = buildConsumerNameSuffix(name, destinationName, key);
		
		strbuf.append(domain + ":");
		strbuf.append("type=" + CONSUMER_TYPE);
		if (suffix != null)
			strbuf.append(suffix);
		return strbuf.toString();
	}

	public void unregisterConsumer(String name,
			String destinationName, Integer key) throws Exception {
		
		logger.debug("JMXServer#unregisterConsumer("+name
				+","+destinationName+","+key+")");
		
		String suffix = buildConsumerNameSuffix(name, destinationName, key);
						
		unregisterMBean(CONSUMER_TYPE, suffix);
	}
	
	
	
	/************************************************************************
	 * DESTINATION MBEAN
	 ************************************************************************/
	
	
	public void registerDestination(Object bean, String name) throws Exception {
		
		logger.debug("JMXServer#registerDestination("+bean+","+name+")");
				
		registerMBean(bean, DESTINATION_TYPE, buildDestinationNameSuffix(name));
	}
	
	private String buildDestinationNameSuffix(String name) {
		
		StringBuffer suffix = new StringBuffer();
		
		if (name != null)
			suffix.append("," + "name=" + name);
		
		if (connectorMBeanName != null) 
			suffix.append("," + CONNECTOR_TYPE + "=" + connectorMBeanName);
		
		return suffix.toString();
	}
	
	public String buildDestinationObjectName
	(String name) {
		StringBuffer strbuf = new StringBuffer();
		String suffix = null;
		
		if (name != null)
			suffix = buildDestinationNameSuffix(name);
		
		strbuf.append(domain + ":");
		strbuf.append("type=" + DESTINATION_TYPE);
		
		if (suffix != null)
			strbuf.append(suffix);
		
		return strbuf.toString();
	}
	
	public void unregisterDestination(String name) throws Exception {
		logger.debug("JMXServer#unregisterDestination("+name+")");
			
		unregisterMBean(DESTINATION_TYPE, buildDestinationNameSuffix(name));
	}
	
	
	/************************************************************************
	 * PRIVATE METHODS
	 ************************************************************************/
	
	
	private void registerMBean
	(Object bean, String type, String suffix) throws Exception {
		if (jmxserver == null)
			return;

		StringBuffer strbuf = new StringBuffer();
		strbuf.append(domain + ":");
		
		if (type != null)
			strbuf.append("type=" + type);
		
		if (suffix != null)
			strbuf.append(suffix);

		logger.info("JMXServer#registerMBean MBeanName: " + strbuf.toString());
		
		try {
			jmxserver.registerMBean(bean, new ObjectName(strbuf.toString()));
		} catch (InstanceAlreadyExistsException exc) {
			logger.warn
			("The MBean is already under the control of the MBean server.");
			throw exc;
		} catch (MBeanRegistrationException exc) {
			// The preRegister (MBeanRegistration interface) method of the MBean
			// has thrown an exception. The MBean will not be registered.
			throw exc;
		} catch (NotCompliantMBeanException exc) {
			logger.warn("This object is not a JMX compliant MBean");
			throw exc;
		} catch (RuntimeOperationsException exc) {
			// Wraps a java.lang.IllegalArgumentException
			throw exc;
		}
	}
	
	private void unregisterMBean(String type, String suffix) throws Exception {
		if (jmxserver == null)
			return;

		StringBuffer strbuf = new StringBuffer();
		strbuf.append(domain + ":");
		
		if (type != null)
			strbuf.append("type=" + type);
				
		if (suffix != null)
			strbuf.append(suffix);

		try {
			jmxserver.unregisterMBean(new ObjectName(strbuf.toString()));
		} catch (InstanceNotFoundException exc) {
			logger.warn("The MBean is not registered in the MBean server.");
			throw exc;
		} catch (MBeanRegistrationException exc) {
			// The preDeregister (MBeanRegistration interface) method of the
			// MBean
			// has thrown an exception.
			throw exc;
		} catch (RuntimeOperationsException exc) {
			// Wraps a java.lang.IllegalArgumentException
			throw exc;
		}
	}
}