/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JmsAdmin.java 10252 2007-04-23 11:27:02Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.admin;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.jms.Topic;
import javax.jms.TopicConnectionFactory;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;

import com.bull.jonasmqconnector.JonasMQConnector;
import com.bull.jonasmqconnector.inbound.ActivationSpecImpl;
import com.bull.jonasmqconnector.inbound.InboundConsumer;
import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.JavaBeanBuilder;
import com.bull.jonasmqconnector.util.JavaBeanExecutor;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * An instance of this class is used for administering JMS objects of a queue
 * manager.
 * 
 * It can perform the following tasks: - Manage JMS connection factories for the
 * queue manager. - Manage JMS destinations (queues and topics) of the queue
 * manager.
 * 
 * @author Kien THAI
 * 
 */
public class JmsAdmin implements JmsAdminMBean {
	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	private JonasMQConnector connector;

	private JavaBeanBuilder builder;

	private transient InitialContext jonasContext = null;

	/**
	 * Non managed Connection Factories
	 */
	private ConnectionFactory cf = null;

	private QueueConnectionFactory qcf = null;

	private TopicConnectionFactory tcf = null;

	/**
	 * Non managed Connection Factory JNDI Names
	 */
	private String jCfName = null;

	private String jQcfName = null;

	private String jTcfName = null;

	/**
	 * Connection Factory Properties Hashtables
	 */
	private Hashtable cfProperties = null;

	private Hashtable qcfProperties = null;

	private Hashtable tcfProperties = null;

	/**
	 * Connection String for QueueManager
	 */
	private String url = null;

	/**
	 * A table to store Inbound Consumers
	 */
	private transient Hashtable consumers;

	private JMXServer jmxServer = null;

	/**
	 * A table that hold all AdminDestinations
	 */
	private Hashtable adminDestinations = null;

	/**
	 * An object that hold destination configuration
	 */
	private XmlConfig conf = null;

	/**
	 * An MBean to monitor queue performance event
	 */
	private AdminQm adminQm = null;

	public JmsAdmin(JonasMQConnector cn)
			throws ResourceAdapterInternalException {
		logger.debug("JmsAdmin#JmsAdmin()");

		// New Consumers Table
		this.consumers = new Hashtable();

		// New Destinations Table
		this.adminDestinations = new Hashtable();

		connector = cn;

		// Create JOnAS JNDI Context
		try {

			jonasContext = new InitialContext();
		} catch (NamingException ne) {
			throw ExceptionUtils.newResourceAdapterInternalException(
					"Can not create JOnAS JNDI context", ne);
		}

		// Create JavaBean builder
		try {
			this.builder = new JavaBeanBuilder();
		} catch (Exception e) {
			throw ExceptionUtils.newResourceAdapterInternalException(
					"Can not init JavaBean builder", e);
		}

		// Retrieve the Connection Factories Properties
		cfProperties = JavaBeanExecutor.parseToProperties(connector
				.getConnectionFactoryProperties());
		tcfProperties = JavaBeanExecutor.parseToProperties(connector
				.getTopicConnectionFactoryProperties());
		qcfProperties = JavaBeanExecutor.parseToProperties(connector
				.getQueueConnectionFactoryProperties());

		// Initial Non-managed Connection Factory for Client application
		try {
			jCfName = connector.getJConnectionFactoryJndiName();
			cf = (ConnectionFactory) createConnectionFactory(Constants.MQ_CF_CLASSNAME);
			jonasContext.rebind(jCfName, cf);
		} catch (Exception e) {
			logger.error("Can not create non-managed connection factories", e);
		}

		// Initial Non-managed Topic Connection Factory for Client application
		try {
			jTcfName = connector.getJTopicConnectionFactoryJndiName();
			tcf = (TopicConnectionFactory) createConnectionFactory(Constants.MQ_TCF_CLASSNAME);
			jonasContext.rebind(jTcfName, tcf);
		} catch (Exception e) {
			logger.error(
					"Can not create non-managed topic connection factories", e);
		}

		// Initial Non-managed Queue Connection Factory for Client application
		try {
			jQcfName = connector.getJQueueConnectionFactoryJndiName();
			qcf = (QueueConnectionFactory) createConnectionFactory(Constants.MQ_QCF_CLASSNAME);
			jonasContext.rebind(jQcfName, qcf);
		} catch (Exception e) {
			logger.error(
					"Can not create non-managed queue connection factories", e);
		}

		// Retrieve JOnAS JMX Server and Register this connector as an MBean
		try {

			this.jmxServer = new JMXServer(this, connector.getMBeanName());

		} catch (Exception e) {
			logger.error("Can not register " + this + " as MBean", e);
		}

		// Create QueueManager admin object
		adminQm = new AdminQm(connector.getJQueueConnectionFactoryJndiName());

		// TODO To implement onMessage method in QueueEventListener
		// Start administering Queue Manager
		try {
			adminQm.start(connector.getTransportType(),
					connector.getHostName(), connector.getChannel(), connector
							.getPortAsString(), connector.getQueueManager());

		} catch (Exception e) {
			logger.error("Can not start administer this Queue Manager", e);
		}

		// Create adminDestinations from XML Config file
		try {
			conf = new XmlConfig(connector.getConfigFileName(), this);
			conf.loadConfig();
		} catch (Exception e) {
			logger.error("Can not create destination from config file", e);
		}

	}

	/**
	 * Stop all the opened consumer
	 * 
	 * @throws ResourceAdapterInternalException
	 */
	public void stop() throws ResourceAdapterInternalException {
		logger.debug("JmsAdmin#stop()");

		Integer key = null;
		InboundConsumer consumer = null;

		// Close all consumer
		for (Enumeration e = consumers.keys(); e.hasMoreElements();) {

			key = (Integer) e.nextElement();
			consumer = (InboundConsumer) consumers.get(key);

			try {
				jmxServer.unregisterConsumer((consumer.getSpec()).getMdbName(),
						(consumer.getSpec()).getDestination(), key);
			} catch (Exception mbeanEx) {
				logger.error("Can not unregister consumer MBean", mbeanEx);
			}

			try {
				consumer.stop();
			} catch (Exception re) {
				logger.error("Can not stop one of the consumers", re);
			}
		}

		// Delete all jms destinations (Not delete physic queues)
		deleteAllJmsDestination();

		// Stop administering Queue Manager
		try {
			adminQm.stop();
		} catch (Exception e) {
			logger.error(
					"Exception while stopping Queue Manager administering", e);
		}

		// unregister connector MBean
		try {
			jmxServer.unregisterConnector(connector.getMBeanName());
		} catch (Exception e) {
			logger
					.error(
							"Can not unregister from the MBean Server this JmsAdmin",
							e);
		}

	}

	/**
	 * Create a connection factory
	 * 
	 * @param className
	 *            Class name of the connection factory
	 * 
	 * @return
	 * @throws Exception
	 */
	public java.lang.Object createConnectionFactory(String className)
			throws Exception {
		logger.debug("JmsAdmin#createConnectionFactory(" + className + ")");
		return builder.build(className,
				getConnectionFactoryProperties(className));
	}

	/**
	 * This method is used by ManagedConnectionFactory to select connection from
	 * connection pool de JOnAS
	 * 
	 * @param className
	 * @return
	 * @throws Exception
	 */
	public Hashtable getConnectionFactoryProperties(String className)
			throws Exception {

		logger.debug("JmsAdmin#getConnectionFactoryProperties(" + className
				+ ")");

		Hashtable properties = null;
		if (className.equals(Constants.MQ_TCF_CLASSNAME)
				|| className.equals(Constants.MQ_XA_TCF_CLASSNAME))
			properties = tcfProperties;
		else if (className.equals(Constants.MQ_QCF_CLASSNAME)
				|| className.equals(Constants.MQ_XA_QCF_CLASSNAME))
			properties = qcfProperties;
		else
			properties = cfProperties;

		String transportType = connector.getTransportType();

		if (transportType.equals(Constants.CLIENT_MODE)) {
			properties.put("TransportType", Constants.MQ_CLIENT_MODE);
			properties.put("HostName", connector.getHostName());
			properties.put("Port", connector.getPortAsString());
			properties.put("Channel", connector.getChannel());
		} else if (transportType.equals(Constants.BINDING_MODE)) {
			properties.put("TransportType", Constants.MQ_BINDING_MODE);
			properties.put("QueueManager", connector.getQueueManager());
		} else
			throw new Exception(
					"Can not determine this QueueManager connection mode: "
							+ connector.getTransportType());
		return properties;
	}

	/**
	 * Prepare session pool for message delivery for an MDB
	 * 
	 * @param mef
	 *            Message Endpoint Factory of the MDB
	 * @param spec
	 *            Activation Specifications of the MDB
	 * @throws ResourceException
	 */
	public void newConsumer(MessageEndpointFactory mef, ActivationSpec spec)
			throws ResourceException {
		logger.debug("JmsAdmin#newConsumer(" + mef + "," + spec + ")");

		InboundConsumer consumer = null;
		try {
			consumer = new InboundConsumer(this.connector, mef, spec);
		} catch (ResourceException re) {
			throw ExceptionUtils.newResourceException(
					"Can not create Inbound Consumer", re);
		}

		// Start the consumer
		try {
			consumer.start();
		} catch (ResourceException re) {
			throw ExceptionUtils.newResourceException("Can not start consumer",
					re);
		}

		// Register the consumer to the consumers list
		Integer key = new Integer(mef.hashCode() + spec.hashCode());
		consumers.put(key, consumer);

		// Register the consumer an MBean
		try {
			jmxServer.registerConsumer(consumer, ((ActivationSpecImpl) spec)
					.getMdbName(),
					((ActivationSpecImpl) spec).getDestination(), key);
		} catch (Exception e) {
			logger.error("Can not Register as MBean this consumer " + consumer,
					e);
		}

	}

	/**
	 * Stop message delivery for an MDB
	 * 
	 * @param mef
	 *            Message Endpoint Factory of the MDB
	 * @param spec
	 *            Activation Specifications of the MDB
	 * @throws ResourceException
	 */
	public void stopConsumer(MessageEndpointFactory mef, ActivationSpec spec)
			throws ResourceException {
		logger.debug("JmsAdmin#stopConsumer(" + mef + "," + spec + ")");
		InboundConsumer consumer;

		// Remove the consumer from the consumers list
		Integer key = new Integer(mef.hashCode() + spec.hashCode());
		consumer = ((InboundConsumer) consumers.remove(key));

		// Unregister the consumer from the JMX Server
		try {
			jmxServer.unregisterConsumer(((ActivationSpecImpl) spec)
					.getMdbName(),
					((ActivationSpecImpl) spec).getDestination(), key);
		} catch (Exception e) {
			logger.error("Can not UnRegister this consumer " + consumer
					+ " from MBeanServer", e);
		}

		// Stop the consumer
		if (consumer != null)
			consumer.stop();
	}

	public String toString() {
		String result = "JmsAdmin#" + connector.getMBeanName();
		return result;
	}

	public Hashtable getAdminDestinations() {
		return adminDestinations;
	}

	/**
	 * Create a jms destination and its physic destination.
	 * 
	 * @param isTopic
	 *            if destination is a topic or a queue.
	 * @param jndiName
	 *            JOnAS JNDI Name.
	 * @param properties
	 * @return Destination.
	 * @throws ResourceException
	 */
	public Destination createDestination(boolean isTopic, String jndiName,
			Hashtable properties) throws ResourceException {
		logger.debug("JmsAdmin#createDestination(" + isTopic + "," + jndiName
				+ "," + properties + ")");

		AdminDestination adminDest = null;
		Destination dest = null;

		if (jndiName == null)
			throw new ResourceException("JNDI Name is null");

		// Get destination from adminDestinations table
		adminDest = (AdminDestination) adminDestinations.get(jndiName);

		// Existing destination is not of the same type
		// with the destination to create
		if ((adminDest != null) && (adminDest.getIsTopic()) != isTopic)
			throw new ResourceException("Destination already exist but "
					+ "in a diffent type");

		// If admin destination does not exist
		if (adminDest == null) {
			logger.debug("Destination " + jndiName + " does not exist");

			if (isTopic) {
				// Set base topic name for the topic if it is not set
				if (!properties.containsKey(Constants.TOPIC_NAME_PROPERTY)) {
					properties.put(Constants.TOPIC_NAME_PROPERTY, jndiName);

				}
			} else {
				// Set base queue name for the queue if it is not set
				if (!properties.containsKey(Constants.QUEUE_NAME_PROPERTY)) {
					properties.put(Constants.QUEUE_NAME_PROPERTY, jndiName);

				}
			}

			// Create physic destination and overwrite existing physic
			// destination
			// if this was a topic
			dest = createMQDestination(isTopic, properties);

			// Bind destination in JOnAS JNDI Context
			try {

				jonasContext.bind(jndiName, dest);
			} catch (Exception e) {
				throw ExceptionUtils.newResourceException("Can not bind "
						+ jndiName + "in JOnAS JNDI context", e);
			}

			// Create new admin destination
			adminDest = new AdminDestination(dest, properties, jndiName,
					isTopic, this.adminQm);

			// Add admin destination to Hashtable
			adminDestinations.put(jndiName, adminDest);

			// TODO We can not overwrite the others MBean.
			// // Unregister existing destination from MBean Server
			// try {
			// jmxServer.unregisterDestination(jndiName);
			// } catch (Exception e) {
			//				
			// }

			// Register the new destination as MBean
			try {
				jmxServer.registerDestination(adminDest, jndiName);
			} catch (Exception e) {
				logger.error("Can not register destination " + jndiName
						+ " as MBean", e);
			}
		}

		// Return physic destination
		return adminDest.getJmsDestination();
	}

	/***************************************************************************
	 * PRIVATE FUNCTIONS
	 **************************************************************************/

	/**
	 * Create JMS MQ destination
	 * 
	 * @param isTopic
	 *            if this will be a topic
	 * @param properties
	 *            destination properties
	 * @return JMS MQ destination
	 * @throws ResourceException
	 */
	private Destination createMQDestination(boolean isTopic,
			Hashtable propertiesTable) throws ResourceException {
		logger.debug("JmsAdmin#createMQDestination(" + isTopic + ","
				+ propertiesTable + ")");
		Destination mqDest = null;

		if (isTopic) {
			// Create MQ Topic
			try {
				logger.debug("JmsAdmin#createMQDestination builder#build("
						+ Constants.MQ_TOPIC_CLASSNAME + "," + propertiesTable
						+ ")");

				mqDest = (Topic) builder.build(Constants.MQ_TOPIC_CLASSNAME,
						propertiesTable);

			} catch (Exception e) {
				throw ExceptionUtils.newResourceException(
						"Can not create JMS MQ topic", e);
			}
		} else {
			// Create MQ Queue
			// Get base queue name
			String queueName = propertiesTable.get(
					Constants.QUEUE_NAME_PROPERTY).toString();

			// Initialize MQ JNDI Context
			InitialContext mqContext = this.createMqJndiContext();

			try {
				// Try to lookup queue from MQ JNDI Context
				// and try to cast type to queue type
				mqDest = (javax.jms.Queue) mqContext.lookup(queueName);

			} catch (Exception mqContextException) {
				logger.debug(queueName
						+ " queue does not exist in queue manager"
						+ " -> create new one");

				// Create MQ queue
				try {
					logger.debug("JmsAdmin#createMQDestination builder#build("
							+ Constants.MQ_QUEUE_CLASSNAME + ","
							+ propertiesTable + ")");

					mqDest = (Queue) builder.build(
							Constants.MQ_QUEUE_CLASSNAME, propertiesTable);
				} catch (Exception e) {
					throw ExceptionUtils.newResourceException(
							"Can not create JMS queue", e);
				}

				// To avoid the case that another JMS object with the same name
				// exist in MQ JNDI Context but we need to create a queue
				// and because that rebind function does not
				// work with MQ JNDI Context
				try {
					mqContext.unbind(queueName);
				} catch (Exception e) {

				}

				// Create physic destination by binding in MQ JNDI Context
				try {
					mqContext.bind(queueName, mqDest);
				} catch (Exception e) {
					// TODO To log or to throw this exception???

					// throw ExceptionUtils.newResourceException
					logger.error("Can not create physical queue ->"
							+ "should use JmsAdmin or runmqsc to do it", e);
				}

			}
		}
		return mqDest;
	}

	/**
	 * Create QueueManager JNDI Context
	 * 
	 * @throws ResourceAdapterInternalException
	 */
	private InitialContext createMqJndiContext() throws ResourceException {
		logger.debug("JmsAdmin#createJndiContext()");

		InitialContext mqContext = null;

		try {
			String transportType = connector.getTransportType();

			String authenticationMode = connector.getAuthenticationMode();

			if (transportType.equals(Constants.CLIENT_MODE)) {
				url = connector.getHostName() + ":"
						+ connector.getPortAsString() + "/"
						+ connector.getChannel();
			} else if (transportType.equals(Constants.BINDING_MODE)) {
				url = connector.getQueueManager();
			} else
				throw new ResourceAdapterInternalException(
						"Can not determine this QueueManager connection mode: "
								+ connector.getTransportType());

			// Properties of QueueManager JNDI context
			java.util.Hashtable mqProps = new Hashtable();
			mqProps.put(Context.INITIAL_CONTEXT_FACTORY,
					Constants.MQ_CONTEXT_FACTORY);
			mqProps.put(Context.PROVIDER_URL, url);
			mqProps.put(Context.SECURITY_AUTHENTICATION, authenticationMode);

			if (!authenticationMode.equals(Constants.NONE_AUTHENTICATION_MODE)) {
				mqProps.put(Context.SECURITY_PRINCIPAL, connector
						.getJndiUserName());
				mqProps.put(Context.SECURITY_CREDENTIALS, connector
						.getJndiPassword());
			}

			// create context for MQ JNDI Service
			mqContext = new javax.naming.InitialContext(mqProps);
		} catch (Exception e) {
			throw ExceptionUtils
					.newResourceException(
							"Can not create QueueManager JNDI context"
									+ " -> Should change url or use JmsAdmin to create Queues",
							e);
		}
		return mqContext;
	}

	private void updateConnectionFactory(String propertyName,
			String propertyValue) throws Exception {

		JavaBeanExecutor.setProperty(cf, propertyName, propertyValue);
		jonasContext.rebind(jCfName, this.cf);
		JavaBeanExecutor.setProperty(tcf, propertyName, propertyValue);
		jonasContext.rebind(jTcfName, this.tcf);
		JavaBeanExecutor.setProperty(qcf, propertyName, propertyValue);
		jonasContext.rebind(jQcfName, this.qcf);

		// As mandatory property is changed, we need to delete all existing
		// destination
		deleteAllJmsDestination();

		// restart administering Queue Manager
		restartAdminQm();
	}

	private void deleteAllJmsDestination() {
		logger.debug("JmsAdmin#deleteAllJmsDestination()");

		String jndiName = null;
		// Delete all jms destination (not physic destination)
		for (Enumeration e = adminDestinations.keys(); e.hasMoreElements();) {
			try {
				jndiName = e.nextElement().toString();

				deleteDestination(jndiName, false);

			} catch (Exception re) {
				logger.error("Can not remove one of the destinations", re);
			}
		}
	}

	/***************************************************************************
	 * MANAGEMENT INTERFACE IMPLEMENTATION
	 **************************************************************************/

	public void createDestination(boolean isTopic, String jndiName,
			String properties) throws ResourceException {
		logger.debug("JmsAdmin#createDestination(" + isTopic + "," + jndiName
				+ "," + properties + ")");

		createDestination(isTopic, jndiName, JavaBeanExecutor
				.parseToProperties(properties));
	}

	public void deleteDestination(String jndiName, boolean deletePhysicDest)
			throws ResourceException {
		logger.debug("JmsAdmin#deleteDestination(" + jndiName + ","
				+ deletePhysicDest + ")");

		if (jndiName == null)
			throw new ResourceException("JNDI Name is null");

		AdminDestination dest = null;

		// Unregister admin destination from MBean Server
		try {
			jmxServer.unregisterDestination(jndiName);
		} catch (Exception e) {
			logger.error("Can not register destination " + jndiName
					+ " as MBean", e);
		}

		// Get admin destination from Hashtable
		try {
			dest = (AdminDestination) adminDestinations.get(jndiName);
		} catch (Exception e) {
			logger.error(jndiName + " destination does not exist in table", e);
		}

		// If we need to delete physic queue
		if ((deletePhysicDest) && (!dest.getIsTopic())) {
			try {
				String mqQueueName = dest
						.getProperty(Constants.QUEUE_NAME_PROPERTY);

				logger.info("JmsAdmin#deleteDestination "
						+ "delete physic queue: " + mqQueueName);
				createMqJndiContext().unbind(mqQueueName);
			} catch (Exception e) {
				logger.error("Can not delete physic queue of " + jndiName, e);
			}
		}

		// Remove admin destination from table
		adminDestinations.remove(jndiName);

		// Remove jms destination from jonas context
		try {
			jonasContext.unbind(jndiName);
		} catch (Exception e) {
			logger.error("Can not unbind " + jndiName + " from JOnAS context",
					e);
		}

	}

	public String getTransportType() {
		logger.debug("JmsAdmin#getTransportType()");

		return connector.getTransportType();
	}

	public void setTransportType(String transportType) throws Exception {
		logger.debug("JmsAdmin#setTransportType(" + transportType + ")");

		connector.setTransportType(transportType);
		if (transportType.equals(Constants.CLIENT_MODE))
			updateConnectionFactory("TransportType", Constants.MQ_CLIENT_MODE);
		else if (transportType.equals(Constants.BINDING_MODE))
			updateConnectionFactory("TransportType", Constants.MQ_BINDING_MODE);
		else
			throw new Exception(
					"Unknown Transport Type for QueueManager Connection");
	}

	public String getHostName() {
		logger.debug("JmsAdmin#getHostName()");

		return connector.getHostName();
	}

	public void setHostName(String hostName) throws Exception {
		logger.debug("JmsAdmin#setHostName(" + hostName + ")");

		connector.setHostName(hostName);
		updateConnectionFactory("HostName", hostName);
	}

	public String getPort() {
		logger.debug("JmsAdmin#getPort()");

		return connector.getPortAsString();
	}

	public void setPort(String port) throws Exception {
		logger.debug("JmsAdmin#setPort(" + port + ")");

		connector.setPort(port);
		updateConnectionFactory("Port", port);
	}

	public String getChannel() {
		logger.debug("JmsAdmin#getChannel()");

		return connector.getChannel();
	}

	public void setChannel(String channel) throws Exception {
		logger.debug("JmsAdmin#setChannel(" + channel + ")");

		connector.setChannel(channel);
		updateConnectionFactory("Channel", channel);
	}

	public String getQueueManager() {
		logger.debug("JmsAdmin#getQueueManager()");

		return connector.getQueueManager();
	}

	public void setQueueManager(String queueManager) throws Exception {
		logger.debug("JmsAdmin#setQueueManager(" + queueManager + ")");

		connector.setQueueManager(queueManager);
		updateConnectionFactory("QueueManager", queueManager);
	}

	public String getCfProperty(String propertyName) throws Exception {
		logger.debug("JmsAdmin#getCfProperty(" + propertyName + ")");

		return JavaBeanExecutor.getProperty(this.cf, propertyName).toString();
	}

	public void setCfProperty(String propertyName, String propertyValue)
			throws Exception {
		logger.debug("JmsAdmin#setCfProperty(" + propertyName + ","
				+ propertyValue + ")");

		cfProperties.put(propertyName, propertyValue);
		JavaBeanExecutor.setProperty(cf, propertyName, propertyValue);
		jonasContext.rebind(jCfName, this.cf);
	}

	public String getQcfProperty(String propertyName) throws Exception {
		logger.debug("JmsAdmin#getQcfProperty(" + propertyName + ")");

		return JavaBeanExecutor.getProperty(this.qcf, propertyName).toString();
	}

	public void setQcfProperty(String propertyName, String propertyValue)
			throws Exception {
		logger.debug("JmsAdmin#setQcfProperty(" + propertyName + ","
				+ propertyValue + ")");
		qcfProperties.put(propertyName, propertyValue);
		JavaBeanExecutor.setProperty(qcf, propertyName, propertyValue);

		jonasContext.rebind(jQcfName, this.qcf);
	}

	public String getTcfProperty(String propertyName) throws Exception {
		logger.debug("JmsAdmin#getTcfProperty(" + propertyName + ")");

		return JavaBeanExecutor.getProperty(this.tcf, propertyName).toString();
	}

	public void setTcfProperty(String propertyName, String propertyValue)
			throws Exception {
		logger.debug("JmsAdmin#setTcfProperty(" + propertyName + ","
				+ propertyValue + ")");

		tcfProperties.put(propertyName, propertyValue);
		JavaBeanExecutor.setProperty(tcf, propertyName, propertyValue);
		jonasContext.rebind(jTcfName, this.tcf);
	}

	public void setPerformanceEventEnabled(boolean enable) throws Exception {
		adminQm.enablePerformanceEvent(enable);
	}

	public boolean getPerformanceEventEnabled() throws Exception {
		return adminQm.isPerformanceEventEnabled();
	}

	public void saveDestinationConfig() throws Exception {
		logger.debug("JmsAdmin#saveDestinationConfig()");

		conf.saveConfig();
	}

	public String[] listDestinationObjectNames() {
		Object[] objs = adminDestinations.keySet().toArray();
		String[] result = new String[objs.length];
		for (int i = 0; i < objs.length; i++) {
			result[i] = this.jmxServer.buildDestinationObjectName(objs[i]
					.toString());
		}
		return result;
	}

	public String[] listConsumerObjectNames() {

		String[] result = new String[consumers.size()];

		InboundConsumer consumer;

		Integer key;

		int count = 0;

		for (Enumeration e = consumers.keys(); e.hasMoreElements();) {

			key = (Integer) e.nextElement();
			consumer = (InboundConsumer) consumers.get(key);

			result[count] = jmxServer.buildConsumerObjectName((consumer
					.getSpec()).getMdbName(), (consumer.getSpec())
					.getDestination(), key);

			logger.info("JmsAdmin#listConsumerObjectNames result: "
					+ result[count]);

			count++;
		}

		return result;

	}

	public void loadDestinationConfig() throws Exception {
		logger.debug("JmsAdmin#loadDestinationConfig()");

		conf.loadConfig();
	}

	public void restartAdminQm() {
		logger.debug("JmsAdmin#restartAdminQm()");
		// Stop administering Queue Manager
		try {
			adminQm.stop();
		} catch (Exception e) {
			logger.warn("Exception while stopping Queue Manager administering");
		}

		// Start adminstering Queue Manager
		try {
			adminQm.start(connector.getTransportType(),
					connector.getHostName(), connector.getChannel(), connector
							.getPortAsString(), connector.getQueueManager());

		} catch (Exception e) {
			logger.error("Can not start administering this Queue Manager", e);
		}
	}

	public String[] listPropertyNames(String jmsObject) {
		logger.debug("listPropertyNames()");
		String[] propertyNames = null;
		try {
			if ("Queue".equalsIgnoreCase(jmsObject)) {
				propertyNames = builder.getPropertyNames(Constants.MQ_QUEUE_CLASSNAME);
			} else if ("Topic".equalsIgnoreCase(jmsObject)) {
				propertyNames = builder.getPropertyNames(Constants.MQ_TOPIC_CLASSNAME);
			} else if ("CF".equalsIgnoreCase(jmsObject)) {
				propertyNames = builder.getPropertyNames(Constants.MQ_XA_CF_CLASSNAME);
			} else if ("QCF".equalsIgnoreCase(jmsObject)) {
				propertyNames = builder.getPropertyNames(Constants.MQ_XA_QCF_CLASSNAME);
			} else if ("TCF".equalsIgnoreCase(jmsObject)) {
				propertyNames = builder.getPropertyNames(Constants.MQ_XA_TCF_CLASSNAME);
			} else if ("NONXACF".equalsIgnoreCase(jmsObject)) {
				propertyNames = builder.getPropertyNames(Constants.MQ_CF_CLASSNAME);
			} else if ("NONXAQCF".equalsIgnoreCase(jmsObject)) {
				propertyNames = builder.getPropertyNames(Constants.MQ_QCF_CLASSNAME);
			} else if ("NONXATCF".equalsIgnoreCase(jmsObject)) {
				propertyNames = builder.getPropertyNames(Constants.MQ_TCF_CLASSNAME);
			} else {
				// jmsObject is assumed to be the fully qualified name of a
				// class
				propertyNames = builder.getPropertyNames(jmsObject);
			}
		} catch (Exception e) {
			logger.error("Cannot list property names of " + jmsObject, e);
		}
		return propertyNames;
	}
}