/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ActivationSpecImpl.java 10253 2007-04-23 11:27:55Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.inbound;

import javax.jms.Session;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.InvalidPropertyException;

import com.bull.jonasmqconnector.ConnectorProperties;
import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * An implementation of ActivationSpec interface as describled in JMS 1.1
 * specification.
 * 
 * An instance of this class holds the activation configuration information for
 * an MDB deployment.
 * 
 * @author Kien THAI
 * 
 */
public class ActivationSpecImpl extends ConnectorProperties implements
		ActivationSpec {
	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * JOnAS JNDI name of the destination for the MDB.
	 */
	private String destination;

	/**
	 * Type of the destination (queue or topic) for the MDB.
	 */
	private String destinationType;

	/**
	 * Properties of the destination for the MDB.
	 */
	private String destinationProperties;

	/**
	 * Message selector.
	 */
	private String messageSelector = null;

	/**
	 * Subscription durability.
	 */
	private String subscriptionDurability = null;

	/**
	 * Durable subscription name.
	 */
	private String subscriptionName = null;

	/**
	 * Acknowledgement mode.
	 */
	private String acknowledgeMode = null;

	/**
	 * Maximum messages number to be sent by a session.
	 */
	private int maxMessages = Constants.DEFAULT_MAX_MESSAGES;

	/**
	 * Max number of server sessions in a pool.
	 */
	private int poolSize = Constants.DEFAULT_MIN_POOL_SIZE;

	/**
	 * Initial number of server session in a pool.
	 */
	private int initPoolSize = 0;

	/**
	 * Max number of works submitted simultaneously.
	 */
	private int maxNumberOfWorks;

	/**
	 * MDB Name
	 */
	private String mdbName;

	
	private String userName = null;
	private String password = null;


	/**
	 * If this consumer need to reconnect after
	 * failure of physical connection to MQ server.
	 */
	private boolean reconnect = true;
	
	/**
	 * Interval between 2 reconnecting attempts
	 */
	private long reconnectInterval = 10000;
	
	/**
	 * Max number of retry
	 */
	private int maxReconnect = 6;
	
	public int getPoolSize() {
		logger.debug("ActivationSpecImpl#getPoolSize()");
		return poolSize;
	}

	public void setPoolSize(int poolSize) {
		logger.debug("ActivationSpecImpl#setPoolSize(" + poolSize + ")");
		this.poolSize = poolSize;
	}

	/**
	 * Checks MDB configuration information.
	 * 
	 * @exception InvalidPropertyException
	 *                Information missing or not correct
	 */
	public void validate() throws InvalidPropertyException {
		logger.debug("ActivationSpecImpl#validate()");
		if (destinationType != null
				&& !destinationType.equals(Constants.QUEUE_CLASSNAME)
				&& !destinationType.equals(Constants.TOPIC_CLASSNAME))
			throw new InvalidPropertyException(
					"Unidentified destination type: " + destinationType);

		if (destination == null)
			throw new InvalidPropertyException(
					"Missing JOnAS JNDI destination name.");

		if (acknowledgeMode != null
				&& !acknowledgeMode.equals(Constants.AUTO_ACKNOWLEDGE)
				&& !acknowledgeMode.equals(Constants.DUPS_OK_ACKNOWLEDGE))
			throw new InvalidPropertyException("Invalid acknowledge mode: "
					+ acknowledgeMode);

		if (subscriptionDurability != null) {

			if (subscriptionDurability.equals(Constants.DURABLE)
					&& destinationType.equals(Constants.QUEUE_CLASSNAME))
				throw new InvalidPropertyException(
						"Can't set a durable subscription " + "on a queue.");

			if (subscriptionDurability.equals(Constants.DURABLE)
					&& (subscriptionName == null))
				throw new InvalidPropertyException(
						"Missing durable subscription name.");
		}
		
		if (this.poolSize < 1) 
			throw new InvalidPropertyException
			("Can not initialize inbound consumer with pool size < 1");
			
		
	}

	public int getAcknowledgeMode() throws InvalidPropertyException {
		logger.debug("ActivationSpecImpl#getAcknowledgeMode()");
		if (acknowledgeMode == null) {
			return Session.AUTO_ACKNOWLEDGE;
		} else {
			if (acknowledgeMode.equals(Constants.AUTO_ACKNOWLEDGE)) {
				return Session.AUTO_ACKNOWLEDGE;
			} else if (acknowledgeMode.equals(Constants.DUPS_OK_ACKNOWLEDGE)) {
				return Session.DUPS_OK_ACKNOWLEDGE;
			} else {
				throw new InvalidPropertyException("Can not identify acknowledge mode");
			}
		}
	}

	public void setAcknowledgeMode(String acknowledgeMode) {
		logger.debug("ActivationSpecImpl#setAcknowledgeMode(" + acknowledgeMode
				+ ")");
		this.acknowledgeMode = acknowledgeMode;
	}

	public String getDestinationProperties() {
		logger.debug("ActivationSpecImpl#getDestinationProperties()");
		return destinationProperties;
	}

	public void setDestinationProperties(String destinationProperties) {
		logger.debug("ActivationSpecImpl#setDestinationProperties("
				+ destinationProperties + ")");
		this.destinationProperties = destinationProperties;
	}

	public String getDestinationType() {
		logger.debug("ActivationSpecImpl#getDestinationType()");
		return destinationType;
	}

	public void setDestinationType(String destinationType) {
		logger.debug("ActivationSpecImpl#setDestinationType(" + destinationType
				+ ")");
		this.destinationType = destinationType;
	}

	public String getMessageSelector() {
		logger.debug("ActivationSpecImpl#getMessageSelector()");
		return messageSelector;
	}

	public void setMessageSelector(String messageSelector) {
		logger.debug("ActivationSpecImpl#setMessageSelector(" + messageSelector
				+ ")");
		this.messageSelector = messageSelector;
	}

	public String getSubscriptionDurability() {
		logger.debug("ActivationSpecImpl#getSubscriptionDurability()");
		return subscriptionDurability;
	}

	public void setSubscriptionDurability(String subscriptionDurability) {
		logger.debug("ActivationSpecImpl#setSubscriptionDurability("
				+ subscriptionDurability + ")");
		this.subscriptionDurability = subscriptionDurability;
	}

	public String getSubscriptionName() {
		logger.debug("ActivationSpecImpl#getSubscriptionName()");
		return subscriptionName;
	}

	public void setSubscriptionName(String subscriptionName) {
		logger.debug("ActivationSpecImpl#setSubscriptionName("
				+ subscriptionName + ")");
		this.subscriptionName = subscriptionName;
	}

	public String getDestination() {
		logger.debug("ActivationSpecImpl#getDestination()");
		return destination;
	}

	public void setDestination(String destination) {
		logger.debug("ActivationSpecImpl#setDestination(" + destination + ")");
		this.destination = destination;
	}

	public int getMaxMessages() {
		logger.debug("ActivationSpecImpl#getMaxMessages()");
		return maxMessages;
	}

	public void setMaxMessages(int maxMessages) {
		logger.debug("ActivationSpecImpl#setMaxMessages(" + maxMessages + ")");
		this.maxMessages = maxMessages;
	}

	public int getMaxNumberOfWorks() {
		logger.debug("ActivationSpecImpl#getMaxNumberOfWorks()");
		return maxNumberOfWorks;
	}

	public void setMaxNumberOfWorks(int maxNumberOfWorks) {
		logger.debug("ActivationSpecImpl#setMaxNumberOfWorks("
				+ maxNumberOfWorks + ")");
		this.maxNumberOfWorks = maxNumberOfWorks;
	}

	public int getInitPoolSize() {
		logger.debug("ActivationSpecImpl#getMinPoolSize()");
		return initPoolSize;
	}

	public void setInitPoolSize(int minPoolSize) {
		logger.debug("ActivationSpecImpl#setMinPoolSize(" + minPoolSize + ")");
		this.initPoolSize = minPoolSize;
	}

	public String getMdbName() {
		return mdbName;
	}

	public void setMdbName(String mdbName) {
		this.mdbName = mdbName;
	}


	public String getPassword() {
		logger.debug("ActivationSpecImpl#getPassword()");
		return password;
	}

	public void setPassword(String password) {
		logger.debug("ActivationSpecImpl#setPassword(" + password + ")");
		this.password = password;
	}

	public String getUserName() {
		logger.debug("ActivationSpecImpl#getUserName()");
		return userName;
	}

	public void setUserName(String userName) {
		logger.debug("ActivationSpecImpl#setUserName(" + userName + ")");
		this.userName = userName;
	}

	
	public String toString() {
		String result = "ActivationSpecImpl#" + mdbName;
		return result;
	}

	public int getMaxReconnect() {
		return maxReconnect;
	}

	public void setMaxReconnect(int maxReconnect) {
		this.maxReconnect = maxReconnect;
	}

	public long getReconnectInterval() {
		return reconnectInterval;
	}

	public void setReconnectInterval(long reconnectInterval) {
		this.reconnectInterval = reconnectInterval;
	}

	public boolean isReconnect() {
		return reconnect;
	}

	public void setReconnect(boolean reconnect) {
		this.reconnect = reconnect;
	}


}