/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: InboundConsumer.java 10257 2007-04-23 12:06:10Z coqp $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.inbound;

import java.lang.reflect.Method;

import javax.jms.Connection;
import javax.jms.ConnectionConsumer;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Session;
import javax.jms.XAConnectionFactory;
import javax.management.MBeanException;
import javax.resource.ResourceException;
import javax.resource.spi.InvalidPropertyException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.resource.spi.work.WorkManager;

import com.bull.jonasmqconnector.JonasMQConnector;
import com.bull.jonasmqconnector.admin.JmsAdmin;
import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.JavaBeanExecutor;
import com.bull.jonasmqconnector.util.JmsUtils;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * An instance of this class acts as a point to start and stop the message
 * delivery.
 *
 * It holds two objects MessageEndpointFactory and ActivationSpec which
 * represent an MDB deployment.
 *
 * Each InboundConsumer instance corresponds to a ServerSessionPool instance.
 *
 *
 * @author Kien THAI
 *
 *
 */
public class InboundConsumer implements InboundConsumerMBean {

	private JonasMQConnector connector;

	private JmsAdmin admin;

	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	private boolean stopped = true;

	/**
	 * Endpoint Factory for the deployed MDB
	 */
	private MessageEndpointFactory endpointFactory;

	/**
	 * Activation spec of the deployed MDB
	 */
	private ActivationSpecImpl specImpl;

	/**
	 * JMS Destination for the deployed MDB
	 */
	private Destination dest = null;

	/**
	 * Number of messages that the MDB has received
	 */
	private int numberOfMessages = 0;

	/**
	 * ServerSession pool
	 */
	private ServerSessionPoolImpl pool;

	/**
	 * The need of XA support
	 */
	boolean transacted = true;

	/**
	 * OnMessage method of class javax.msg.MessageListener
	 */
	Method onMsgMethod = null;

	/**
	 * JMS Connection for this consumer
	 */
	private Connection con = null;

	private ConnectionFactory cf = null;

	private XAConnectionFactory xacf = null;

	private ConnectionConsumer conConsumer = null;

	/**
	 * Constructor used for transaction recovery
	 *
	 * @param Activation
	 *            spec instance
	 * @throws ResourceException
	 */
	public InboundConsumer(
			JonasMQConnector con,
			javax.resource.spi.ActivationSpec spec)
			throws ResourceException {
		this(con, null, spec);
		logger.debug("InboundConsumer#MessageConsumer(" + spec + ")");
	}

	/**
	 * This constructor validate the provided activation spec,
	 *
	 * @param mef
	 *            MessageEndpointFactory for the deployed MDB
	 * @param spec
	 *            Activation spec of the deployed MDB
	 * @throws ResourceException
	 *             if the activation spec can not be validated or the
	 *             destination can not be retrieved
	 */
	public InboundConsumer(
			JonasMQConnector con,
			MessageEndpointFactory mef,
			javax.resource.spi.ActivationSpec spec) throws ResourceException {
		logger.debug("InboundConsumer#MessageConsumer(" + mef + "," + spec
				+ ")");

		this.connector = con;

		this.endpointFactory = mef;

		// Check to see if the activation spec provided by the MDB
		// is an instance of JonasMQConnection Activation Spec
		try {
			specImpl = (ActivationSpecImpl) spec;
		} catch (ClassCastException cce) {
			throw ExceptionUtils.newResourceException(
					"Spec provided is not an instance of"
							+ " JonasMQConnection Activation Spec", cce);
		}

		// Check to see if the activation spec provided is valid
		try {
			specImpl.validate();
		} catch (InvalidPropertyException ipe) {
			throw ExceptionUtils.newResourceException(
					"Spec provided is not valid", ipe);
		}

		// Check to see if message delivery need to be transacted
		try {
			Class listenerClass = Class
					.forName(Constants.MSG_LISTENER_CLASSNAME);
			Class[] parameters = { Class.forName(Constants.MESSAGE_CLASSNAME) };
			onMsgMethod = listenerClass.getMethod(Constants.ONMESSAGE_METHOD,
					parameters);
			if (endpointFactory != null)
				transacted = endpointFactory.isDeliveryTransacted(onMsgMethod);
			else
				transacted = true;
		} catch (Exception e) {
			throw ExceptionUtils.newResourceException(
					"Can not determine transactional context", e);
		}

	}

	/**
	 * Based on the activation spec of the deployed MDB, config the destination
	 * for the MDB, initialize the server session pool, create an appropriate
	 * connection for the message delivery, create a ConnectionConsumer instance
	 * and start the connection for message delivery
	 *
	 * @throws ResourceException
	 *             if server session pool can not be initiated, or
	 *             ConnectionConsumer instance can not be created, or Connection
	 *             can not be started
	 */
	public void start() throws ResourceException {

		logger.debug("InboundConsumer#start()");

		this.admin = connector.getJmsAdmin();

		// Get (or create) and config the destination for the MDB
		try {
			dest = admin.createDestination(
					isTopic(),
					specImpl.getDestination(),
					JavaBeanExecutor.parseToProperties
					(specImpl.getDestinationProperties())
					);
		} catch (Exception cde) {
			throw new ResourceException(cde.getMessage());
		}

		String cfType = isXA() ? (isTopic() ? Constants.MQ_XA_TCF_CLASSNAME
				: Constants.MQ_XA_QCF_CLASSNAME)
				: (isTopic() ? Constants.MQ_TCF_CLASSNAME
						: Constants.MQ_QCF_CLASSNAME);

		// Create the appropriate ConnectionFactory
		try {
			if (isXA())
				this.xacf = (XAConnectionFactory) admin
						.createConnectionFactory(cfType);
			else
				this.cf = (ConnectionFactory) admin
						.createConnectionFactory(cfType);
		} catch (Exception e) {
			throw ExceptionUtils.newResourceException(
					"Can not create connection factory for consumer", e);
		}

		// Create the Connection
		String user = this.specImpl.getUserName();
		String password = this.specImpl.getPassword();

		try {
			if (isXA())
				con = JmsUtils.createXAConnection(xacf, isTopic(), user,
						password);
			else
				con = JmsUtils.createConnection(cf, isTopic(), user, password);
		} catch (JMSException jmse) {
			throw ExceptionUtils.newResourceException(
					"Can not create connection", jmse);
		}

		// Config the server session pool
		try {
			pool = new ServerSessionPoolImpl(this);
			pool.init();
		} catch (JMSException jmse) {
			throw ExceptionUtils.newResourceException(
					"Can not initialize session pool", jmse);
		}

		// Create the ConnectionConsumer
		try {
			if (specImpl.getSubscriptionDurability().equals(Constants.DURABLE)) {
				logger.debug("MessageConsumer#start() : " + con
						+ "#createDurableConnectionConsumer(" + dest + ","
						+ specImpl.getSubscriptionName() + ","
						+ specImpl.getMessageSelector() + "," + this.pool + ","
						+ specImpl.getMaxMessages() + ")");

				if (specImpl.getSubscriptionName() != null)

					conConsumer = con.createDurableConnectionConsumer(
							(javax.jms.Topic) dest,
							specImpl.getSubscriptionName(),
							specImpl.getMessageSelector(),
							this.pool,
							specImpl.getMaxMessages());
				else
					throw new ResourceException(
							"Subscription name is not provided for a durable consumer");
			} else {
				conConsumer = JmsUtils.createConnectionConsumer(con, isTopic(),
						dest,
						specImpl.getMessageSelector(),
						this.pool,
						specImpl.getMaxMessages());
			}
		} catch (JMSException jmse) {

			throw ExceptionUtils.newResourceException(
					"Can not create ConnectionConsumer", jmse);
		}


		// TODO Test
		// Set ExceptionListener in order to catch run time connection exception
		if (this.specImpl.isReconnect()) {
			try {
				con.setExceptionListener(new ExceptionListenerImpl(this));
			} catch (JMSException jmse) {
				logger.warn("MessageConsumer#start() : "
						+ "Can not set exception listener for the connection",
						jmse);
			}
		}


		try {
			con.start();
		} catch (JMSException jmse) {
			throw ExceptionUtils.newResourceException(
					"Can not start the connection", jmse);
		}

		logger.info("InboundConsumer#start() : start connection " + con);
		stopped = false;
	}

	/**
	 * Stop the message delivery. Close all sessions, connection consumer and
	 * connection
	 *
	 * @throws ResourceException
	 *             if the sessions or the connection consumer or the connection
	 *             can not be closed
	 */
	public void stop() throws ResourceException {
		logger.debug("InboundConsumer#stop()");

		// Close all sessions in pool
		if (pool != null) {
			try {
				pool.close();
				pool = null;
				logger.info("Pool closed");
			} catch (JMSException jmse) {

				// TODO To throw exception or to log ???
				// throw ExceptionUtils.newResourceException
				logger.warn
				("InboundConsumer#stop() : "
						+ "Can not close ServerSession pool",jmse);
			}
		}

		// Close the connection consumer
		if (conConsumer != null) {
			try {
				conConsumer.close();
				conConsumer = null;
				logger.info("Connection consumer closed");
			} catch (JMSException jmse) {
				logger.error("InboundConsumer#stop() : "
						+ "Can not close connection consumer",jmse);
			}
		}

		// TODO To verify if we really need this
		// Unsubscribe the topic
		try {
			if (specImpl.getSubscriptionName() != null) {
				Session session = con.createSession
				(true, Session.AUTO_ACKNOWLEDGE);
				session.unsubscribe(specImpl.getSubscriptionName());
				logger.info(specImpl.getSubscriptionName()+
						" unsubscribed");
			}
		} catch(Exception e) {
			logger.error("InboundConsumer#stop() : Can not unsubscribe " +
					specImpl.getSubscriptionName(), e);
		}

		// Close the connection
		if (con != null) {
			try {
				con.close();
				con = null;
				logger.info("Connection closed");
			} catch (JMSException jmse) {
				logger.error("InboundConsumer#stop() : "
						+ "Can not close connection", jmse);
			}
		}

		stopped = true;
	}

	// TODO Check this: this method is for the purpose to reconnect
	// to MQ Server in case physic connection is broken
	public void reconnect() throws ResourceException {
		logger.info("InboundConsumer#reconnect()");

		Connection recon;

		String cfType = isXA() ? (isTopic() ? Constants.MQ_XA_TCF_CLASSNAME
				: Constants.MQ_XA_QCF_CLASSNAME)
				: (isTopic() ? Constants.MQ_TCF_CLASSNAME
						: Constants.MQ_QCF_CLASSNAME);

		// Create the appropriate ConnectionFactory
		try {
			if (isXA())
				this.xacf = (XAConnectionFactory) admin
						.createConnectionFactory(cfType);
			else
				this.cf = (ConnectionFactory) admin
						.createConnectionFactory(cfType);
		} catch (Exception e) {
			throw ExceptionUtils.newResourceException(
					"Can not create connection factory for consumer", e);
		}

		// Create the connection.
		String user = this.specImpl.getUserName();
		String password = this.specImpl.getPassword();

		try {
			if (isXA())
				recon = JmsUtils.createXAConnection(xacf, isTopic(), user,
						password);
			else
				recon = JmsUtils.createConnection(cf, isTopic(), user, password);
		} catch (JMSException jmse) {
			throw ExceptionUtils.newResourceException(
					"Can not create connection", jmse);
		}

		// Start the connection.
		try {
			recon.start();
		} catch (JMSException jmse) {
			throw ExceptionUtils.newResourceException(
					"Can not start the connection", jmse);
		}

		// Close the connection
		if (recon != null) {
			try {
				recon.close();
				recon = null;
				logger.info("Connection closed");
			} catch (JMSException jmse) {
				throw ExceptionUtils.newResourceException(
						"Can not stop the connection", jmse);
			}
		}

	}

	/***************************************************************************
	 * Getter Methods
	 **************************************************************************/

	public boolean isStop() {
		logger.debug("InboundConsumer#isStop()");
		return stopped;
	}

	public boolean isXA() {
		logger.debug("InboundConsumer#isXA()");
		return transacted;
	}

	boolean isTopic() {
		logger.debug("InboundConsumer#isTopic()");
		return specImpl.getDestinationType().equals(Constants.TOPIC_CLASSNAME);
	}

	public Connection getConnection() {
		logger.debug("InboundConsumer#getConnection()");
		return con;
	}

	public MessageEndpointFactory getEndpointFactory() {
		logger.debug("InboundConsumer#getEndpointFactory()");
		return endpointFactory;
	}

	public ActivationSpecImpl getSpec() {
		logger.debug("InboundConsumer#getSpec()");
		return specImpl;
	}

	public Method getOnMsgMethod() {
		logger.debug("InboundConsumer#getOnMsgMethod()");
		return onMsgMethod;
	}

	public synchronized void messageReceived() {
		logger.debug("InboundConsumer#messageReceived()");
		this.numberOfMessages++;
	}


	public WorkManager getWorkManager() {
		return connector.getWorkManager();
	}

	public String toString() {
		String result = "InboundConsumer#" + this.specImpl.getMdbName();
		return result;
	}


	/***************************************************************************
	 * Management Interface Implementation
	 **************************************************************************/

	public Object getProperty(String propertyName) throws MBeanException {
		logger.debug("InboundConsumer#getProperty(" + propertyName + ")");
		try {

			return JavaBeanExecutor.getProperty(this.specImpl, propertyName);
		} catch (Exception e) {
			throw new MBeanException(e, "Can not get property : "
					+ propertyName + " from this InboundConsumer : " + this);
		}
	}

	public synchronized int getNumberOfReceivedMessages() {
		logger.debug("InboundConsumer#getNumberOfReceivedMessages()");
		return numberOfMessages;
	}

	public int getNumberOfWorks() {

		return this.pool.getWorkCount();
	}

}