/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerSessionImpl.java 10253 2007-04-23 11:27:55Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.inbound;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ServerSession;
import javax.jms.Session;
import javax.jms.XASession;
import javax.resource.ResourceException;
import javax.resource.spi.UnavailableException;
import javax.resource.spi.endpoint.MessageEndpoint;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.resource.spi.work.Work;
import javax.resource.spi.work.WorkException;
import javax.resource.spi.work.WorkManager;
import javax.transaction.xa.XAResource;

import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * An implementation of ServerSession interface as describled in JMS 1.1
 * specification.
 * 
 * This class implement the interface javax.resource.spi.work.Work to assure
 * that each message delivery of this session will be carried out in an
 * independent thread.
 * 
 * @author Kien THAI
 * 
 */
public class ServerSessionImpl implements ServerSession, MessageListener, Work {

	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * JOnAS WorkManager instance
	 */
	private WorkManager workManager;

	/**
	 * ServerSessions Pool
	 */
	private ServerSessionPoolImpl ssPool;

	private MessageEndpointFactory endpointFactory;

	private MessageEndpoint endpoint = null;

	/**
	 * MQ XAResource
	 */
	private XAResource xar = null;

	/**
	 * JMS Session
	 */
	private Session session;

	private boolean isXA = false;

	/**
	 * Constructor
	 * 
	 * @param pool
	 *            The pool which holds this session
	 * @param sess
	 *            JMS session
	 * @param mef
	 *            Endpoint Factory
	 * @throws JMSException
	 *             if work manager can not be retrieved or
	 */
	public ServerSessionImpl(
			ServerSessionPoolImpl pool, 
			Session sess,
			MessageEndpointFactory mef,
			WorkManager wm) throws JMSException {
		logger.debug("ServerSessionImpl#ServerSessionImpl(" + pool + "," + sess
				+ "," + mef + ")");
		
		this.workManager = wm;
		
		this.ssPool = pool;
		this.session = sess;
		this.endpointFactory = mef;
		if (ssPool.getConsumer().isXA()) {
			isXA = true;
			xar = ((XASession) this.session).getXAResource();
		}
		try {
			this.session.setMessageListener(this);
		} catch (JMSException je) {
			throw ExceptionUtils
					.newJMSException(
							"ServerSessionImpl#ServerSessionImpl Can not set listener ",
							je);
		}
	}

	public Session getSession() {
		logger.debug("ServerSessionImpl#getSession()");
		return session;
	}

	/**
	 * Called by connection consumer to schedule a thread to start this session
	 * 
	 * @throws JMSException
	 *             if thread can not be scheduled
	 */
	public void start() throws JMSException {
		logger.debug("ServerSessionImpl#start()");
		try {
			workManager.scheduleWork(this);
		} catch (WorkException we) {
			throw new JMSException("Cannot schedule work: " + we.getMessage());
		}
	}

	public void stop() throws JMSException {
		logger.debug("ServerSessionImpl#stop()");
		if (session != null) {
			try {
				session.close();
			} catch (JMSException jmse) {
				logger.error("ServerSessionImpl#stop() : "
						+ "Can not stop server session: ", jmse);
				throw ExceptionUtils.newJMSException
				("Can not stop server session", jmse);
			}
		}
		if (endpoint != null) {
			releaseEndpoint();
		}
	}

	/**
	 * Called by JOnAS work manager to start the transaction and the JMS session
	 */
	public void run() {
		logger.debug("ServerSessionImpl#run()");

		try {
			if (isXA) {
				endpoint = this.endpointFactory.createEndpoint(xar);
			} else
				endpoint = this.endpointFactory.createEndpoint(null);
		} catch (UnavailableException ue) {
			logger.error("ServerSessionImpl#run() createEndpoint", ue);
		}

		if (endpoint != null) {
			try {
				if (isXA)
					endpoint.beforeDelivery(ssPool.getConsumer()
							.getOnMsgMethod());
				session.run();
			} catch (NoSuchMethodException nsme) {
				logger.warn("ServerSessionImpl#run() : " + nsme);
			} catch (ResourceException re) {
				logger.error("ServerSessionImpl#run() session.run()", re);
				
				// Why should I close session here ?
				// TODO To test this
//				try {
//					session.close();
//				} catch (JMSException jmse) {
//					logger.error("ServerSessionImpl#run() session.close() ", jmse);
//				}
				
			} finally {
				
				// TODO To explain why session have to be closed here
				// in order that non-XA message reception can work 
				// (if not, messages arrive very slowly) 
				// and why session have not to be closed here
				// in order that XA message reception can work
				// (if not, message arrive very slowly and others exceptions 
				// will be generated)
//				if (!isXA)
//					try {
//						logger.info
//						("non-XA ServerSessionImpl#run() session.close()");
//						session.close();
//					} catch (JMSException jmse) {
//						logger.error(
//								"ServerSessionImpl#run() session.close()",
//								jmse);
//					}
//					
				
				// Because of the pool, when I dont initialize the pool for
				// samplemdb_WSMQ example, this will open one session for
				// receiving one message and so when I have to receive 100
				// message, JOnAS can not provide 100 thread for 100 session
				// That why, 15 messages can not be received by the MDB.
				// SOLUTION: Firstly, pool size has to be 1 at minimum.
				// Secondly, when session can not return to pool, we have
				// to destroy it!
				
				
				releaseEndpoint();
			}
		}
	}

	/**
	 * finish the delivery transaction and release the endpoint
	 * 
	 */
	public void releaseEndpoint() {
		logger.debug("ServerSessionImpl#releaseEndpoint()");
		try {
			if (isXA)
				this.endpoint.afterDelivery();
		} catch (ResourceException re) {
			logger.error("ServerSessionImpl#run() : "
					+ "Can not finish message delivery", re);
		} finally {
			this.endpoint.release();
			this.endpoint = null;
			ssPool.releaseServerSession(this);
		}
	}

	/**
	 * feed the MDB with the message
	 * 
	 * @param message
	 *            Message to be consumed by the MDB
	 */
	public void onMessage(Message message) {
		logger.debug("ServerSessionImpl#onMessage()");
		try {
			((javax.jms.MessageListener) endpoint).onMessage(message);
			this.ssPool.getConsumer().messageReceived();
		} catch (Exception e) {
			logger.error("onMessage Exception in ServerSession", e);
		}
	}

	public void release() {
		// TODO To verify if we need this or not
		logger.info("ServerSessionImpl#release() session.close");
//		try {
//		      session.close();
//		    } catch (JMSException jmse) {
//		    	logger.error("ServerSessionImpl#release() session.close", jmse);
//		    }
	}
}