/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerSessionPoolImpl.java 10253 2007-04-23 11:27:55Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.inbound;

import java.util.ArrayList;
import java.util.Iterator;

import javax.jms.JMSException;
import javax.jms.ServerSession;
import javax.jms.ServerSessionPool;
import javax.jms.Session;
import javax.jms.XAConnection;

import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.JmsUtils;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * An implementation of ServerSessionPool interface as described in JMS 1.1
 * specification.
 * 
 * This class provides a pool of ServerSession objects used for concurrent
 * messages processing.
 * 
 * Each ServerSessionPool instance corresponds to a MessageConsumer instance.
 * 
 * @author Kien THAI
 * 
 */
public class ServerSessionPoolImpl implements ServerSessionPool {

	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * Message consumer associated with this pool
	 */
	private InboundConsumer consumer;

	/**
	 * Server Session list
	 */
	private ArrayList pool;

	/**
	 * Number of ServerSession in work
	 */
	private int workCount = 0;

	/**
	 * Max number of Server Session in pool
	 */
	private int poolSize = Constants.DEFAULT_MIN_POOL_SIZE;

	/**
	 * Initial number of Server Session in Pool
	 */
	private int initPoolSize = 0;

	/**
	 * Max number of ServerSession objects in work
	 */
	private int maxNumberOfWorks = 0;

	/**
	 * Constructor
	 * 
	 * @param consumer
	 */
	public ServerSessionPoolImpl(InboundConsumer consumer) {
		logger.debug("ServerSessionPoolImpl#ServerSessionPoolImpl(" + consumer
				+ ")");
		this.consumer = consumer;

		poolSize = consumer.getSpec().getPoolSize();
		initPoolSize = consumer.getSpec().getInitPoolSize();
		maxNumberOfWorks = consumer.getSpec().getMaxNumberOfWorks();
	}

	/**
	 * Initiate the pool, pre-allocate the minimum number of ServerSession
	 * objects
	 * 
	 * @throws JMSException
	 *             if ServerSession objects can not be created
	 */
	public void init() throws JMSException {
		logger.debug("ServerSessionPoolImpl#init()");
		ServerSessionImpl session;

		pool = new ArrayList();

		if (initPoolSize != 0) {
			logger.info("Pre-allocate a set of " + initPoolSize
					+ " server session instances");
			synchronized (pool) {
				for (int i = 0; i < initPoolSize; i++) {
					try {
						session = newServerSession();
						pool.add(session);
					} catch (JMSException e) {
						throw new JMSException(
								"Can not init server session pool: "
										+ e.getMessage());
					}
				}
			}
		}
	}

	/**
	 * Create a new ServerSession objects
	 * 
	 * @return ServerSessionImpl
	 * @throws JMSException
	 *             if JMS Session can not be created
	 */
	private ServerSessionImpl newServerSession() throws JMSException {
		logger.debug("ServerSessionPoolImpl#newServerSession()");
		Session session = null;
		ServerSessionImpl ss = null;

		try {
			if (consumer.isXA())
				session = JmsUtils.createXASession(
						(XAConnection) consumer
						.getConnection(), 
						consumer.isTopic());
			else
				session = JmsUtils.createSession(
						consumer.getConnection(),
						consumer.isTopic(),
						consumer.getSpec().getAcknowledgeMode());
		} catch (Exception jmse) {
			throw new JMSException("Can not create JMS Session: "
					+ jmse.getMessage());
		}

		ss = new ServerSessionImpl(
				this, 
				session, 
				consumer.getEndpointFactory(),
				this.consumer.getWorkManager()
		);

		return ss;
	}

	public void releaseServerSession(ServerSessionImpl ss) {
		logger.debug("ServerSessionPoolImpl#releaseServerSession(" + ss + ")");
		synchronized (pool) {
			if (!consumer.isStop()) {
				workCount--;
				logger.debug("Number of Session in work: " + workCount);
				if (pool.size() < poolSize) {
					pool.add(ss);
					logger.debug("Session return to Pool");
					pool.notifyAll();
				}
				
				else {
					try {
						ss.stop();
					} catch (JMSException jmse) {
						logger.warn("Can not stop ServerSession", jmse);
					}
				}
			}
			
			//TODO Check it
			else {
				try {
					ss.stop();
				} catch (JMSException jmse) {
					logger.warn("Can not stop ServerSession", jmse);
				}
			}
		}
	}

	/**
	 * Returns a new server session instance. Try to get one from the pool, and
	 * create a new one if the pool is busy and the number of works did not
	 * exceed limit.
	 * 
	 * If not, wait for the pool to be free.
	 * 
	 * @return a ServerSession instance.
	 */
	public ServerSession getServerSession() throws JMSException {
		logger.debug("ServerSessionPoolImpl#getServerSession()");
		ServerSession session;
		synchronized (pool) {
			if (consumer.isStop())
				return null;
			if (!pool.isEmpty()) {
				return getSessionFromPool();
			} else {

				// max number of works <= 0 mean that
				// no limit for the creation of server session
				if ((maxNumberOfWorks <= 0) || (workCount < maxNumberOfWorks)) {
					try {
						session = newServerSession();
						workCount++;
						logger.debug("Return new session");
						logger.debug("Number of Session in work: " + workCount);
						return session;
					} catch (JMSException jmse) {
						throw new JMSException(jmse.getMessage());
					}
				} else {
					while (pool.isEmpty()) {
						logger.info("Reach max works, Pool empty -> wait");
						try {
							pool.wait();
						} catch (InterruptedException ie) {

						} catch (IllegalArgumentException iae) {

						}
					}

					return getSessionFromPool();
				}
			}
		}
	}

	public ServerSession getSessionFromPool() throws JMSException {
		logger.debug("ServerSessionPoolImpl#getSessionFromPool()");
		ServerSession session;
		try {
			session = (ServerSession) pool.remove(0);
			workCount++;

			logger.debug("Number of Session in work: " + workCount);
			return session;
		} catch (IndexOutOfBoundsException iobe) {
			throw new JMSException("Can not get ServerSession from Pool: "
					+ iobe);
		}
	}

	/**
	 * Close all ServerSession
	 * 
	 * @throws JMSException
	 *             if session can not be closed
	 */
	public void close() throws JMSException {
		logger.debug("ServerSessionPoolImpl#close()");
		workCount = 0;
		ServerSessionImpl serverSession;
		Iterator it = pool.iterator();
		while (it.hasNext()) {
			try {
				serverSession = (ServerSessionImpl) it.next();
				serverSession.stop();
				logger.info("ServerSession stopped");
			} catch (JMSException jmse) {
				logger.error("Can not close server session", jmse);
			}
		}
	}

	public InboundConsumer getConsumer() {
		logger.debug("ServerSessionPoolImpl#getConsumer()");
		return consumer;
	}

	public int getWorkCount() {
		return workCount;
	}

}