/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbstractManagedConnectionFactory.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import java.io.PrintWriter;
import java.util.Hashtable;
import java.util.Set;

import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.XAConnectionFactory;
import javax.naming.Reference;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ResourceAdapterAssociation;
import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;

import com.bull.jonasmqconnector.ConnectorProperties;
import com.bull.jonasmqconnector.JonasMQConnector;
import com.bull.jonasmqconnector.admin.JmsAdmin;
import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;
import com.bull.jonasmqconnector.util.SecurityUtils;
import com.bull.jonasmqconnector.util.StringUtils;

/**
 * This class serve to abstract the functionnality of the JMS,Queue and Topic
 * Managed Connection Factory.
 * 
 * @author Yacine TOUIL
 */
public abstract class AbstractManagedConnectionFactory extends
		ConnectorProperties implements ManagedConnectionFactory,
		ResourceAdapterAssociation {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * Connection factory JNDI
	 */
	private String connectionFactoryJndiName;

	/**
	 * Connection factory client ID
	 */
	private String clientId = null;

	/**
	 * disabled by default
	 */
	private boolean connectionValidationEnabled = false;

	/**
	 * This is the log print writer.
	 */
	private PrintWriter pw;

	/**
	 * this is a the connecion factory
	 */
	private XAConnectionFactory connectionFactory = null;

	/**
	 * The destination mode
	 */
	protected int destinationMode = Constants.JMS_SESSION;

	/**
	 * by default, run as non Application Client Container. Use System Property
	 * or MCF property to enable
	 */
	private static boolean inApplicationClientContainer = false;

	
	/**
	 * A hashtable that holds connection factory properties
	 */
	private Hashtable properties = null;
	
	/**
	 * Use system property to determine Application Client Container status.
	 */
	static {
		String s = System.getProperty(Constants.INACC_SYSTEM_PROP_KEY);
		if (s != null) {
			inApplicationClientContainer = (new Boolean(s)).booleanValue();
		}
	}

	/**
	 * The default parent constructor
	 */
	public AbstractManagedConnectionFactory() {
		super();
		logger.debug("AbstractManagedConnectionFactory#"
				+ "AbstractManagedConnectionFactory()");
	}

	/**
	 * instantiate connection factory with RA's default simple ConnectionManager
	 * 
	 * @return the instanciated connection factory
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public Object createConnectionFactory() throws ResourceException {
		logger
				.debug("AbstractManagedConnectionFactory#createConnectionFactory("
						+ ")");
		ConnectionManager cm = new ConnectionManagerImpl();
		ConnectionFactoryImpl cf = new ConnectionFactoryImpl(this, cm);
		Reference ref = new Reference(cf.getClass().getName(),
				"com.bull.jonasmqconnector.outbound.ConnectionFactoryImpl",
				null);
		cf.setReference(ref);

		return cf;
	}

	/**
	 * instantiate connection factory with RA's default simple ConnectionManager
	 * 
	 * @param cm
	 *            the conneciton manager to use
	 * @return the instanciated connection factory
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public Object createConnectionFactory(ConnectionManager cm)
			throws ResourceException {
		logger
				.debug("AbstractManagedConnectionFactory#createConnectionFactory("
						+ cm + ")");
		ConnectionFactoryImpl cf = new ConnectionFactoryImpl(this, cm);
		Reference ref = new Reference(cf.getClass().getName(),
				"com.bull.jonasmqconnector.outbound.ConnectionFactoryImpl",
				null);
		cf.setReference(ref);

		return cf;
	}

	/**
	 * Create or lookup JMS connection factory if not already done Create a
	 * connection from the JMS CF and create a ManagedConnection from the
	 * created connection
	 * 
	 * @param subject
	 *            this is the subject
	 * @param connRequestInfo
	 * @return the instanciated managed connection
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public ManagedConnection createManagedConnection(Subject subject,
			ConnectionRequestInfo connRequestInfo) throws ResourceException {
		logger
				.debug("AbstractManagedConnectionFactory#createManagedConnection("
						+ subject + "," + connRequestInfo + ")");
		try {
			initializeConnectionFactory();
			PasswordCredential pc = SecurityUtils.getPasswordCredential(this,
					subject, connRequestInfo);
			javax.jms.Connection connection = createPhysicalConnection(pc);

			return new ManagedConnectionImpl(this, pc, connRequestInfo,
					connection);
		} catch (ResourceException e) {
			throw (ResourceException) new ResourceException(e.getMessage())
					.initCause(e);
		} catch (JMSException e) {
			throw (ResourceException) new ResourceException(e.getMessage())
					.initCause(e);
		}
	}

	/**
	 * Create a physical connection
	 * 
	 * @param pc
	 *            the Password Credential
	 * @return the physical created connection
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	private javax.jms.Connection createPhysicalConnection(PasswordCredential pc)
			throws JMSException {
		logger.debug("AbstractManagedConnectionFactory#"
				+ "createPhysicalConnection(" + pc + ")");
		try {
			return createXAConnection(pc, connectionFactory);
		} catch (JMSException jmse) {
			logger.error("AbstractManagedConnectionFactory#"
					+ "createPhysicalConnection return null");
			return null;
		}
	}

	/**
	 * Create a XA connection
	 * 
	 * @param pc
	 *            the Password Credential
	 * @param cf
	 *            the connection factory
	 * @return a new XA Connection
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	protected abstract javax.jms.XAConnection createXAConnection(
			PasswordCredential pc, XAConnectionFactory cf) throws JMSException;

	/**
	 * Create a connection
	 * 
	 * @param pc
	 *            the Password Credential
	 * @param cf
	 *            the connection factory
	 * @return a new JMS Connection
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	protected abstract javax.jms.Connection createConnection(
			PasswordCredential pc, ConnectionFactory cf) throws JMSException;

	/**
	 * Overridden by MCF for TCF, QCF and JMS CF to return their appropriate CF
	 * class names, so that the relevant JavaBean class can be used by the
	 * builder.
	 * 
	 * @return String ClassName of the MoM-specific ConnectionFactory to be
	 *         created by the MCF.
	 */
	protected abstract String getActualConnectionFactoryClassName();

	/**
	 * initialize the Connection Factory
	 * 
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	private void initializeConnectionFactory() throws ResourceException {
		logger.debug("AbstractManagedConnectionFactory#"
				+ "initializeConnectionFactory()");

		String classname = getActualConnectionFactoryClassName();
		try {
			JmsAdmin jmsAdmin = ((JonasMQConnector) getResourceAdapter())
			.getJmsAdmin();
			this.properties = jmsAdmin.getConnectionFactoryProperties(classname); 
			this.connectionFactory = (XAConnectionFactory)
			jmsAdmin.createConnectionFactory(classname);
		} catch (Exception e) {
			throw ExceptionUtils.newResourceException(
					"Can not create ConnectionFactory", e);
		}
	}

	/**
	 * Find a Managed connection that match with the parameters
	 * 
	 * @param connectionSet
	 *            the set of managed connection
	 * @param subject
	 *            this is the subject
	 * @param connRequestInfo
	 *            the infos about the managed connection
	 * @return the matched connection if found, otherwise null
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public ManagedConnection matchManagedConnections(Set connectionSet,
			Subject subject, ConnectionRequestInfo connRequestInfo)
			throws ResourceException {
		logger
				.debug("AbstractManagedConnectionFactory#matchManagedConnections("
						+ connectionSet
						+ ","
						+ subject
						+ ","
						+ connRequestInfo
						+ ")");
		if (connectionSet == null) {
			return null;
		}

		PasswordCredential pc = SecurityUtils.getPasswordCredential(this,
				subject, connRequestInfo);

		java.util.Iterator iter = connectionSet.iterator();
		ManagedConnectionImpl mc = null;
		while (iter.hasNext()) {
			try {
				mc = (ManagedConnectionImpl) iter.next();
				logger.debug("AbstractManagedConnectionFactory#"
						+ "matchManagedConnections : "
						+ "Matching managed connections ->" + mc);
			} catch (java.util.NoSuchElementException nsee) {
				throw (ResourceException) new ResourceException(nsee
						.getMessage()).initCause(nsee);
			}

			if (pc == null && this.equals(mc.getManagedConnectionFactory())) {
				if (!mc.isDestroyed())
					return mc;
			} else if (SecurityUtils.isPasswordCredentialEqual(pc, mc
					.getPasswordCredential()) == true) {
				if (!mc.isDestroyed())
					return mc;
			}
		}
		return null;
	}

	/**
	 * Gets the log witer
	 * 
	 * @return the log writer
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public PrintWriter getLogWriter() throws ResourceException {
		logger.debug("AbstractManagedConnectionFactory#getLogWriter()");
		return pw;
	}

	/**
	 * Sets the log writer
	 * 
	 * @param pw
	 *            the print writer
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public void setLogWriter(PrintWriter pw) throws ResourceException {
		logger.debug("AbstractManagedConnectionFactory#setLogWriter(" + pw
				+ ")");
		this.pw = pw;
	}

	/**
	 * Gets the client id
	 * 
	 * @return the client id
	 */
	public String getClientId() {
		logger.debug("AbstractManagedConnectionFactory#getClientId()");
		return this.clientId;
	}

	/**
	 * Sets the client id
	 * 
	 * @param clientId
	 *            the client id
	 */
	public void setClientId(String clientId) {
		logger.debug("AbstractManagedConnectionFactory#setClientId(" + clientId
				+ ")");
		this.clientId = clientId;
	}

	/**
	 * Gets the Connection Factory Jndi Name
	 * 
	 * @return Returns the connection Factory Jndi Name
	 */
	public String getConnectionFactoryJndiName() {
		logger.debug("AbstractManagedConnectionFactory#"
				+ "getConnectionFactoryJndiName()");
		return connectionFactoryJndiName;
	}

	/**
	 * Sets the Connection Factory Jndi Name
	 * 
	 * @param connectionFactoryJndiName
	 *            the Connection Factory Jndi Name to set.
	 */
	public void setConnectionFactoryJndiName(String connectionFactoryJndiName) {
		logger.debug("AbstractManagedConnectionFactory#"
				+ "setConnectionFactoryJndiName(" + connectionFactoryJndiName
				+ ")");
		this.connectionFactoryJndiName = connectionFactoryJndiName;
	}

	/**
	 * The App Client Container status
	 * 
	 * @return Returns the App Client Container status.
	 */
	public boolean isInApplicationClientContainer() {
		logger.debug("AbstractManagedConnectionFactory#isInAppClientContainer("
				+ ")");
		return inApplicationClientContainer;
	}

	/**
	 * Gets the destination mode
	 * 
	 * @return the destination mode
	 */
	public int getDestinationMode() {
		logger.debug("AbstractManagedConnectionFactory#getDestinationMode()");
		return this.destinationMode;
	}

	/**
	 * The Validation enable status
	 * 
	 * @return Returns the Validation enable status.
	 */
	public Boolean getConnectionValidationEnabled() {
		logger.debug("AbstractManagedConnectionFactory#"
				+ "getConnectionValidationEnabled()");
		return new Boolean(connectionValidationEnabled);
	}

	/**
	 * Sets the Validation enable status
	 * 
	 * @param enableValidation
	 *            The Validation enable to set.
	 */
	public void setConnectionValidationEnabled(
			Boolean connectionValidationEnabled) {
		logger.debug("AbstractManagedConnectionFactory#"
				+ "setConnectionValidationEnabled("
				+ connectionValidationEnabled + ")");
		this.connectionValidationEnabled = connectionValidationEnabled
				.booleanValue();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		logger.debug("AbstractManagedConnectionFactory#equals(" + obj + ")");

		if (obj == null)
			return false;
		if (!(obj instanceof AbstractManagedConnectionFactory))
			return false;
		if (!(super.equals(obj)))
			return false;

		AbstractManagedConnectionFactory obj2 = 
			(AbstractManagedConnectionFactory) obj;
		return (StringUtils.isEqual(this.clientId, obj2.clientId)
				&& StringUtils.isEqual(this.connectionFactoryJndiName,
						obj2.connectionFactoryJndiName) && 
						this.destinationMode == obj2.destinationMode)
						
						// TODO To eliminate the old connections in the pool
						&& this.properties.equals(obj2.properties);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return connectionFactoryJndiName + " " + clientId + " "
				+ connectionValidationEnabled + " "
				+ inApplicationClientContainer;
	}

}