/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConnectionFactoryImpl.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.naming.Reference;
import javax.resource.Referenceable;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ManagedConnectionFactory;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * <code>ConnectionFactory<code> implementation for the connector.
 * Implements the <code>ConnectionFactory</code>, 
 * <code>TopicConnectionFactory</code>, <code>QueueConnectionFactory</code> 
 * interfaces. Provides an interface for getting a connection to a JMS provider. 
 * Application code looks up a <code>ConnectionFactoryImpl </code> instance from 
 * JNDI namespace and uses it to get EIS connections.
 * 
 * The <code>ManagedConnectionFactory</code> creates an instance of this class
 * while asked to create a <code>ConnectionFactory</code>.
 * 
 * @author Yacine TOUIL
 */
public class ConnectionFactoryImpl implements ConnectionFactory,
		TopicConnectionFactory, QueueConnectionFactory, Referenceable {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * the managed connection factory associated with this object.
	 */
	private ManagedConnectionFactory mcf;

	/**
	 * the connection manager associated with this object.
	 */
	private ConnectionManager cm;

	/**
	 * The naming reference of this factory..
	 */
	private Reference reference;

	/**
	 * The constructor of the class
	 * 
	 * @param mcf
	 *            the managed connection factory associated with this object.
	 * @param cm
	 *            the connection manager associated with this object.
	 */
	public ConnectionFactoryImpl(ManagedConnectionFactory mcf,
			ConnectionManager cm) {
		logger.debug("ConnectionFactoryImpl#ConnectionFactoryImpl(" + mcf + ","
				+ cm + ")");
		this.mcf = mcf;
		this.cm = cm;
	}

	/**
	 * create a connection with the default configured username and password!
	 * 
	 * @return a Connection
	 * @throws JMSException
	 *             exception occure if faild to create the connection
	 */
	public Connection createConnection() throws JMSException {
		logger.debug("ConnectionFactoryImpl#createConnection()");
		Connection conn = null;
		try {
			conn = (Connection) this.cm.allocateConnection(this.mcf, null);
		} catch (ResourceException e) {
			logger.info("ConnectionFactoryImpl#createConnection: "
					+ "Failed to allocate! : " + e.getMessage(), e);
			throw (JMSException) new JMSException(e.getMessage()).initCause(e);
		}
		return conn;
	}

	/**
	 * allocate a connection using userName and password
	 * 
	 * @param userName
	 *            the userName for get the connection
	 * @param password
	 *            the password for get the connection
	 * @return a Connection
	 * @throws JMSException
	 *             exception occure if faild to create the connection
	 */
	public Connection createConnection(String userName, String password)
			throws JMSException {
		logger.debug("ConnectionFactoryImpl#createConnection(" + userName + ","
				+ password + ")");
		ConnectionRequestInfoImpl info = new ConnectionRequestInfoImpl(
				this.mcf, userName, password);
		Connection con = null;
		try {
			con = (Connection) cm.allocateConnection(this.mcf, info);
		} catch (ResourceException e) {
			logger.info(e.getMessage(), e);
			// Failed to allocate!
			throw (JMSException) new JMSException(e.getMessage()).initCause(e);
		}
		return con;
	}

	/**
	 * create a topic connection
	 * 
	 * @return a Topic Connection
	 * @throws JMSException
	 *             exception occure if faild to create the connection
	 */
	public TopicConnection createTopicConnection() throws JMSException {
		logger.debug("ConnectionFactoryImpl#createTopicConnection()");
		return (TopicConnection) createConnection();
	}

	/**
	 * create a topic connection with the default configured username and
	 * password!
	 * 
	 * @param userName
	 *            the userName for get the topic connection
	 * @param password
	 *            the password for get the topic connection
	 * @return a Topic Connection
	 * @throws JMSException
	 *             exception occure if faild to create the connection
	 */
	public TopicConnection createTopicConnection(String userName,
			String password) throws JMSException {
		logger.debug("ConnectionFactoryImpl#createTopicConnection(" + userName
				+ "," + password + ")");
		return (TopicConnection) createConnection(userName, password);
	}

	/**
	 * create a queue connection
	 * 
	 * @return a Queue Connection
	 * @throws JMSException
	 *             exception occure if faild to create the connection
	 */
	public QueueConnection createQueueConnection() throws JMSException {
		logger.debug("ConnectionFactoryImpl#createQueueConnection()");
		return (QueueConnection) createConnection();
	}

	/**
	 * create a queue connection with the default configured username and
	 * password!
	 * 
	 * @param userName
	 *            the userName for get the queue connection
	 * @param password
	 *            the password for get the queue connection
	 * @return a Queue Connection
	 * @throws JMSException
	 *             exception occure if faild to create the connection
	 */
	public QueueConnection createQueueConnection(String userName,
			String password) throws JMSException {
		logger.debug("ConnectionFactoryImpl#createQueueConnection(" + userName
				+ "," + password + ")");
		return (QueueConnection) createConnection(userName, password);
	}

	/**
	 * Sets the naming reference of this factory.
	 */
	public void setReference(Reference ref) {
		logger.debug("ConnectionFactoryImpl#setReference(" + ref + ")");
		this.reference = ref;
	}

	/**
	 * Returns the naming reference of this factory.
	 */
	public Reference getReference() {
		logger.debug("ConnectionFactoryImpl#getReference()");
		return reference;
	}
}