/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConnectionHandle.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jms.ConnectionConsumer;
import javax.jms.ConnectionMetaData;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.IllegalStateException;
import javax.jms.InvalidClientIDException;
import javax.jms.InvalidDestinationException;
import javax.jms.InvalidSelectorException;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.ServerSessionPool;
import javax.jms.Session;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.resource.spi.ConnectionEvent;

import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * A <code>ConnectionHandle</code> instance is a handler for a physical
 * connection to an underlying MQ server, allowing a component to transparently
 * use this physical connection possibly within a transaction.
 * 
 * @author Yacine TOUIL
 */
public class ConnectionHandle implements javax.jms.Connection, TopicConnection,
		QueueConnection {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * the managed connection.
	 */
	private ManagedConnectionImpl mc;

	/**
	 * <code>true</code> if this "handle" is valid.
	 */
	private javax.jms.Connection connnection;

	/**
	 * <code>true</code> if the session is started.
	 */
	private boolean started = true;

	/**
	 * <code>true</code> if this "handle" is valid.
	 */
	private boolean valid = true;

	/**
	 * list of sessions.
	 */
	private ArrayList sessions = new ArrayList();

	/**
	 * list of temp destinations.
	 */
	private ArrayList tempDestinations = new ArrayList();

	/**
	 * The constructor of the class
	 * 
	 * @param managedConnection
	 *            the managed connection
	 */
	public ConnectionHandle(ManagedConnectionImpl managedConnection) {
		logger.debug("ConnectionHandle#ConnectionHandle(" + managedConnection
				+ ")");
		this.mc = managedConnection;
		this.connnection = mc.getPhysicalConnection();
	}

	/**
	 * Is the connector in a Application Client Container mode ?
	 * 
	 * @return true if the connector in a Application Client Container mode;
	 *         false otherwise
	 */
	private boolean inApplicationClientContainer() {
		logger.debug("ConnectionHandle#inApplicationClientContainer()");
		return ((AbstractManagedConnectionFactory) mc
				.getManagedConnectionFactory())
				.isInApplicationClientContainer();
	}

	/**
	 * Is this object started
	 * 
	 * @return True if this object is started, false otherwise.
	 */
	public boolean isStarted() {
		logger.debug("ConnectionHandle#isStarted()");
		return started;
	}

	/**
	 * Is this object valid
	 * 
	 * @return True if this object is valid, false otherwise.
	 */
	public boolean isValid() {
		logger.debug("ConnectionHandle#isValid()");
		return valid;
	}

	/**
	 * Invalidate this object
	 */
	public void invalidate() {
		logger.debug("ConnectionHandle#invalidate()");
		valid = false;
	}

	/**
	 * Starts (or restarts) a connection's delivery of incoming messages.
	 * 
	 * @throws JMSException
	 *             if the JMS provider fails to stop message delivery due to
	 *             some internal error.
	 */
	public void start() throws JMSException {
		logger.debug("ConnectionHandle#start()");
		checkIfValid();
		this.connnection.start();
	}

	/**
	 * Temporarily stops a connection's delivery of incoming messages.
	 * 
	 * @throws JMSException
	 *             if the JMS provider fails to stop message delivery due to
	 *             some internal error.
	 */
	public void stop() throws JMSException {
		logger.debug("ConnectionHandle#stop()");
		if (!inApplicationClientContainer())
			throw new JMSException("Conncetion.stop() cannot be called on "
					+ "non Application Client Container clients");
		checkIfValid();
		this.connnection.stop();
	}

	/**
	 * Closes the connection.
	 * 
	 * @throws JMSException
	 *             if the JMS provider fails to close the connection due to some
	 *             internal error. For example, a failure to release resources
	 *             or to close a socket connection can cause this exception to
	 *             be thrown.
	 */
	public void close() throws JMSException {
		logger.debug("ConnectionHandle#close()");
		if (!isStarted()) {
			return;
		}

		// connnection.close();
		// connnection.stop();
		synchronized (sessions) {
			Object[] listSession = sessions.toArray();
			for (int i = 0; i < listSession.length; i++) {
				SessionAdapter sa = (SessionAdapter) listSession[i];
				sa.close();
			}
		}

		synchronized (tempDestinations) {
			Object[] listDests = tempDestinations.toArray();
			for (int i = 0; i < listDests.length; i++) {
				if (listDests[i] instanceof TemporaryQueue) {
					((TemporaryQueue) listDests[i]).delete();
				}
				if (listDests[i] instanceof TemporaryTopic) {
					((TemporaryTopic) listDests[i]).delete();
				}
			}
		}

		this.started = false;

		invalidate();
		mc.dissociateConnection(this);
		mc.sendConnectionEvent(ConnectionEvent.CONNECTION_CLOSED, null, this);

		if (!mc.isTransactionInProgress()) {
			mc.sendConnectionErrorEvent(this);
		}

		// if (true)
		// return;
	}

	/**
	 * Iterate through Sessions and close them.
	 * 
	 * @throws JMSException
	 *             exception ocure if the operation faild
	 */
	public void cleanup() throws JMSException {
		logger.debug("ConnectionHandle#cleanup()");
		for (Iterator iter = this.sessions.iterator(); iter.hasNext();) {
			SessionAdapter sa = (SessionAdapter) iter.next();
			sa.invalidate();
			iter.remove();
		}
	}

	/**
	 * Gets the metadata for this connection.
	 * 
	 * @return the connection metadata
	 * @throws JMSException
	 *             if the JMS provider fails to get the connection metadata for
	 *             this connection
	 */
	public ConnectionMetaData getMetaData() throws JMSException {
		logger.debug("ConnectionHandle#getMetaData()");
		checkIfValid();
		return connnection.getMetaData();
	}

	/**
	 * Sets the client identifier for this connection.
	 * 
	 * @param clientID
	 *            the unique client identifier
	 * @throws JMSException
	 *             if the JMS provider fails to set the client ID for this
	 *             connection due to some internal error.
	 * @throws InvalidClientIDException -
	 *             if the JMS client specifies an invalid or duplicate client
	 *             ID.
	 * @throws IllegalStateException -
	 *             if the JMS client attempts to set a connection's client ID at
	 *             the wrong time or when it has been administratively
	 *             configured.
	 */
	public void setClientID(String clientID) throws JMSException {
		logger.debug("ConnectionHandle#setClientID(" + clientID + ")");
		checkIfValid();
		if (!inApplicationClientContainer())
			throw new JMSException("Client ID cannot be set in non "
					+ "Application Client Container clients");
		connnection.setClientID(clientID);
	}

	/**
	 * Gets the client identifier for this connection.
	 * 
	 * @return the unique client identifier
	 * @throws JMSException
	 *             if the JMS provider fails to return the client ID for this
	 *             connection due to some internal error.
	 */
	public String getClientID() throws JMSException {
		logger.debug("ConnectionHandle#getClientID()");
		if (!inApplicationClientContainer())
			throw new JMSException("Client ID cannot be get in non "
					+ "Application Client Container clients");
		checkIfValid();
		return connnection.getClientID();
	}

	/**
	 * Sets an exception listener for this connection.
	 * 
	 * @param listener
	 *            the exception listener
	 * @throws JMSException
	 *             if the JMS provider fails to set the exception listener for
	 *             this connection.
	 */
	public void setExceptionListener(ExceptionListener listener)
			throws JMSException {
		logger.debug("ConnectionHandle#setExceptionListener(" + listener + ")");
		if (!inApplicationClientContainer())
			throw new JMSException(
					"Conncetion.setExceptionListener() cannot be"
							+ " called on non Application Client Container clients");
		checkIfValid();
		connnection.setExceptionListener(listener);
	}

	/**
	 * Gets the ExceptionListener object for this connection.
	 * 
	 * @return the unique client identifier
	 * @throws JMSException
	 *             the ExceptionListener for this connection, or null. if no
	 *             ExceptionListener is associated with this connection.
	 */
	public ExceptionListener getExceptionListener() throws JMSException {
		logger.debug("ConnectionHandle#getExceptionListener()");
		if (!inApplicationClientContainer())
			throw new JMSException(
					"Conncetion.getExceptionListener() cannot be"
							+ " called on non Application Client Container clients");
		checkIfValid();
		return connnection.getExceptionListener();
	}

	/**
	 * Creates a Session object.
	 * 
	 * @param transacted
	 *            indicates whether the session is transacted
	 * @param acknowledgeMode
	 *            indicates whether the consumer or the client will acknowledge
	 *            any messages it receives; ignored if the session is
	 *            transacted. Legal values are Session.AUTO_ACKNOWLEDGE,
	 *            Session.CLIENT_ACKNOWLEDGE, and Session.DUPS_OK_ACKNOWLEDGE.
	 * @return a newly created session
	 * @throws JMSException
	 *             if the Connection object fails to create a session due to
	 *             some internal error or lack of support for the specific
	 *             transaction and acknowledgement mode.
	 */
	public Session createSession(boolean transacted, int acknowledgeMode)
			throws JMSException {
		logger.debug("ConnectionHandle#createSession(" + transacted + ","
				+ acknowledgeMode + ")");
		checkIfValid();

		if (mc.isTransactionInProgress()) {
			if (!mc.canSessionsBeCreated())
				throw new JMSException("More than one sessions cannot be "
						+ "created within a transaction");
		}

		Session s = mc.getSession(transacted, acknowledgeMode,
				Constants.JMS_SESSION);
		SessionAdapter sa = new SessionAdapter(s, this);
		this.sessions.add(sa);
		return sa;

	}

	/**
	 * a newly created topic session
	 * 
	 * @param transacted
	 *            indicates whether the session is transacted
	 * @param acknowledgeMode
	 *            indicates whether the consumer or the client will acknowledge
	 *            any messages it receives; ignored if the session is
	 *            transacted. Legal values are Session.AUTO_ACKNOWLEDGE,
	 *            Session.CLIENT_ACKNOWLEDGE, and Session.DUPS_OK_ACKNOWLEDGE.
	 * @return a newly created queue session
	 * @throws JMSException
	 *             if the TopicConnection object fails to create a session due
	 *             to some internal error or lack of support for the specific
	 *             transaction and acknowledgement mode.
	 */
	public TopicSession createTopicSession(boolean transacted,
			int acknowledgeMode) throws JMSException {
		logger.debug("ConnectionHandle#createTopicSession(" + transacted + ","
				+ acknowledgeMode + ")");
		TopicSession s = (TopicSession) mc.getSession(transacted,
				acknowledgeMode, Constants.TOPIC_SESSION);
		SessionAdapter sa = new SessionAdapter(s, this);
		this.sessions.add(sa);
		return sa;
	}

	/**
	 * Creates a QueueSession object
	 * 
	 * @param transacted
	 *            indicates whether the session is transacted
	 * @param acknowledgeMode
	 *            indicates whether the consumer or the client will acknowledge
	 *            any messages it receives; ignored if the session is
	 *            transacted. Legal values are Session.AUTO_ACKNOWLEDGE,
	 *            Session.CLIENT_ACKNOWLEDGE, and Session.DUPS_OK_ACKNOWLEDGE.
	 * @return a newly created queue session
	 * @throws JMSException
	 *             if the QueueConnection object fails to create a session due
	 *             to some internal error or lack of support for the specific
	 *             transaction and acknowledgement mode.
	 */
	public QueueSession createQueueSession(boolean transacted,
			int acknowledgeMode) throws JMSException {
		logger.debug("ConnectionHandle#createQueueSession(" + transacted + ","
				+ acknowledgeMode + ")");
		QueueSession s = (QueueSession) mc.getSession(transacted,
				acknowledgeMode, Constants.QUEUE_SESSION);
		SessionAdapter sa = new SessionAdapter(s, this);
		this.sessions.add(sa);
		return sa;
	}

	/**
	 * Creates a connection consumer for this connection (optional operation).
	 * This is an expert facility not used by regular JMS clients.
	 * 
	 * @param destination
	 *            the destination to access
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @param sessionPool
	 *            the server session pool to associate with this connection
	 *            consumer
	 * @param maxMessages
	 *            the maximum number of messages that can be assigned to a
	 *            server session at one time
	 * @return the ConnectionConsumer created object.
	 * @throws JMSException
	 *             if the Connection object fails to create a connection
	 *             consumer due to some internal error or invalid arguments for
	 *             sessionPool and messageSelector.
	 * @throws InvalidDestinationException -
	 *             if an invalid destination is specified.
	 * @throws InvalidSelectorException -
	 *             if the message selector is invalid.
	 */
	public ConnectionConsumer createConnectionConsumer(Destination destination,
			String messageSelector, ServerSessionPool sessionPool,
			int maxMessages) throws JMSException {
		logger.debug("ConnectionHandle#createConnectionConsumer(" + destination
				+ "," + messageSelector + "," + sessionPool + "," + maxMessages
				+ ")");
		checkIfValid();
		return connnection.createConnectionConsumer(destination,
				messageSelector, sessionPool, maxMessages);

	}

	/**
	 * Creates a connection consumer for this connection (optional operation).
	 * This is an expert facility not used by regular JMS clients.
	 * 
	 * @param topic
	 *            the topic to access
	 * @param subscriptionName
	 *            subscription name
	 * @param sessionPool
	 *            the server session pool to associate with this connection
	 *            consumer
	 * @param maxMessages
	 *            the maximum number of messages that can be assigned to a
	 *            server session at one time
	 * @return the unique client identifier
	 * @throws JMSException
	 *             if the TopicConnection object fails to create a connection
	 *             consumer due to some internal error or invalid arguments for
	 *             sessionPool and messageSelector.
	 * @throws InvalidDestinationException
	 *             if an invalid topic is specified.
	 * @throws InvalidSelectorException
	 *             if the message selector is invalid.
	 * 
	 */
	public ConnectionConsumer createConnectionConsumer(Topic topic,
			String subscriptionName, ServerSessionPool sessionPool,
			int maxMessages) throws JMSException {
		logger.debug("ConnectionHandle#createConnectionConsumer(" + topic + ","
				+ subscriptionName + "," + sessionPool + "," + maxMessages
				+ ")");
		checkIfValid();
		return connnection.createConnectionConsumer(topic, subscriptionName,
				sessionPool, maxMessages);
	}

	/**
	 * Creates a connection consumer for this connection (optional operation).
	 * This is an expert facility not used by regular JMS clients.
	 * 
	 * @param queue
	 *            the queue to access
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @param sessionPool
	 *            the server session pool to associate with this connection
	 *            consumer
	 * @param maxMessages
	 *            the maximum number of messages that can be assigned to a
	 *            server session at one time
	 * @return the connection consumer
	 * @throws JMSException
	 *             if the QueueConnection object fails to create a connection
	 *             consumer due to some internal error or invalid arguments for
	 *             sessionPool and messageSelector.
	 * @throws InvalidDestinationException
	 *             if an invalid queue is specified.
	 * @throws InvalidSelectorException
	 *             if the message selector is invalid.
	 */
	public ConnectionConsumer createConnectionConsumer(Queue queue,
			String messageSelector, ServerSessionPool sessionPool,
			int maxMessages) throws JMSException {
		logger
				.debug("ConnectionHandle#createConnectionConsumer(" + queue
						+ "," + messageSelector + "," + sessionPool + ","
						+ maxMessages + ")");
		checkIfValid();
		return connnection.createConnectionConsumer(queue, messageSelector,
				sessionPool, maxMessages);
	}

	/**
	 * Create a durable connection consumer for this connection (optional
	 * operation). This is an expert facility not used by regular JMS clients.
	 * 
	 * @param topic
	 *            topic to access
	 * @param subscriptionName
	 *            durable subscription name
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @param sessionPool
	 *            the server session pool to associate with this durable
	 *            connection consumer
	 * @param maxMessages
	 *            the maximum number of messages that can be assigned to a
	 *            server session at one time
	 * @return the durable connection consumer
	 * @throws JMSException
	 *             if the Connection object fails to create a connection
	 *             consumer due to some internal error or invalid arguments for
	 *             sessionPool and messageSelector.
	 * @throws InvalidDestinationException -
	 *             if an invalid destination is specified.
	 * @throws InvalidSelectorException -
	 *             if the message selector is invalid.
	 */
	public ConnectionConsumer createDurableConnectionConsumer(Topic topic,
			String subscriptionName, String messageSelector,
			ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		logger.debug("ConnectionHandle#createDurableConnectionConsumer("
				+ topic + "," + subscriptionName + "," + messageSelector + ","
				+ sessionPool + "," + maxMessages + ")");
		checkIfValid();
		return connnection.createDurableConnectionConsumer(topic,
				subscriptionName, messageSelector, sessionPool, maxMessages);
	}

	/**
	 * Gets the session's list.
	 * 
	 * @return session's list.
	 */
	public List getSessions() {
		logger.debug("ConnectionHandle#getSessions()");
		return this.sessions;
	}

	/**
	 * Associate with a managed connection.
	 * 
	 * @param mc
	 *            the managed connection.
	 */
	public void associateConnection(ManagedConnectionImpl mc) {
		logger.debug("ConnectionHandle#associateConnection(" + mc + ")");
		this.mc = mc;
		connnection = mc.getPhysicalConnection();
	}

	/**
	 * Gets the associated managed connection.
	 * 
	 * @return the managed connection.
	 */
	public ManagedConnectionImpl getManagedConnection() {
		logger.debug("ConnectionHandle#getManagedConnection()");
		return mc;
	}

	/**
	 * Add a temporary destination
	 * 
	 * @param destination
	 *            to add.
	 */
	public void addTemporaryDest(Destination destination) {
		logger.debug("ConnectionHandle#addTemporaryDest(" + destination + ")");
		tempDestinations.add(destination);
	}

	/**
	 * Check if the this object is started and valid
	 * 
	 * @throws JMSException
	 */
	private void checkIfValid() throws JMSException {
		logger.debug("ConnectionHandle#checkIfValid()");
		if (!started)
			throw new IllegalStateException("Connection is closed");
		if (!valid)
			throw new IllegalStateException("Invalid connection");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "ConnectionHandle[" + connnection.toString() + "," + started
				+ "," + valid + "]:" + super.toString();
	}
}