/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DestinationAdapter.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */
package com.bull.jonasmqconnector.outbound;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.resource.ResourceException;

import com.bull.jonasmqconnector.ConnectorProperties;
import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.JavaBeanBuilder;
import com.bull.jonasmqconnector.util.JavaBeanExecutor;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;
import com.bull.jonasmqconnector.util.StringUtils;
 
/** 
 * A Destination Adapter object encapsulates a provider-specific address.
 * @author Yacine TOUIL 
 */

public abstract class DestinationAdapter extends ConnectorProperties
	implements Destination {

	/** 
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();
	
    /**
     * Destination properties
     */
    private String destinationProperties = null;
    
    /**
     * jndi name 
     */
    private String jndiName = null;
    
    /**
     * Physical destination
     */
    private javax.jms.Destination physicalDestination;

    /** 
	 * This is the constructor of the class.
	 */
	public DestinationAdapter() {
		logger.debug("DestinationAdapter#DestinationAdapter()");

	}

    /**
     * Set the physical sestination
     * @param physicalDestination physical sestination
     * @throws JMSException exception occure if if operation failed  
     */
    public void setPhysicalDestination(Destination physicalDestination) 
    		throws JMSException {
		logger.debug("DestinationAdapter#setPhysicalDestination()");
        this.physicalDestination = physicalDestination;
    }

    /**
     * Get the physical sestination
     * @return physical sestination
     * @throws JMSException exception occure if if operation failed  
     */
    public Destination getPhysicalDestination() throws JMSException {
		logger.debug("DestinationAdapter#getPhysicalDestination()");
        logger.debug("DestinationAdapter#getPhysicalDestination() : "+
        		"Returning physical destintion " + this.physicalDestination);
        if (this.physicalDestination == null) {
            try {
                init();
            }catch (ResourceException re) {
                throw (JMSException)
                	new JMSException(re.getMessage()).initCause(re);
            }
        }

        logger.debug("DestinationAdapter#getPhysicalDestination() : "+
        		"Physical destintion type" + 
        		this.physicalDestination.getClass().getName());

        return this.physicalDestination;
    }

    /**
     * Connector 1.5 Specification does not support ResourceAdapterAssociation
     * for Administered Objects. This call is made so that the current 
     * active resource adapter instance can be set in 
     * <code>GenericJMSRAProperties</code> so that all properties 
     * already configured for the resource adapter can be reused in this 
     * Destination instance (for example ProviderIntegrationMode)
     * @throws ResourceException exception occure if if operation failed
     */
    private void init() throws ResourceException {
    	logger.debug("DestinationAdapter#init()");
   		try {
   	    	JavaBeanBuilder jbBuilder = new JavaBeanBuilder();
			this.physicalDestination = (javax.jms.Destination) jbBuilder.build(
				getDestinationClassName(), 
				JavaBeanExecutor.parseToProperties(getDestinationProperties()));
		} catch (Exception e) {
			ExceptionUtils.newException("Canot init destination adapter", e);
		}
    }
    
    /**
     * Get the destination class name
     * @return destination class name
     */
    protected abstract String getDestinationClassName();
    
    /**
     * set the destination jndi name
     * @param jndiName destination jndi name
     */
    public void setDestinationJndiName(String jndiName) {
		logger.debug("DestinationAdapter#setDestinationJndiName("+jndiName+")");
        this.jndiName = jndiName;
    }

    /**
     * Get the destination jndi name
     * @return destination jndi name
     */
    public String getDestinationJndiName() {
		logger.debug("DestinationAdapter#getDestinationJndiName()");
        return this.jndiName;
    }
    
    /**
     * get the destination properties
     * @return destination properties
     */
    public String getDestinationProperties() {
		logger.debug("DestinationAdapter#getDestinationProperties()");
        return destinationProperties;
    }

    /**
     * Setting destination property is mandatory, unless we find a 
     * way out.
     * @param destinationProperties The destinationProperties to set.
     */
    public void setDestinationProperties(String destinationProperties) {
		logger.debug("DestinationAdapter#setDestinationProperties("+
				destinationProperties+")");
        if (!StringUtils.isEmpty(destinationProperties)) {
        	this.destinationProperties = destinationProperties;
            try {
                //Call initialized here.
                init();
            } catch (ResourceException e) {
        		logger.debug("DestinationAdapter#setDestinationProperties: "+
        				"Cannot create Destination " + e);
            }
        }
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return physicalDestination+" "+destinationProperties+" "+jndiName;
    }
}