/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ManagedConnectionImpl.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jms.Connection;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.XAConnection;
import javax.jms.XAQueueConnection;
import javax.jms.XAQueueSession;
import javax.jms.XASession;
import javax.jms.XATopicConnection;
import javax.jms.XATopicSession;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;

import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;
import com.bull.jonasmqconnector.util.SecurityUtils;

/**
 * This class is the implementation of ManagedConnection for the connector.
 * 
 * @author Yacine TOUIL
 */
public class ManagedConnectionImpl implements ManagedConnection {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * The managed connection factory
	 */
	private AbstractManagedConnectionFactory mcf;

	/**
	 * This is the connecion.
	 */
	private javax.jms.Connection connection;

	/**
	 * This is the JMS session.
	 */
	private Session session;

	/**
	 * This is the XA Session.
	 */
	private XASession xaSession;

	/**
	 * This is the XA Ressource.
	 */
	private XAResource xaResource;

	/**
	 * This is the active handle.
	 */
	private ConnectionHandle activeHandle;

	/**
	 * This is the ConnectionEventListenerHandler of this connector.
	 */
	private ConnectionEventListenerHandler celHandler;

	/**
	 * List of the connectionHandles.
	 */
	private ArrayList connectionHandles = new ArrayList();

	/**
	 * This is the password credential.
	 */
	private PasswordCredential passwordCredential;

	/**
	 * Indicate if there are a transaction in progress.
	 */
	private boolean transactionInProgress = false;

	/**
	 * Indicate if this object was destroyed.
	 */
	private boolean destroyed = false;

	/**
	 * log Writer.
	 */
	private PrintWriter pw;

	/**
	 * The acrive session adapter
	 */
	private SessionAdapter activeSA = null;

	/**
	 * Indicate if this object was destroyed.
	 */
	private ConnectionRequestInfo connRequestInfo;

	/**
	 * This is the Constructor of the class.
	 * 
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public ManagedConnectionImpl(AbstractManagedConnectionFactory factory,
			PasswordCredential pc, ConnectionRequestInfo connRequestInfo,
			javax.jms.Connection connection) throws ResourceException {
		logger.debug("ManagedConnectionImpl#ManagedConnectionImpl()");
		this.mcf = factory;
		this.passwordCredential = pc;
		this.connRequestInfo = connRequestInfo;
		this.connection = connection;
		this.celHandler = new ConnectionEventListenerHandler(this);
		init();
	}

	/**
	 * Set exception listener to perform validation of ManagedConnection
	 * Validation is disabled by default.
	 * 
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	private void init() throws ResourceException {
		logger.debug("ManagedConnectionImpl#init()");
		try {
			if (((AbstractManagedConnectionFactory) getManagedConnectionFactory())
					.getConnectionValidationEnabled().booleanValue()) {
				ExceptionListener l = new ExceptionListener() {
					public void onException(JMSException jmsEx) {
						ManagedConnectionImpl.this.celHandler.sendEvent(
								ConnectionEvent.CONNECTION_ERROR_OCCURRED,
								jmsEx, null);
					}
				};
				this.connection.setExceptionListener(l);
			}
		} catch (JMSException e) {
			String msg = "An exception occured while setting exception listener"
					+ " on JMS physical connection.";
			throw (ResourceException) new ResourceException(msg).initCause(e);
		}
	}

	/**
	 * Get the session, if transaction is in progress physical session will be
	 * created by ManagedConnection itself.
	 * 
	 * @param transacted
	 *            true is transacted, false otherwise
	 * @param ack
	 *            the acknowledge mode
	 * @param sessionType
	 *            the session type
	 * @return the session
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	public Session getSession(boolean transacted, int ack, int sessionType)
			throws JMSException {
		logger.debug("ManagedConnectionImpl#getSession(" + transacted + ","
				+ ack + "," + sessionType + ")");
		if (isTransactionInProgress()) {
			return session;
		} else {
			return createLocalSession(transacted, ack, sessionType);
		}
	}

	/**
	 * Create a XA session
	 * 
	 * @param sessionType
	 *            the type
	 * @return the session
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	private javax.jms.Session createXaSession(int sessionType)
			throws JMSException {
		logger.debug("ManagedConnectionImpl#createXaSession(" + sessionType
				+ ")");
		Session result = null;
		switch (sessionType) {
		case Constants.JMS_SESSION:
			XASession xas = ((XAConnection) connection).createXASession();
			this.xaResource = xas.getXAResource();
			this.xaSession = xas;
			result = xas.getSession();
			break;
		case Constants.TOPIC_SESSION:
			XATopicSession xats = ((XATopicConnection) connection)
					.createXATopicSession();
			this.xaResource = xats.getXAResource();
			this.xaSession = xats;
			result = xats.getTopicSession();
			break;
		case Constants.QUEUE_SESSION:
			XAQueueSession xaqs = ((XAQueueConnection) connection)
					.createXAQueueSession();
			this.xaResource = xaqs.getXAResource();
			this.xaSession = xaqs;
			result = xaqs.getQueueSession();
			break;
		}
		return result;
	}

	/**
	 * Create a local session
	 * 
	 * @param transacted
	 *            true if transacted; false otherwise
	 * @param acknowledgeMode
	 *            the acknowledge mode
	 * @param sessionType
	 *            the session type
	 * @return a session
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	public javax.jms.Session createLocalSession(boolean transacted,
			int acknowledgeMode, int sessionType) throws JMSException {
		logger.debug("ManagedConnectionImpl#createLocalSession(" + transacted
				+ "," + acknowledgeMode + "," + sessionType + ")");
		Session result = null;
		switch (sessionType) {
		case Constants.JMS_SESSION:
			Session s = ((Connection) connection).createSession(transacted,
					acknowledgeMode);
			result = s;
			break;
		case Constants.TOPIC_SESSION:
			TopicSession st = ((TopicConnection) connection)
					.createTopicSession(transacted, acknowledgeMode);
			result = st;
			break;
		case Constants.QUEUE_SESSION:
			QueueSession sq = ((QueueConnection) connection)
					.createQueueSession(transacted, acknowledgeMode);
			result = sq;
			break;
		}
		return result;
	}

	/**
	 * Return a proxy of the JMS actual-connection which intercepts all
	 * <code> Connection </code> calls and implements javax.jms.Connection,
	 * Topic and QueueConnection Intercepts, and delegates to actual connection,
	 * if connection handle is not closed already. Intercepts close and calls
	 * mc.connectionClosed, so that MC can invalidate this connection handle and
	 * send ConnectionHandle closed event to all registered listeners.
	 * 
	 * @param subject
	 *            the subject
	 * @param connRequestInfo
	 *            the request connection informatiosn
	 * @return the connection
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public Object getConnection(Subject subject,
			ConnectionRequestInfo connRequestInfo) throws ResourceException {
		logger.debug("ManagedConnectionImpl#getConnection(" + subject + ","
				+ connRequestInfo + ")");
		isValid();
		ConnectionRequestInfoImpl cxRequestInfo = (ConnectionRequestInfoImpl) connRequestInfo;
		PasswordCredential passedInCred = SecurityUtils.getPasswordCredential(
				this.mcf, subject, cxRequestInfo);
		if (SecurityUtils.isPasswordCredentialEqual(this.passwordCredential,
				passedInCred) == false) {
			throw new javax.resource.spi.SecurityException(
					"Re-authentication not supported");
		}
		ConnectionHandle connectionHandle = new ConnectionHandle(this);
		addConnectionHandle(connectionHandle);
		try {
			if (this.mcf.getClientId() != null
					&& this.connection.getClientID() == null) {
				this.connection.setClientID(this.mcf.getClientId());
			}
		} catch (JMSException e) {
			throw (ResourceException) new ResourceException(e.getMessage())
					.initCause(e);
		}
		return connectionHandle;
	}

	/**
	 * Destroys the physical connection to the underlying resource manager.
	 * 
	 * @throws ResourceException
	 *             if there is an error in closing the physical connection
	 */
	public void destroy() throws ResourceException {
		logger.debug("ManagedConnectionImpl#destroy()");
		if (isDestroyed())
			return;
		cleanup();
		try {
			if (this.session != null) {
				this.session.close();
			}
			if (this.xaSession != null) {
				this.xaSession.close();
			}
			connection.close();
			connection = null;
		} catch (JMSException e) {
			String msg = "ManagedConnectionImpl#destroyError occured while "
					+ "closing JMS connection." + e.getMessage();
			throw (ResourceException) new ResourceException(msg).initCause(e);
		} finally {
			this.destroyed = true;
		}
	}

	/**
	 * Application server calls this method to force any cleanup on the
	 * <code>ManagedConnection</code> instance. This method calls the
	 * invalidate method on all logical connection handles associated with this
	 * <code>ManagedConnection</code>.
	 * 
	 * @throws ResourceException
	 *             if the physical connection is no more valid
	 */
	public void cleanup() throws ResourceException {
		logger.debug("ManagedConnectionImpl#cleanup()");
		ConnectionHandle ch = null;
		if (!(isTransactionInProgress())) {
			Iterator iterator = connectionHandles.iterator();
			while (iterator.hasNext()) {
				ch = (ConnectionHandle) iterator.next();
				try {
					ch.cleanup();
				} catch (JMSException jmse) {
					logger.debug("ManagedConnectionImpl#cleanup : "
							+ "Exception while cleaning up connection handle"
							+ jmse);
				}
				iterator.remove();
			}
		}

	}

	/**
	 * associate a connecion handle with this object
	 * 
	 * @param connectionHandle
	 *            the connecion handle
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public void associateConnection(Object connectionHandle)
			throws ResourceException {
		logger.debug("ManagedConnectionImpl#associateConnection("
				+ connectionHandle + ")");
		isValid();
		if (connectionHandle == null) {
			throw new ResourceException("Connection handle is null");
		}
		ConnectionHandle ch = (ConnectionHandle) connectionHandle;
		ManagedConnection mc2 = (ManagedConnection) ch.getManagedConnection();
		ch.associateConnection(this);
		addConnectionHandle(ch);
	}

	/**
	 * dissociate a connecion handle from this object
	 * 
	 * @param connectionHandle
	 *            the connecion handle
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	public void dissociateConnection(Object connectionHandle)
			throws JMSException {
		// TODO throws JMSException ? or ResourceException ?
		logger.debug("ManagedConnectionImpl#dissociateConnection("
				+ connectionHandle + ")");
		if (connectionHandle == null) {
			throw new JMSException("Connection handle is null");
		}
		removeConnectionHandle((ConnectionHandle) connectionHandle);
	}

	/**
	 * add an associated connection handle
	 * 
	 * @param ch
	 *            the connection handle
	 */
	private void addConnectionHandle(ConnectionHandle ch) {
		logger.debug("ManagedConnectionImpl#addConnectionHandle(" + ch + ")");
		activeHandle = ch;
		connectionHandles.add(this.activeHandle);
	}

	/**
	 * remove an associated connection handle
	 * 
	 * @param ch
	 *            the connection handle
	 */
	private void removeConnectionHandle(ConnectionHandle ch) {
		logger
				.debug("ManagedConnectionImpl#removeConnectionHandle(" + ch
						+ ")");
		// connectionHandles.remove(this.activeHandle);
		connectionHandles.remove(ch);
		// TODO Verify the active Handle when test
		if (connectionHandles.size() == 0)
			activeHandle = null;
		else
			activeHandle = (ConnectionHandle) connectionHandles.get(0);
	}

	/**
	 * Add a connection event listener to the ConnectionEventListenerHandler
	 * 
	 * @param listener
	 *            the connection event listener to add.
	 */
	public void addConnectionEventListener(ConnectionEventListener listener) {
		logger.debug("ManagedConnectionImpl#addConnectionEventListener("
				+ listener + ")");
		celHandler.addConnectionEventListener(listener);
	}

	/**
	 * Remove a connection event listener from the
	 * ConnectionEventListenerHandler
	 * 
	 * @param listener
	 *            the connection event listener to remove.
	 */
	public void removeConnectionEventListener(ConnectionEventListener listener) {
		logger.debug("ManagedConnectionImpl#removeConnectionEventListener("
				+ listener + ")");
		celHandler.removeConnectionEventListener(listener);
	}

	/**
	 * Gets the XA Resource
	 * 
	 * @return the XA Resource
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public XAResource getXAResource() throws ResourceException {
		logger.debug("ManagedConnectionImpl#getXAResource()");
		isValid();

		XAResourceImpl proxy = new XAResourceImpl(this);
		proxy.setConnection(connection);
		return proxy;
	}

	/**
	 * The connector don't implements a local transaction manager, it use the
	 * transaction manager of the applicaiton server.
	 * 
	 * @return because of the connector don't implements a local transation
	 *         manager, this method return null.
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public LocalTransaction getLocalTransaction() throws ResourceException {
		logger.debug("ManagedConnectionImpl#getLocalTransaction()");
		return null;
	}

	/**
	 * Gets the metadata
	 * 
	 * @return the metadata
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public ManagedConnectionMetaData getMetaData() throws ResourceException {
		logger.debug("ManagedConnectionImpl#getMetaData()");
		isValid();
		return new ManagedConnectionMetaData() {
			public int getMaxConnections() throws ResourceException {
				logger.debug("ManagedConnectionImpl#ManagedConnectionMetaData#"
						+ "getMaxConnections()");
				return 1;
			}

			public String getEISProductName() throws ResourceException {
				logger.debug("ManagedConnectionImpl#ManagedConnectionMetaData#"
						+ "getEISProductName()");
				try {
					return ManagedConnectionImpl.this.connection.getMetaData()
							.getJMSProviderName();
				} catch (JMSException e) {
					throw (ResourceException) new ResourceException(e
							.getMessage()).initCause(e);
				}
			}

			public String getEISProductVersion() throws ResourceException {
				logger.debug("ManagedConnectionImpl#ManagedConnectionMetaData#"
						+ "getEISProductVersion()");
				try {
					return ManagedConnectionImpl.this.connection.getMetaData()
							.getProviderVersion();
				} catch (JMSException e) {
					throw (ResourceException) new ResourceException(e
							.getMessage()).initCause(e);
				}
			}

			public String getUserName() throws ResourceException {
				logger.debug("ManagedConnectionImpl#ManagedConnectionMetaData#"
						+ "getUserName()");
				return ManagedConnectionImpl.this.passwordCredential
						.getUserName();
			}
		};
	}

	/**
	 * set the log wroter
	 * 
	 * @param pw
	 *            the log writer
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public void setLogWriter(PrintWriter pw) throws ResourceException {
		logger.debug("ManagedConnectionImpl#setLogWriter(" + pw + ")");
		this.pw = pw;
	}

	/**
	 * get the log writer
	 * 
	 * @return the log writer
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	public PrintWriter getLogWriter() throws ResourceException {
		logger.debug("ManagedConnectionImpl#getLogWriter()");
		return pw;
	}

	/**
	 * Send a connection event.
	 * 
	 * @param eventType
	 *            the event type
	 * @param ex
	 *            the exception if there is one
	 */
	public void sendConnectionEvent(int eventType, Exception ex) {
		logger.debug("ManagedConnectionImpl#sendConnectionEvent(" + eventType
				+ "," + ex + ")");
		celHandler.sendEvent(eventType, ex, null);
	}

	/**
	 * Send a connection event.
	 * 
	 * @param eventType
	 *            the event type
	 * @param ex
	 *            the exception if there is one
	 * @param connectionHandle
	 *            the connection handle
	 */
	public void sendConnectionEvent(int eventType, Exception ex,
			Object connectionHandle) {
		logger.debug("ManagedConnectionImpl#sendConnectionEvent(" + eventType
				+ "," + ex + "," + connectionHandle + ")");
		celHandler.sendEvent(eventType, ex, connectionHandle);
	}

	/**
	 * Send a error event.
	 * 
	 * @param handle
	 *            the connection handle
	 */
	public void sendConnectionErrorEvent(ConnectionHandle handle) {
		logger.debug("ManagedConnectionImpl#sendConnectionErrorEvent(" + handle
				+ ")");
		if (this.mcf.getClientId() != null) {
			handle.invalidate();
			this.connectionHandles.remove(handle);
			sendConnectionEvent(ConnectionEvent.CONNECTION_ERROR_OCCURRED,
					null, handle);
		}
	}

	/**
	 * Gets the phisical connection.
	 * 
	 * @return the phisical connection.
	 */
	public javax.jms.Connection getPhysicalConnection() {
		logger.debug("ManagedConnectionImpl#getPhysicalConnection()");
		return connection;
	}

	/**
	 * Close a session.
	 * 
	 * @session the session.
	 */
	void closeSession(Session session) throws JMSException {
		logger.debug("ManagedConnectionImpl#closeSession(" + session + ")");
		if (session != this.session) {
			session.close();
		}
	}

	/**
	 * Gets the managed connection factory
	 * 
	 * @return the managed connection factory
	 */
	public Object getManagedConnectionFactory() {
		logger.debug("ManagedConnectionImpl#getManagedConnectionFactory()");
		return mcf;
	}

	/**
	 * Gets if the managed connection factory is destroyed
	 * 
	 * @return true if the managed connection factory is destroyed, false
	 *         otherwise
	 */
	public boolean isDestroyed() {
		logger.debug("ManagedConnectionImpl#isDestroyed()");
		return destroyed;
	}

	/**
	 * Verify if this object is valid
	 * 
	 * @throws ResourceException
	 */
	public void isValid() throws ResourceException {
		logger.debug("ManagedConnectionImpl#isValid()");
		if (isDestroyed()) {
			throw new ResourceException("MC destroyed!!");
		}
	}

	/**
	 * Gets the Password Credential
	 * 
	 * @return the Password Credential
	 */
	public PasswordCredential getPasswordCredential() {
		logger.debug("ManagedConnectionImpl#getPasswordCredential()");
		return passwordCredential;
	}

	/**
	 * Can sessions be created
	 * 
	 * @return true if can sessions be created; false otherwise
	 */
	public boolean canSessionsBeCreated() {
		logger.debug("ManagedConnectionImpl#canSessionsBeCreated()");
		for (Iterator iter = this.connectionHandles.iterator(); iter.hasNext();) {
			ConnectionHandle ch = (ConnectionHandle) iter.next();
			if (ch.getSessions().size() > 1)
				return false;
		}
		return true;
	}

	/**
	 * Create an XA session and get the XA resource.
	 * 
	 * @return the XA Resource
	 */
	XAResource _getXAResource() throws JMSException {
		logger.debug("ManagedConnectionImpl#_getXAResource()");
		verifyXASessionCreated();
		return xaResource;
	}

	/**
	 * Start a XA Transaction.
	 * 
	 * @throws Exception
	 *             exception ocure if the operation fail
	 */
	void _startXaTx() throws Exception {
		logger.debug("ManagedConnectionImpl#_startXaTx()");
		transactionStarted();
		verifyXASessionCreated();
		syncSessions();
	}

	/**
	 * End a XA Transaction.
	 * 
	 * @throws Exception
	 *             exception ocure if the operation fail
	 */
	void _endXaTx() throws JMSException {
		logger.debug("ManagedConnectionImpl#_endXaTx()");
		try {
			reSyncSessions();
		} finally {
			transactionCompleted();
		}
	}

	/**
	 * Verify if the XASession was created, if not this method create it.
	 * 
	 * @exception exception
	 *                ocure if the operation fail
	 */
	private void verifyXASessionCreated() throws JMSException {
		logger.debug("ManagedConnectionImpl#_endXaTx()");
		if (session == null) {
			int sessionMode = ((AbstractManagedConnectionFactory) this.mcf)
					.getDestinationMode();
			this.session = createXaSession(sessionMode);
		}
	}

	/**
	 * If a session is obtained before the start of transaction, the
	 * SessionAdapter will contain a non-transactional session with ackmode and
	 * transaction mode specified by app. When transaction is started by
	 * appserver after getting session replace that session with a session that
	 * can actually take part in transaction.
	 * 
	 * @exception exception
	 *                ocure if the operation fail
	 */
	private void syncSessions() throws ResourceException {
		logger.debug("ManagedConnectionImpl#_endXaTx()");
		for (Iterator iter = connectionHandles.iterator(); iter.hasNext();) {
			ConnectionHandle ch = (ConnectionHandle) iter.next();
			List sessions = ch.getSessions();
			if (sessions.size() > 1) {
				throw new ResourceException(
						"Cannot start a transaction when one or more open "
								+ "sessions are present");
			} else if (sessions.size() == 1) {
				activeSA = (SessionAdapter) ch.getSessions().get(0);
				break;
			}
		}

		if (activeSA != null) {
			activeSA.swapPhysicalSession(session);
		}
	}

	/**
	 * Nullify the session swapping done in syncSessions method. This will be
	 * issued at the end of transaction.
	 */
	private void reSyncSessions() {
		logger.debug("ManagedConnectionImpl#reSyncSessions()");
		if (activeSA != null) {
			activeSA.swapPhysicalSession();
		}
		activeSA = null;
	}

	/**
	 * Mark the managed connection as a transaction in progress
	 * 
	 * @throws ResourceException
	 *             exception ocure if the operation fail
	 */
	private void transactionStarted() throws ResourceException {
		logger.debug("ManagedConnectionImpl#transactionStarted()()");
		transactionInProgress = true;
	}

	/**
	 * Mark the managed connection as no transaction in progress
	 */
	private void transactionCompleted() {
		logger.debug("ManagedConnectionImpl#transactionCompleted()");
		transactionInProgress = false;
		if (activeHandle != null && !activeHandle.isStarted()) {
			sendConnectionErrorEvent(activeHandle);
		}

		// try {
		// activeHandle.close2();
		// } catch (Exception e) {
		// e.printStackTrace();
		// }
	}

	/**
	 * Indicate if there are a transaction in progress.
	 * 
	 * @return true if there are a transaction in progress, false otherwise.
	 */
	public boolean isTransactionInProgress() {
		logger.debug("ManagedConnectionImpl#isTransactionInProgress()");
		return transactionInProgress;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "ManagedConnectionImpl: " + destroyed + " "
				+ transactionInProgress + " " + connectionHandles;
	}
}