/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MessageProducerProxy.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import javax.jms.Destination;
import javax.jms.InvalidDestinationException;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageFormatException;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.QueueSender;
import javax.jms.Topic;
import javax.jms.TopicPublisher;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/** 
 * MessageProducerProxy. This is used to update messages with unwrapped 
 * destinations and reply destinations.
 * @author Yacine TOUIL 
 */
public class MessageProducerProxy implements QueueSender,TopicPublisher {

	/** 
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();
	
    /**
     * this is the Message Producer
     */
    MessageProducer mp = null;
	
    /**
     * this is the constructuer of the class
     * @param mp The message producer
     */
    public MessageProducerProxy(MessageProducer mp) {
		logger.debug("MessageProducerProxy#MessageProducerProxy("+mp+")");
    	this.mp = mp;
    }
    
	/** 
	 * Gets the destination associated with this MessageProducer.
	 * @return this producer's Destination
	 * @throws JMSException if the JMS provider fails to get the destination for 
	 * this MessageProducer due to some internal error.
	 */
	public Destination getDestination() throws JMSException {
		logger.debug("MessageProducer#getDestination()");
		return mp.getDestination();
	}

    /** 
	 * Gets the queue associated with this QueueSender.
	 * @return this sender's queue
	 * @throws JMSException if the JMS provider fails to get the queue for this 
	 * MessageProducer due to some internal error.
	 */
	public Queue getQueue() throws JMSException {
		logger.debug("MessageProducer#getQueue()");
		return ((QueueSender)mp).getQueue();
	}

	/** 
	 * Gets the topic associated with this MessageProducer.
	 * @return this publisher's topic
	 * @throws JMSException if the JMS provider fails to get the topic for this 
	 * MessageProducer due to some internal error.
	 */
	public Topic getTopic() throws JMSException {
		logger.debug("MessageProducer#getTopic()");
		return ((TopicPublisher)mp).getTopic();
	}

	/** 
	 * Sets whether message IDs are disabled.
	 * @param value indicates if message IDs are disabled
	 * @throws JMSException  if the JMS provider fails to set message ID to 
	 * disabled due to some internal error.
	 */
	public void setDisableMessageID(boolean value) throws JMSException {
		logger.debug("MessageProducer#setDisableMessageID("+value+")");
		mp.setDisableMessageID(value);
	}

	/** 
	 * Gets an indication of whether message IDs are disabled.
	 * @return an indication of whether message IDs are disabled
	 * @throws JMSException if the JMS provider fails to determine if message 
	 * IDs are disabled due to some internal error.
	 */
	public boolean getDisableMessageID() throws JMSException {
		logger.debug("MessageProducer#getDisableMessageID()");
		return mp.getDisableMessageID();
	}

	/** 
	 * Sets whether message timestamps are disabled. 
	 * @param value indicates if message timestamps are disabled
	 * @throws JMSException if the JMS provider fails to set timestamps to 
	 * disabled due to some internal error.
	 */
	public void setDisableMessageTimestamp(boolean value) throws JMSException {
		logger.debug("MessageProducer#getQueue("+value+")");
		mp.setDisableMessageTimestamp(value);
	}

	/** 
	 * Gets an indication of whether message timestamps are disabled.
	 * @return an indication of whether message timestamps are disabled 
	 * @throws JMSException if the JMS provider fails to determine if timestamps
	 * are disabled due to some internal error.
	 */
	public boolean getDisableMessageTimestamp() throws JMSException {
		logger.debug("MessageProducer#getDisableMessageTimestamp()");
		return mp.getDisableMessageTimestamp();
	}

	/**
	 * Sets the producer's default delivery mode.
	 * Delivery mode is set to PERSISTENT by default. 
	 * @param deliveryMode the message delivery mode for this message producer; 
	 * legal values are DeliveryMode.NON_PERSISTENT and DeliveryMode.PERSISTENT 
	 * @throws JMSException if the JMS provider fails to set the delivery mode 
	 * due to some internal error.
	 */
	public void setDeliveryMode(int deliveryMode) throws JMSException {
		logger.debug("MessageProducer#setDeliveryMode("+deliveryMode+")");
		mp.setDeliveryMode(deliveryMode);
	}

	/** 
	 * Gets the producer's default delivery mode.
	 * @return the message delivery mode for this message producer 
	 * @throws JMSException  if the JMS provider fails to get the delivery mode 
	 * due to some internal error.
	 */
	public int getDeliveryMode() throws JMSException {
		logger.debug("MessageProducer#getDeliveryMode()");
		return mp.getDeliveryMode();
	}

	/** 
	 * Sets the producer's default priority.
	 * @param defaultPriority the message priority for this message producer; 
	 * must be a value between 0 and 9 
	 * @throws JMSException  if the JMS provider fails to set the priority due 
	 * to some internal error.
	 */
	public void setPriority(int defaultPriority) throws JMSException {
		logger.debug("MessageProducer#setPriority("+defaultPriority+")");
		mp.setPriority(defaultPriority);
	}

	/** 
	 * Gets the producer's default priority. 
	 * @return the message priority for this message producer
	 * @throws JMSException if the JMS provider fails to get the priority due to 
	 * some internal error.
	 */
	public int getPriority() throws JMSException {
		logger.debug("MessageProducer#getPriority()");
		return mp.getPriority();
	}

	/** 
	 * Sets the default length of time in milliseconds from its dispatch time 
	 * that a produced message should be retained by the message system. 
	 * Time to live is set to zero by default. 
	 * @param timeToLive the message time to live in milliseconds; 
	 * zero is unlimited
	 * @throws JMSException if the JMS provider fails to set the time to live 
	 * due to some internal error.
	 */
	public void setTimeToLive(long timeToLive) throws JMSException {
		logger.debug("MessageProducer#setTimeToLive("+timeToLive+")");
		mp.setTimeToLive(timeToLive);
	}

	/** 
	 * Gets the default length of time in milliseconds from its dispatch time 
	 * that a produced message should be retained by the message system. 
	 * @return the message time to live in milliseconds; 
	 * zero is unlimited
	 * @throws JMSException  if the JMS provider fails to get the time to live 
	 * due to some internal error.
	 */
	public long getTimeToLive() throws JMSException {
		logger.debug("MessageProducer#getTimeToLive()");
		return mp.getTimeToLive();
	}

	/** 
	 * Sends a message using the MessageProducer's default delivery mode, 
	 * priority, and time to live.
	 * @param message the message to send
	 * @throws JMSException if the JMS provider fails to send the message due 
	 * to some internal error.
	 * @throws MessageFormatException if an invalid message is specified.
	 * @throws InvalidDestinationException if a client uses this method with a 
	 * MessageProducer with an invalid destination.
	 * @throws UnsupportedOperationException - if a client uses this method with 
	 * a MessageProducer that did not specify a destination at creation time.
	 */
	public void send(Message message) throws JMSException {
		logger.debug("MessageProducer#send("+message+")");
        mp.send(unwrapDestinations(message));
	}

	/** 
	 * Sends a message to the destination, specifying delivery mode, priority, 
	 * and time to live. 
	 * @param message the message to send
	 * @param deliveryMode the delivery mode to use
	 * @param priority the priority for this message
	 * @param timeToLive the message's lifetime (in milliseconds)
	 * @throws JMSException if the JMS provider fails to send the message due to 
	 * some internal error.
	 * @throws MessageFormatException if an invalid message is specified.
	 * @throws InvalidDestinationException if a client uses this method with a 
	 * MessageProducer with an invalid destination.
	 * @throws UnsupportedOperationException if a client uses this method with a 
	 * MessageProducer that did not specify a destination at creation time.
	 */
	public void send(Message message, int deliveryMode, int priority, 
			long timeToLive) throws JMSException {
		logger.debug("MessageProducer#send("+message+","+deliveryMode+","+
				priority+","+timeToLive+")");
        mp.send(unwrapDestinations(message), deliveryMode, priority, timeToLive);
	}

	/** 
	 * Sends a message to a destination for an unidentified message producer. 
	 * Uses the MessageProducer's default delivery mode, priority, and time to 
	 * live.
	 * @param destination the destination to send this message to
	 * @param message the message to send
	 * @throws JMSException if the JMS provider fails to send the message due to 
	 * some internal error.
	 * @throws MessageFormatException if an invalid message is specified.
	 * @throws InvalidDestinationException if a client uses this method with an 
	 * invalid destination.
	 * @throws UnsupportedOperationException if a client uses this method with a 
	 * MessageProducer that specified a destination at creation time.
	 */
	public void send(Destination destination, Message message) 
			throws JMSException {
		logger.debug("MessageProducer#send("+destination+","+message+")");
        mp.send(unwrapDestinations(destination), unwrapDestinations(message));
		
	}

	/** 
	 * Sends a message to a destination for an unidentified message producer, 
	 * specifying delivery mode, priority and time to live.
	 * @param destination the destination to send this message to
	 * @param message the message to send
	 * @param deliveryMode the delivery mode to use
	 * @param priority the priority for this message
	 * @param timeToLive the message's lifetime (in milliseconds)
	 * @throws JMSException if the JMS provider fails to send the message due to 
	 * some internal error.
	 * @throws MessageFormatException if an invalid message is specified.
	 * @throws InvalidDestinationException - if a client uses this method with 
	 * an invalid destination.
	 */
	public void send(Destination destination, Message message, int deliveryMode,
			int priority, long timeToLive) throws JMSException {
		logger.debug("MessageProducer#send("+destination+","+message+","+
				deliveryMode+","+priority+","+timeToLive+")");
        mp.send(unwrapDestinations(destination), unwrapDestinations(message), 
        		deliveryMode, priority, timeToLive);
	}

	/** 
	 * Sends a message to a queue for an unidentified message producer. Uses 
	 * the MessageProducer's default delivery mode, priority, and time to live. 
	 * @param the queue to send this message to
	 * @param message the message to send
	 * @throws JMSException if the JMS provider fails to send the message due to 
	 * some internal error.
	 * @throws MessageFormatException if an invalid message is specified.
	 * @throws InvalidDestinationException if a client uses this method with an 
	 * invalid queue.
	 */
	public void send(Queue queue, Message message) throws JMSException {
		logger.debug("MessageProducer#send("+queue+","+message+")");
        mp.send(unwrapDestinations(queue), unwrapDestinations(message));
	}

	/** 
	 * Sends a message to a queue for an unidentified message producer, 
	 * specifying delivery mode, priority and time to live.
	 * @param queue the queue to send this message to
	 * @param message the message to send
	 * @param deliveryMode the delivery mode to use
	 * @param priority the priority for this message
	 * @param timeToLive the message's lifetime (in milliseconds)
	 * @throws JMSException if the JMS provider fails to send the message due to 
	 * some internal error.
	 * @throws MessageFormatException - if an invalid message is specified.
	 * @throws InvalidDestinationException - if a client uses this method with 
	 * an invalid queue.
	 */
	public void send(Queue queue, Message message, int deliveryMode, 
			int priority, long timeToLive) throws JMSException {
		logger.debug("MessageProducer#send("+queue+","+message+","+deliveryMode+
				","+priority+","+timeToLive+")");
        mp.send(unwrapDestinations(queue), unwrapDestinations(message), 
        		deliveryMode, priority, timeToLive);
	}

	/** 
	 * Publishes a message to the topic. Uses the MessageProducer's default 
	 * delivery mode, priority, and time to live.
	 * @param message the message to publish
	 * @throws JMSException if the JMS provider fails to publish the message due 
	 * to some internal error.
	 * @throws MessageFormatException if an invalid message is specified.
	 * @throws InvalidDestinationException if a client uses this method with a 
	 * MessageProducer with an invalid topic.
	 * @throws UnsupportedOperationException if a client uses this method with a 
	 * MessageProducer that did not specify a topic at creation time
	 */
	public void publish(Message message) throws JMSException {
		logger.debug("MessageProducer#publish("+message+")");
        ((TopicPublisher)mp).publish(unwrapDestinations(message));
	}

	/** 
	 * Publishes a message to the topic, specifying delivery mode, priority, 
	 * and time to live.
	 * @param message the message to publish
	 * @param deliveryMode the delivery mode to use
	 * @param priority the priority for this message
	 * @param timeToLive the message's lifetime (in milliseconds)
	 * @throws JMSException if the JMS provider fails to publish the message due
	 * to some internal error.
	 * @throws MessageFormatException if an invalid message is specified.
	 * @throws InvalidDestinationException if a client uses this method with a 
	 * MessageProducer with an invalid topic.
	 * @throws UnsupportedOperationException if a client uses this method with a
	 * MessageProducer that did not specify a topic at creation time.
	 */
	public void publish(Message message, int deliveryMode, int priority, 
			long timeToLive) throws JMSException {
		logger.debug("MessageProducer#publish("+message+","+deliveryMode+","+
				priority+","+timeToLive+")");
        ((TopicPublisher)mp).publish(unwrapDestinations(message), deliveryMode, 
        		priority, timeToLive);
	}

	/** 
	 * Publishes a message to a topic for an unidentified message producer. Uses
	 * the MessageProducer's default delivery mode, priority, and time to live.
	 * @param topic the topic to publish this message to
	 * @param message the message to publish
	 * @throws JMSException if the JMS provider fails to publish the message due 
	 * to some internal error.
	 * @throws MessageFormatException if an invalid message is specified.
	 * @throws InvalidDestinationException if a client uses this method with an 
	 * invalid topic.
	 */
	public void publish(Topic topic, Message message) throws JMSException {
		logger.debug("MessageProducer#publish("+topic+","+message+")");
        ((TopicPublisher)mp).publish(unwrapDestinations(topic), 
        		unwrapDestinations(message));
	}

	/** 
	 * Publishes a message to a topic for an unidentified message producer, 
	 * specifying delivery mode, priority and time to live.
	 * @param topic the topic to publish this message to
	 * @param message the message to publish
	 * @param deliveryMode the delivery mode to use
	 * @param priority the priority for this message
	 * @param timeToLive the message's lifetime (in milliseconds)
	 * @throws JMSException if the JMS provider fails to publish the message due 
	 * to some internal error.
	 *  @throws MessageFormatException if an invalid message is specified.
	 *  @throws InvalidDestinationException - if a client uses this method with 
	 *  an invalid topic.
	 */
	public void publish(Topic topic, Message message, int deliveryMode, 
			int priority, long timeToLive) throws JMSException {
		logger.debug("MessageProducer#publish("+topic+","+message+","+
				deliveryMode+","+priority+","+timeToLive+")");
        ((TopicPublisher)mp).publish(unwrapDestinations(topic), 
        		unwrapDestinations(message), deliveryMode, priority, timeToLive);
	}

	/** 
	 * Closes the message producer.
	 * @throws JMSException if the JMS provider fails to close the producer due 
	 * to some internal error.
	 */
	public void close() throws JMSException {
		logger.debug("MessageProducer#close()");
		mp.close();
	}

    /**
     * This method unwrap a message
     * @param msg the message to unwrap
     * @return the unwrapped message
     * @throws JMSException exception occure if if operation failed
     */
    private Message unwrapDestinations(Message msg) throws JMSException {
		logger.debug("MessageProducerProxy#unwrapDestinations("+msg+")");
        Destination jmsReply = msg.getJMSReplyTo();
        if (jmsReply != null) {
            msg.setJMSReplyTo(unwrapDestinations(jmsReply));
        }

        Destination jmsDest = msg.getJMSDestination();
        if (jmsDest != null) {
            msg.setJMSDestination(unwrapDestinations(jmsDest));
        }
        return msg;
   }

    /**
     * This method unwrap a destination
     * @param dest the destination to unwrap
     * @return the unwrapped destination
     * @throws JMSException exception occure if if operation failed
     */
    private Destination unwrapDestinations(Destination dest) throws JMSException{
		logger.debug("MessageProducerProxy#unwrapDestinations("+dest+")");
    	return ((dest instanceof DestinationAdapter) 
    			|| (dest instanceof QueueProxy) || (dest instanceof TopicProxy)) 
    			? ((DestinationAdapter)dest).getPhysicalDestination()
    			: dest;
    }

    /**
     * This method unwrap a queue
     * @param queue the queue to unwrap
     * @return the unwrapped queue
     * @throws JMSException exception occure if if operation failed
     */
    private Queue unwrapDestinations(Queue queue) throws JMSException {
		logger.debug("MessageProducerProxy#unwrapDestinations("+queue+")");
    	return (queue instanceof QueueProxy) ?
    			(Queue)((DestinationAdapter) queue).getPhysicalDestination() :
    			queue;
    }

    /**
     * This method unwrap a topic
     * @param topic the topic to unwrap
     * @return the unwrapped topic
     * @throws JMSException exception occure if if operation failed
     */
    private Topic unwrapDestinations(Topic topic) throws JMSException {
		logger.debug("MessageProducerProxy#unwrapDestinations("+topic+")");
    	return (topic instanceof TopicProxy) ?
    			(Topic)((DestinationAdapter)topic).getPhysicalDestination() : 
    			topic;
    }
}