/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SessionAdapter.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

import javax.jms.BytesMessage;
import javax.jms.Destination;
import javax.jms.IllegalStateException;
import javax.jms.InvalidDestinationException;
import javax.jms.InvalidSelectorException;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.StreamMessage;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.jms.TransactionRolledBackException;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * Session Adapter of the connector.
 * 
 * @author Yacine TOUIL
 */
public class SessionAdapter implements Session, TopicSession, QueueSession {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * The <code>OutboundConnection</code> the session belongs to.
	 */
	private ConnectionHandle ch;

	/**
	 * The JMS session.
	 */
	private Session session;

	/**
	 * The swrapped JMS session.
	 */
	private Session swappedSession;

	/**
	 * <code>true</code> if this "handle" is valid.
	 */
	private boolean valid = true;

	/**
	 * <code>true</code> if the session is started.
	 */
	private boolean started = true;

	/**
	 * <code>true</code> if the session is started.
	 */
	protected boolean transacted;

	/**
	 * <code>true</code> if the session is used.
	 */
	private boolean inUse = false;

	/**
	 * List of message consumer associated with this session.
	 */
	private ArrayList messageConsumers = new ArrayList();

	/**
	 * List of message producer associated with this session.
	 */
	private ArrayList messageProducers = new ArrayList();

	/**
	 * List of queue browser associated with this session.
	 */
	private ArrayList queueBrowsers = new ArrayList();

	/**
	 * Constructor of the class
	 * 
	 * @param session
	 *            the session
	 * @param connHandle
	 *            the connection handle
	 */
	public SessionAdapter(Session session, ConnectionHandle connHandle) {
		logger.debug("SessionAdapter#SessionAdapter(" + session + ","
				+ connHandle + ")");
		this.session = session;
		this.ch = connHandle;
	}

	/**
	 * Indicates whether the session is in transacted mode.
	 * 
	 * @return true if the session is in transacted mode; false otherwise
	 * @throws JMSException
	 *             if the JMS provider fails to return the transaction mode due
	 *             to some internal error.
	 */
	public boolean getTransacted() throws JMSException {
		logger.debug("SessionAdapter#getTransacted()");
		checkIfValid();
		return session.getTransacted();
	}

	/**
	 * Returns the acknowledgement mode of the session. The acknowledgement mode
	 * is set at the time that the session is created. If the session is
	 * transacted, the acknowledgement mode is ignored.
	 * 
	 * @return If the session is not transacted, returns the current
	 *         acknowledgement mode for the session. If the session is
	 *         transacted, returns SESSION_TRANSACTED.
	 * @throws JMSException
	 *             if the JMS provider fails to return the acknowledgment mode
	 *             due to some internal error.
	 */
	public int getAcknowledgeMode() throws JMSException {
		logger.debug("SessionAdapter#getAcknowledgeMode()");
		checkIfValid();
		return session.getAcknowledgeMode();
	}

	/**
	 * Indicates whether the session is in use.
	 * 
	 * @return true if the session is in use; false otherwise
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	public boolean isInUse() {
		logger.debug("SessionAdapter#isInUse()");
		return inUse;
	}

	/**
	 * Indicates whether the session is started.
	 * 
	 * @return true if the session is started; false otherwise
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	public boolean isStarted() {
		logger.debug("SessionAdapter#isStarted()");
		return started;
	}

	/**
	 * Invalidate the session adapter
	 */
	public void invalidate() {
		logger.debug("SessionAdapter#invalidate()");
		valid = false;
	}

	/**
	 * Returns the session's distinguished message listener
	 * 
	 * @return the message listener associated with this session
	 * @throws JMSException
	 *             if the JMS provider fails to get the message listener due to
	 *             an internal error.
	 */
	public MessageListener getMessageListener() throws JMSException {
		logger.debug("SessionAdapter#getMessageListener()");
		checkIfValid();
		return session.getMessageListener();
	}

	/**
	 * Sets the session's distinguished message listener
	 * 
	 * @param listener
	 *            the message listener to associate with this session
	 * @throws JMSException
	 *             if the JMS provider fails to set the message listener due to
	 *             an internal error.
	 */
	public void setMessageListener(MessageListener listener)
			throws JMSException {
		logger.debug("SessionAdapter#setMessageListener(" + listener + ")");
		checkIfValid();
		session.setMessageListener(listener);
	}

	/**
	 * Unsubscribes a durable subscription that has been created by a client.
	 * 
	 * @param name
	 *            the name used to identify this subscription
	 * @throws JMSException
	 *             if the session fails to unsubscribe to the durable
	 *             subscription due to some internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid subscription name is specified.
	 */
	public void unsubscribe(String name) throws JMSException {
		logger.debug("SessionAdapter#unsubscribe(" + name + ")");
		checkIfValid();
		session.unsubscribe(name);
	}

	/**
	 * Creates a BytesMessage object. A BytesMessage object is used to send a
	 * message containing a stream of uninterpreted bytes.
	 * 
	 * @return the BytesMessage object
	 * @throws JMSException
	 *             the JMS provider fails to create this message due to some
	 *             internal error.
	 */
	public BytesMessage createBytesMessage() throws JMSException {
		logger.debug("XARessourceImpl#createBytesMessage()");
		checkIfValid();
		return session.createBytesMessage();
	}

	/**
	 * Creates a MapMessage object. A MapMessage object is used to send a
	 * self-defining set of name-value pairs, where names are String objects and
	 * values are primitive values in the Java programming language.
	 * 
	 * @return the MapMessage object
	 * @throws JMSException
	 *             if the JMS provider fails to create this message due to some
	 *             internal error.
	 */
	public MapMessage createMapMessage() throws JMSException {
		logger.debug("SessionAdapter#createMapMessage()");
		checkIfValid();
		return session.createMapMessage();
	}

	/**
	 * Creates a Message object. The Message interface is the root interface of
	 * all JMS messages. A Message object holds all the standard message header
	 * information. It can be sent when a message containing only header
	 * information is sufficient.
	 * 
	 * @return the Message object
	 * @throws JMSException
	 *             if the JMS provider fails to create this message due to some
	 *             internal error.
	 */
	public Message createMessage() throws JMSException {
		logger.debug("SessionAdapter#createMessage()");
		checkIfValid();
		return session.createMessage();
	}

	/**
	 * Creates an ObjectMessage object. An ObjectMessage object is used to send
	 * a message that contains a serializable Java object.
	 * 
	 * @return the ObjectMessage object
	 * @throws JMSException
	 *             if the JMS provider fails to create this message due to some
	 *             internal error.
	 */
	public ObjectMessage createObjectMessage() throws JMSException {
		logger.debug("SessionAdapter#createObjectMessage()");
		checkIfValid();
		return session.createObjectMessage();
	}

	/**
	 * Creates an initialized ObjectMessage object. An ObjectMessage object is
	 * used to send a message that contains a serializable Java object.
	 * 
	 * @param object
	 * @return the initialized ObjectMessage object
	 * @throws JMSException
	 *             if the JMS provider fails to create this message due to some
	 *             internal error.
	 */
	public ObjectMessage createObjectMessage(Serializable object)
			throws JMSException {
		logger.debug("SessionAdapter#createObjectMessage(" + object + ")");
		checkIfValid();
		return session.createObjectMessage(object);
	}

	/**
	 * Creates a StreamMessage object. A StreamMessage object is used to send a
	 * self-defining stream of primitive values in the Java programming
	 * language.
	 * 
	 * @return the StreamMessage object
	 * @throws JMSException
	 *             if the JMS provider fails to create this message due to some
	 *             internal error.
	 */
	public StreamMessage createStreamMessage() throws JMSException {
		logger.debug("SessionAdapter#createStreamMessage()");
		checkIfValid();
		return session.createStreamMessage();
	}

	/**
	 * Creates a TextMessage object. A TextMessage object is used to send a
	 * message containing a String object.
	 * 
	 * @return the TextMessage object
	 * @throws JMSException
	 *             if the JMS provider fails to create this message due to some
	 *             internal error.
	 */
	public TextMessage createTextMessage() throws JMSException {
		logger.debug("SessionAdapter#createTextMessage()");
		checkIfValid();
		return session.createTextMessage();
	}

	/**
	 * Creates an initialized TextMessage object. A TextMessage object is used
	 * to send a message containing a String.
	 * 
	 * @param text
	 *            the string used to initialize this message
	 * @return the TextMessage object
	 * @throws JMSException
	 *             if the JMS provider fails to create this message due to some
	 *             internal error.
	 */
	public TextMessage createTextMessage(String text) throws JMSException {
		logger.debug("SessionAdapter#createTextMessage(" + text + ")");
		checkIfValid();
		return session.createTextMessage(text);
	}

	/**
	 * Creates a MessageProducer to send messages to the specified destination.
	 * 
	 * @param destination
	 *            the Destination to send to, or null if this is a producer
	 *            which does not have a specified destination.
	 * @return the MessageProducer
	 * @throws JMSException
	 *             if the session fails to create a MessageProducer due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid destination is specified.
	 */
	public MessageProducer createProducer(Destination destination)
			throws JMSException {
		logger.debug("SessionAdapter#createProducer(" + destination + ")");
		checkIfValid();
		MessageProducer mp = this.session
				.createProducer(getWrappedDestination(destination));
		MessageProducer wrappedMp = new MessageProducerProxy(mp);
		this.messageProducers.add(wrappedMp);
		return wrappedMp;
	}

	/**
	 * Creates a MessageConsumer for the specified destination. Since Queue and
	 * Topic both inherit from Destination, they can be used in the destination
	 * parameter to create a MessageConsumer.
	 * 
	 * @param destination
	 *            the Destination to access.
	 * @return the MessageConsumer
	 * @throws JMSException
	 *             if the session fails to create a consumer due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid destination is specified.
	 */
	public MessageConsumer createConsumer(Destination destination)
			throws JMSException {
		logger.debug("SessionAdapter#createConsumer(" + destination + ")");
		checkIfValid();
		MessageConsumer mc = session
				.createConsumer(getWrappedDestination(destination));
		messageConsumers.add(mc);
		return mc;
	}

	/**
	 * Creates a MessageConsumer for the specified destination, using a message
	 * selector. Since Queue and Topic both inherit from Destination, they can
	 * be used in the destination parameter to create a MessageConsumer.
	 * 
	 * @param destination
	 *            the Destination to access
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @return the MessageConsumer
	 * @throws JMSException
	 *             if the session fails to create a MessageConsumer due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid destination is specified.
	 * @throws InvalidSelectorException -
	 *             if the message selector is invalid.
	 * 
	 */
	public MessageConsumer createConsumer(Destination destination,
			String messageSelector) throws JMSException {
		logger.debug("SessionAdapter#createConsumer(" + destination + ","
				+ messageSelector + ")");
		checkIfValid();
		MessageConsumer mc = session.createConsumer(
				getWrappedDestination(destination), messageSelector);
		messageConsumers.add(mc);
		return mc;
	}

	/**
	 * Creates MessageConsumer for the specified destination, using a message
	 * selector. This method can specify whether messages published by its own
	 * connection should be delivered to it, if the destination is a topic.
	 * 
	 * @param destination
	 *            the Destination to access
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @param noLocal
	 *            if true, and the destination is a topic, inhibits the delivery
	 *            of messages published by its own connection. The behavior for
	 *            NoLocal is not specified if the destination is a queue.
	 * @return the MessageConsumer
	 * @throws JMSException
	 *             if the session fails to create a MessageConsumer due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid destination is specified.
	 * @throws InvalidSelectorException -
	 *             if the message selector is invalid.
	 */
	public MessageConsumer createConsumer(Destination destination,
			String messageSelector, boolean noLocal) throws JMSException {
		logger.debug("SessionAdapter#createConsumer(" + destination + ","
				+ messageSelector + "," + noLocal + ")");
		checkIfValid();
		MessageConsumer mc = session.createConsumer(
				getWrappedDestination(destination), messageSelector, noLocal);
		messageConsumers.add(mc);
		return mc;
	}

	/**
	 * Creates a queue identity given a Queue name.
	 * 
	 * @param queueName
	 *            the name of this Queue
	 * @return a Queue with the given name
	 * @throws JMSException
	 *             if the session fails to create a queue due to some internal
	 *             error.
	 */
	public Queue createQueue(String queueName) throws JMSException {
		logger.debug("SessionAdapter#createQueue(" + queueName + ")");
		checkIfValid();
		return session.createQueue(queueName);
	}

	/**
	 * Creates a topic identity given a Topic name.
	 * 
	 * @param topicName
	 *            the name of this Topic
	 * @return a Topic with the given name
	 * @throws JMSException
	 *             if the session fails to create a topic due to some internal
	 *             error.
	 */
	public Topic createTopic(String topicName) throws JMSException {
		logger.debug("SessionAdapter#createTopic(" + topicName + ")");
		checkIfValid();
		return session.createTopic(topicName);
	}

	/**
	 * Creates a durable subscriber to the specified topic.
	 * 
	 * @param topic
	 *            the non-temporary Topic to subscribe to
	 * @param name
	 *            the name used to identify this subscription
	 * @return the TopicSubscriber
	 * @throws JMSException
	 *             if the session fails to create a subscriber due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid topic is specified.
	 */
	public TopicSubscriber createDurableSubscriber(Topic topic, String name)
			throws JMSException {
		logger.debug("SessionAdapter#createDurableSubscriber(" + topic + ","
				+ name + ")");
		checkIfValid();
		TopicSubscriber ts = session.createDurableSubscriber(
				getWrappedTopic(topic), name);
		messageConsumers.add(ts);
		return ts;
	}

	/**
	 * Creates a durable subscriber to the specified topic, using a message
	 * selector and specifying whether messages published by its own connection
	 * should be delivered to it.
	 * 
	 * @param topic
	 *            the non-temporary Topic to subscribe to
	 * @param name
	 *            the name used to identify this subscription
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @param noLocal
	 *            if set, inhibits the delivery of messages published by its own
	 *            connection
	 * @return the durable subscriber TopicSubscriber
	 * @throws JMSException
	 *             if the session fails to create a subscriber due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid topic is specified.
	 * @throws InvalidSelectorException -
	 *             if the message selector is invalid.
	 */
	public TopicSubscriber createDurableSubscriber(Topic topic, String name,
			String messageSelector, boolean noLocal) throws JMSException {
		logger.debug("SessionAdapter#createDurableSubscriber(" + topic + ","
				+ name + "," + messageSelector + "," + noLocal + ")");
		checkIfValid();
		TopicSubscriber ts = session.createDurableSubscriber(
				getWrappedTopic(topic), name, messageSelector, noLocal);
		this.messageConsumers.add(ts);
		return ts;
	}

	/**
	 * Creates a QueueBrowser object to peek at the messages on the specified
	 * queue.
	 * 
	 * @param queue
	 *            the queue to access
	 * @return the QueueBrowser
	 * @throws JMSException
	 *             if the session fails to create a browser due to some internal
	 *             error.
	 * @throws InvalidDestinationException -
	 *             if an invalid destination is specified
	 */
	public QueueBrowser createBrowser(Queue queue) throws JMSException {
		logger.debug("SessionAdapter#createBrowser(" + queue + ")");
		checkIfValid();
		QueueBrowser qb = session.createBrowser(getWrappedQueue(queue));
		queueBrowsers.add(qb);
		return qb;
	}

	/**
	 * Creates a QueueBrowser object to peek at the messages on the specified
	 * queue using a message selector.
	 * 
	 * @param queue
	 *            the queue to access
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @return the QueueBrowser
	 * @throws JMSException
	 *             if the session fails to create a browser due to some internal
	 *             error.
	 * @throws InvalidDestinationException -
	 *             if an invalid destination is specified
	 * @throws InvalidSelectorException -
	 *             if the message selector is invalid.
	 */
	public QueueBrowser createBrowser(Queue queue, String messageSelector)
			throws JMSException {
		logger.debug("SessionAdapter#createBrowser(" + queue + ","
				+ messageSelector + ")");
		checkIfValid();
		QueueBrowser qb = session.createBrowser(getWrappedQueue(queue),
				messageSelector);
		queueBrowsers.add(qb);
		return qb;
	}

	/**
	 * Creates a TemporaryQueue object. Its lifetime will be that of the
	 * Connection unless it is deleted earlier.
	 * 
	 * @return a temporary queue identity
	 * @throws JMSException
	 *             if the session fails to create a temporary queue due to some
	 *             internal error.
	 */
	public TemporaryQueue createTemporaryQueue() throws JMSException {
		logger.debug("SessionAdapter#createTemporaryQueue()");
		checkIfValid();
		TemporaryQueue q = session.createTemporaryQueue();
		ch.addTemporaryDest(q);
		return q;
	}

	/**
	 * Creates a TemporaryTopic object. Its lifetime will be that of the
	 * Connection unless it is deleted earlier.
	 * 
	 * @return a temporary topic identity
	 * @throws JMSException
	 *             if the session fails to create a temporary topic due to some
	 *             internal error.
	 */
	public TemporaryTopic createTemporaryTopic() throws JMSException {
		logger.debug("SessionAdapter#createTemporaryTopic()");
		checkIfValid();
		TemporaryTopic t = session.createTemporaryTopic();
		ch.addTemporaryDest(t);
		return t;
	}

	/**
	 * Creates a nondurable subscriber to the specified topic.
	 * 
	 * @param topic
	 *            the Topic to subscribe to
	 * @return the TopicSubscriber
	 * @throws JMSException
	 *             if the session fails to create a subscriber due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid topic is specified.
	 */
	public TopicSubscriber createSubscriber(Topic topic) throws JMSException {
		logger.debug("SessionAdapter#createSubscriber(" + topic + ")");
		checkIfValid();
		TopicSubscriber ts = ((TopicSession) session)
				.createSubscriber(getWrappedTopic(topic));
		messageConsumers.add(ts);
		return ts;
	}

	/**
	 * Creates a nondurable subscriber to the specified topic, using a message
	 * selector or specifying whether messages published by its own connection
	 * should be delivered to it.
	 * 
	 * @param topic
	 *            the Topic to subscribe to
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @param noLocal
	 *            if set, inhibits the delivery of messages published by its own
	 *            connection
	 * @return the nondurable subscriber TopicSubscriber
	 * @throws JMSException
	 *             if the session fails to create a subscriber due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid topic is specified.
	 * @throws InvalidSelectorException -
	 *             if the message selector is invalid.
	 */
	public TopicSubscriber createSubscriber(Topic topic,
			String messageSelector, boolean noLocal) throws JMSException {
		logger.debug("SessionAdapter#createSubscriber(" + topic + ","
				+ messageSelector + "," + noLocal + ")");
		checkIfValid();
		TopicSubscriber ts = ((TopicSession) session).createSubscriber(
				getWrappedTopic(topic), messageSelector, noLocal);
		messageConsumers.add(ts);
		return ts;
	}

	/**
	 * Creates a publisher for the specified topic.
	 * 
	 * @param topic
	 *            the Topic to publish to, or null if this is an unidentified
	 *            producer
	 * @return the publisher
	 * @throws JMSException
	 *             if the session fails to create a publisher due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid topic is specified.
	 */
	public TopicPublisher createPublisher(Topic topic) throws JMSException {
		logger.debug("SessionAdapter#createPublisher(" + topic + ")");
		checkIfValid();
		TopicPublisher tp = ((TopicSession) session)
				.createPublisher(getWrappedTopic(topic));
		TopicPublisher wrappedTp = new MessageProducerProxy(tp);
		messageProducers.add(wrappedTp);
		return wrappedTp;
	}

	/**
	 * Creates a QueueReceiver object to receive messages from the specified
	 * queue.
	 * 
	 * @param queue
	 *            the Queue to access
	 * @return
	 * @throws JMSException
	 *             if the session fails to create a receiver due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid queue is specified.
	 */
	public QueueReceiver createReceiver(Queue queue) throws JMSException {
		logger.debug("SessionAdapter#starcreateReceivert(" + queue + ")");
		checkIfValid();
		QueueReceiver qr = ((QueueSession) session)
				.createReceiver(getWrappedQueue(queue));
		messageConsumers.add(qr);
		return qr;
	}

	/**
	 * Creates a QueueReceiver object to receive messages from the specified
	 * queue using a message selector.
	 * 
	 * @param queue
	 *            the Queue to access
	 * @param messageSelector
	 *            only messages with properties matching the message selector
	 *            expression are delivered. A value of null or an empty string
	 *            indicates that there is no message selector for the message
	 *            consumer.
	 * @return QueueReceiver object
	 * @throws JMSException
	 *             if the session fails to create a receiver due to some
	 *             internal error.
	 * @throws InvalidDestinationException -
	 *             if an invalid queue is specified.
	 * @throws InvalidSelectorException -
	 *             if the message selector is invalid.
	 */
	public QueueReceiver createReceiver(Queue queue, String messageSelector)
			throws JMSException {
		logger.debug("SessionAdapter#createReceiver(" + queue + ","
				+ messageSelector + ")");
		checkIfValid();
		QueueReceiver qr = ((QueueSession) session).createReceiver(
				getWrappedQueue(queue), messageSelector);
		messageConsumers.add(qr);
		return qr;
	}

	/**
	 * Creates a QueueSender object to send messages to the specified queue.
	 * 
	 * @param queue
	 *            the Queue to access, or null if this is an unidentified
	 *            producer
	 * @return the QueueSender object
	 * @throws JMSException
	 *             if the session fails to create a sender due to some internal
	 *             error.
	 * @throws InvalidDestinationException -
	 *             if an invalid queue is specified.
	 */
	public QueueSender createSender(Queue queue) throws JMSException {
		logger.debug("SessionAdapter#createSender(" + queue + ")");
		checkIfValid();
		QueueSender qs = ((QueueSession) session)
				.createSender(getWrappedQueue(queue));
		QueueSender wrappedQs = new MessageProducerProxy(qs);
		messageProducers.add(wrappedQs);
		return wrappedQs;
	}

	/**
	 * Commits all messages done in this transaction and releases any locks
	 * currently held.
	 * 
	 * @throws JMSException
	 *             if the JMS provider fails to commit the transaction due to
	 *             some internal error.
	 * @throws TransactionRolledBackException -
	 *             if the transaction is rolled back due to some internal error
	 *             during commit.
	 * @throws IllegalStateException -
	 *             if the method is not called by a transacted session.
	 */
	public void commit() throws JMSException {
		logger.debug("SessionAdapter#commit()");
		checkIfValid();
		session.commit();
	}

	/**
	 * Rolls back any messages done in this transaction and releases any locks
	 * currently held.
	 * 
	 * @throws JMSException
	 *             if the JMS provider fails to roll back the transaction due to
	 *             some internal error.
	 * @throws IllegalStateException -
	 *             if the method is not called by a transacted session.
	 */
	public void rollback() throws JMSException {
		logger.debug("SessionAdapter#rollback()");
		checkIfValid();
		session.rollback();
	}

	/**
	 * Stops message delivery in this session, and restarts message delivery
	 * with the oldest unacknowledged message.
	 * 
	 * @throws JMSException
	 *             if the JMS provider fails to stop and restart message
	 *             delivery due to some internal error.
	 * @throws IllegalStateException -
	 *             if the method is called by a transacted session.
	 */
	public void recover() throws JMSException {
		logger.debug("SessionAdapter#recover()");
		checkIfValid();
		session.recover();
	}

	/**
	 * Intended to be used only by Application Servers, not by ordinary JMS
	 * clients.
	 */
	public void run() {
		logger.debug("SessionAdapter#listener()");
		try {
			checkIfValid();
		} catch (JMSException e) {
			throw new RuntimeException(e);
		}
		session.run();
	}

	/**
	 * Closes the session.
	 * 
	 * @throws JMSException
	 *             if the JMS provider fails to close the session due to some
	 *             internal error.
	 */
	public void close() throws JMSException {
		logger.debug("SessionAdapter#close()");
		if (!this.started)
			return;
		this.started = false;
		this.inUse = false;

		// TODO test if we dont make a closeAllJMSEntities(); if it is better !
		closeAllJMSEntities();
		ch.getManagedConnection().closeSession(session);
	}

	/**
	 * Check if the this object is started and valid
	 * 
	 * @throws JMSException
	 */
	private void checkIfValid() throws JMSException {
		logger.debug("SessionAdapter#checkIfValid()");
		if (!started) {
			logger.info("SessionAdapter#checkIfValid() : "
					+ "Connection is closed");
			throw new IllegalStateException("Connection is closed");
		}
		if (!valid) {
			logger.info("SessionAdapter#checkIfValid() : "
					+ "Invalid connection");
			throw new IllegalStateException("Invalid connection");
		}
	}

	/**
	 * Close all entities created via this session
	 * 
	 * @throws JMSException
	 */
	private void closeAllJMSEntities() throws JMSException {
		logger.debug("SessionAdapter#closeAllJMSEntities()");
		logger.debug("SessionAdapter#closeAllJMSEntities() : closing message "
				+ "consumers " + messageConsumers.size());
		for (Iterator i = messageConsumers.iterator(); i.hasNext();) {
			MessageConsumer mc = (MessageConsumer) i.next();
			logger.debug("SessionAdapter#closeAllJMSEntities() : closing "
					+ "message consumer " + mc);
			mc.close();
		}
		logger.debug("SessionAdapter#closeAllJMSEntities() : closing message "
				+ "producers " + messageProducers.size());
		for (Iterator i = messageProducers.iterator(); i.hasNext();) {
			MessageProducer mp = (MessageProducer) i.next();
			logger.debug("SessionAdapter#closeAllJMSEntities() : closing "
					+ "message producer " + mp);
			mp.close();
		}
		logger.debug("SessionAdapter#closeAllJMSEntities() : closing queue "
				+ "browsers " + queueBrowsers.size());
		for (Iterator i = queueBrowsers.iterator(); i.hasNext();) {
			QueueBrowser qp = (QueueBrowser) i.next();
			logger.debug("SessionAdapter#closeAllJMSEntities() : closing "
					+ "Queue browser " + qp);
			qp.close();
		}

		session.setMessageListener(null);
		messageConsumers.clear();
		messageProducers.clear();
		queueBrowsers.clear();
		ch.getSessions().remove(this);
	}

	/**
	 * get a wrapped destination
	 * 
	 * @param dest
	 *            a destination
	 * @return a wrapped destination
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	private Destination getWrappedDestination(Destination dest)
			throws JMSException {
		logger.debug("SessionAdapter#getWrappedDestination(" + dest + ")");
		return (dest instanceof DestinationAdapter) ? (Destination) (((DestinationAdapter) dest)
				.getPhysicalDestination())
				: dest;
	}

	/**
	 * get a wrapped topic
	 * 
	 * @param dest
	 *            a topic
	 * @return a wrapped topic
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	private Topic getWrappedTopic(Topic topic) throws JMSException {
		logger.debug("SessionAdapter#getWrappedTopic(" + topic + ")");
		return (topic instanceof TopicProxy) ? (Topic) (((TopicProxy) topic)
				.getPhysicalDestination()) : topic;
	}

	/**
	 * get a wrapped queue
	 * 
	 * @param dest
	 *            a queue
	 * @return a wrapped queue
	 * @throws JMSException
	 *             exception ocure if the operation fail
	 */
	private Queue getWrappedQueue(Queue queue) throws JMSException {
		logger.debug("SessionAdapter#getWrappedQueue(" + queue + ")");
		return (queue instanceof QueueProxy) ? (Queue) (((QueueProxy) queue)
				.getPhysicalDestination()) : queue;
	}

	/**
	 * used to swap the current session when we bascule from non transactional
	 * to trasactional mode
	 * 
	 * @param session
	 *            the session in transactional mode
	 */
	void swapPhysicalSession(Session session) {
		logger.debug("SessionAdapter#_swapPhysicalSession(" + session + ")");
		if (session != this.session) {
			swappedSession = this.session;
			this.session = session;
		}
	}

	/**
	 * used to restore the swaped session when we bascule from non transactional
	 * to trasactional mode
	 */
	void swapPhysicalSession() {
		logger.debug("SessionAdapter#_swapPhysicalSession()");
		if (swappedSession != null) {
			session = swappedSession;
			swappedSession = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "XAResourceImpl: " + valid + " " + started + " " + transacted
				+ " " + inUse;
	}

}