/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: XAResourceImpl.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import javax.jms.Connection;
import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * <code>XAResource</code> wrapper for Jonas MQ Connector. This class
 * intercepts all calls to the actual XAResource object of the physical JMS
 * connection and performs corresponding book-keeping tasks in the
 * ManagedConnection representing the physical connection.
 * 
 * @author Yacine TOUIL
 */
public class XAResourceImpl implements XAResource {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * the managed connection
	 */
	private ManagedConnectionImpl mc;

	/**
	 * The conneciton
	 */
	private Connection connection;

	/**
	 * Constructor for XAResourceImpl
	 * 
	 * @param mc
	 *            <code>ManagedConnection</code>
	 */
	public XAResourceImpl(ManagedConnectionImpl mc) {
		logger.debug("XARessourceImpl#start(" + mc + ")");
		this.mc = mc;
	}

	/**
	 * Set the physical jms connection object associated with this XAResource
	 * wrapper
	 * 
	 * @param connection
	 *            the connection
	 */
	public void setConnection(Connection connection) {
		logger.debug("XARessourceImpl#setConnection(" + connection + ")");
		this.connection = connection;
	}

	/**
	 * Retrieves the physical JMS connection object.
	 * 
	 * @return the connection
	 */
	public Connection getConnection() {
		logger.debug("XARessourceImpl#getConnection()");
		return this.connection;
	}

	/**
	 * Starts work on behalf of a transaction branch specified in xid. If TMJOIN
	 * is specified, the start applies to joining a transaction previously seen
	 * by the resource manager. If TMRESUME is specified, the start applies to
	 * resuming a suspended transaction specified in the parameter xid. If
	 * neither TMJOIN nor TMRESUME is specified and the transaction specified by
	 * xid has previously been seen by the resource manager, the resource
	 * manager throws the XAException exception with XAER_DUPID error code.
	 * 
	 * @param xid
	 *            A global transaction identifier to be associated with the
	 *            resource.
	 * @param flags
	 *            One of TMNOFLAGS, TMJOIN, or TMRESUME.
	 * @throws XAException
	 *             An error has occurred. Possible exceptions are XA_RB*,
	 *             XAER_RMERR, XAER_RMFAIL, XAER_DUPID, XAER_OUTSIDE, XAER_NOTA,
	 *             XAER_INVAL, or XAER_PROTO.
	 */
	public void start(Xid xid, int flags) throws XAException {
		logger.debug("XARessourceImpl#start(" + xid + "," + flags + ")");
		try {
			mc._startXaTx();
		} catch (Exception e) {
			throw (XAException) new XAException().initCause(e);
		}
		getXAResource().start(xid, flags);
	}

	/**
	 * Ends the work performed on behalf of a transaction branch. The resource
	 * manager disassociates the XA resource from the transaction branch
	 * specified and lets the transaction complete. If TMSUSPEND is specified in
	 * the flags, the transaction branch is temporarily suspended in an
	 * incomplete state. The transaction context is in a suspended state and
	 * must be resumed via the start method with TMRESUME specified. If TMFAIL
	 * is specified, the portion of work has failed. The resource manager may
	 * mark the transaction as rollback-only. If TMSUCCESS is specified, the
	 * portion of work has completed successfully.
	 * 
	 * @param xid
	 *            A global transaction identifier that is the same as the
	 *            identifier used previously in the start method
	 * @param flags
	 *            One of TMSUCCESS, TMFAIL, or TMSUSPEND
	 * @throws XAException
	 *             An error has occurred. Possible XAException values are
	 *             XAER_RMERR, XAER_RMFAILED, XAER_NOTA, XAER_INVAL, XAER_PROTO,
	 *             or XA_RB*.
	 */
	public void end(Xid xid, int flags) throws XAException {
		logger.debug("XARessourceImpl#end(" + xid + "," + flags + ")");
		getXAResource().end(xid, flags);
	}

	/**
	 * Ask the resource manager to prepare for a transaction commit of the
	 * transaction specified in xid.
	 * 
	 * @param xid
	 *            A global transaction identifier.
	 * @return A value indicating the resource manager's vote on the outcome of
	 *         the transaction. The possible values are: XA_RDONLY or XA_OK. If
	 *         the resource manager wants to roll back the transaction, it
	 *         should do so by raising an appropriate XAException in the prepare
	 *         method.
	 * @throws XAException
	 *             An error has occurred. Possible exception values are: XA_RB*,
	 *             XAER_RMERR, XAER_RMFAIL, XAER_NOTA, XAER_INVAL, or
	 *             XAER_PROTO.
	 */
	public int prepare(Xid xid) throws XAException {
		logger.debug("XARessourceImpl#prepare(" + xid + ")");
		return getXAResource().prepare(xid);
	}

	/**
	 * Commits the global transaction specified by xid
	 * 
	 * @param xid
	 *            A global transaction identifier
	 * @param onePhase
	 *            If true, the resource manager should use a one-phase commit
	 *            protocol to commit the work done on behalf of xid.
	 * @throws XAException
	 *             An error has occurred. Possible XAExceptions are XA_HEURHAZ,
	 *             XA_HEURCOM, XA_HEURRB, XA_HEURMIX, XAER_RMERR, XAER_RMFAIL,
	 *             XAER_NOTA, XAER_INVAL, or XAER_PROTO.
	 */
	public void commit(Xid xid, boolean onePhase) throws XAException {
		logger.debug("XARessourceImpl#commit(" + xid + "," + onePhase + ")");
		try {
			getXAResource().commit(xid, onePhase);
		} finally {
			try {
				mc._endXaTx();
			} catch (Exception e) {
				throw (XAException) new XAException(e.getMessage())
						.initCause(e);
			}
		}
	}

	/**
	 * Informs the resource manager to roll back work done on behalf of a
	 * transaction branch.
	 * 
	 * @param xid
	 *            A global transaction identifier.
	 * @throws XAException
	 *             An error has occurred.
	 */
	public void rollback(Xid xid) throws XAException {
		logger.debug("XARessourceImpl#rollback(" + xid + ")");
		try {
			getXAResource().rollback(xid);
		} finally {
			try {
				mc._endXaTx();
			} catch (Exception e) {
				throw (XAException) new XAException(e.getMessage())
						.initCause(e);
			}
		}
	}

	/**
	 * Tells the resource manager to forget about a heuristically completed
	 * transaction branch.
	 * 
	 * @param xid
	 *            A global transaction identifier.
	 * @throws XAException
	 *             An error has occurred. Possible exception values are
	 *             XAER_RMERR, XAER_RMFAIL, XAER_NOTA, XAER_INVAL, or
	 *             XAER_PROTO.
	 */
	public void forget(Xid xid) throws XAException {
		logger.debug("XARessourceImpl#forget(" + xid + ")");
		getXAResource().forget(xid);
	}

	/**
	 * Obtains a list of prepared transaction branches from a resource manager.
	 * The transaction manager calls this method during recovery to obtain the
	 * list of transaction branches that are currently in prepared or
	 * heuristically completed states.
	 * 
	 * @param flag
	 *            One of TMSTARTRSCAN, TMENDRSCAN, TMNOFLAGS. TMNOFLAGS must be
	 *            used when no other flags are set in the parameter.
	 * @return The resource manager returns zero or more XIDs of the transaction
	 *         branches that are currently in a prepared or heuristically
	 *         completed state. If an error occurs during the operation, the
	 *         resource manager should throw the appropriate XAException.
	 * @throws XAException
	 *             An error has occurred. Possible values are XAER_RMERR,
	 *             XAER_RMFAIL, XAER_INVAL, and XAER_PROTO.
	 */
	public Xid[] recover(int flag) throws XAException {
		logger.debug("XARessourceImpl#recover(" + flag + ")");
		return getXAResource().recover(flag);
	}

	/**
	 * Obtains the current transaction timeout value set for this XAResource
	 * instance. If XAResource.setTransactionTimeout was not used prior to
	 * invoking this method, the return value is the default timeout set for the
	 * resource manager; otherwise, the value used in the previous
	 * setTransactionTimeout call is returned.
	 * 
	 * @return the transaction timeout value in seconds.
	 * @throws XAException
	 *             An error has occurred. Possible exception values are
	 *             XAER_RMERR and XAER_RMFAIL.
	 */
	public int getTransactionTimeout() throws XAException {
		logger.debug("XARessourceImpl#getTransactionTimeout()");
		return getXAResource().getTransactionTimeout();
	}

	/**
	 * Sets the current transaction timeout value for this XAResource instance.
	 * Once set, this timeout value is effective until setTransactionTimeout is
	 * invoked again with a different value. To reset the timeout value to the
	 * default value used by the resource manager, set the value to zero. If the
	 * timeout operation is performed successfully, the method returns true;
	 * otherwise false. If a resource manager does not support explicitly
	 * setting the transaction timeout value, this method returns false.
	 * 
	 * @param seconds
	 *            The transaction timeout value in seconds.
	 * @return true if the transaction timeout value is setsuccessfully;
	 *         otherwise false.
	 * @throws XAException
	 *             An error has occurred. Possible exception values are
	 *             XAER_RMERR, XAER_RMFAIL, or XAER_INVAL.
	 */
	public boolean setTransactionTimeout(int seconds) throws XAException {
		logger.debug("XARessourceImpl#setTransactionTimeout(" + seconds + ")");
		return getXAResource().setTransactionTimeout(seconds);
	}

	/**
	 * This method is called to determine if the resource manager instance
	 * represented by the target object is the same as the resouce manager
	 * instance represented by the parameter xaResource.
	 * 
	 * @param xaResource
	 *            An XAResource object whose resource manager instance is to be
	 *            compared with the resource manager instance of the target
	 *            object.
	 * @return true if it's the same RM instance; otherwise false.
	 * @throws XAException
	 *             An error has occurred. Possible exception values are
	 *             XAER_RMERR and XAER_RMFAIL.
	 */
	public boolean isSameRM(XAResource xaResource) throws XAException {
		logger.debug("XARessourceImpl#isSameRM(" + xaResource + ")");
		if (!(xaResource instanceof XAResourceImpl))
			return false;
		XAResourceImpl other = (XAResourceImpl) xaResource;
		return getXAResource().isSameRM(other);
	}

	/**
	 * Gets the XA resource of the managed connection
	 * 
	 * @return the XA resource of the managed connection
	 * @throws XAException
	 *             exception ocure if the operation fail
	 */
	private XAResource getXAResource() throws XAException {
		logger.debug("XARessourceImpl#getXAResource()");
		try {
			return mc._getXAResource();
		} catch (Exception e) {
			throw (XAException) new XAException(e.getMessage()).initCause(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "XAResourceImpl " + connection;
	}

}