/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaBeanBuilder.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.util;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * An util class used for create object and set properties by using reflection
 * mecanism.
 * 
 * Each instance of the connector should have one and only one instance of this
 * class.
 * 
 * @author Kien THAI
 * 
 */
public class JavaBeanBuilder implements Serializable {
	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	private Class mqCfClass;

	private Class mqQcfClass;

	private Class mqTcfClass;

	private Class mqXaCfClass;

	private Class mqXaQcfClass;

	private Class mqXaTcfClass;

	private Class mqQueueClass;

	private Class mqTopicClass;
	
	private Hashtable propertyNamesCache = new Hashtable();

	/**
	 * Create new JavaBeanBuilder instance,
	 * 
	 * @throws Exception
	 *             if the MQ JMS classes can not be created
	 */
	public JavaBeanBuilder() throws Exception {
		try {
			ClassLoader loader = Thread.currentThread().getContextClassLoader();
			
			mqCfClass = loader.loadClass(Constants.MQ_CF_CLASSNAME);
			mqQcfClass = loader.loadClass(Constants.MQ_QCF_CLASSNAME);
			mqTcfClass = loader.loadClass(Constants.MQ_TCF_CLASSNAME);

			mqXaCfClass = loader.loadClass(Constants.MQ_XA_CF_CLASSNAME);
			mqXaQcfClass = loader.loadClass(Constants.MQ_XA_QCF_CLASSNAME);
			mqXaTcfClass = loader.loadClass(Constants.MQ_XA_TCF_CLASSNAME);

			mqTopicClass = loader.loadClass(Constants.MQ_TOPIC_CLASSNAME);
			mqQueueClass = loader.loadClass(Constants.MQ_QUEUE_CLASSNAME);

		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not get MQ classes ", e);
		}
	}

	/**
	 * Create new MQ JMS Object and set properties for this new object
	 * 
	 * @param className
	 *            MQ JMS Class name
	 * @param properties
	 *            A Hashtable that hold PropertyName-PropertyValue paires
	 * @return new instance
	 * @throws Exception
	 *             if instance can not be created or one of the properties can
	 *             not be set
	 */
	public Object build(String className, Hashtable properties)
			throws Exception {
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#build(" + className + ","
				+ properties + ")");
		Object javaBean = null;

		if (javaBean == null) {
			javaBean = createObject(className);
			JavaBeanExecutor.setProperties(javaBean, properties);
		}

		return javaBean;
	}

	/**
	 * Create MQ JMS object using reflection
	 * 
	 * @param className
	 *            MQ JMS Class name
	 * @return new instance
	 * @throws Exception
	 *             if the new instance can not be created
	 */
	private Object createObject(String className) throws Exception {
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#createObject(" + className + ")");
		Class classObject = null;
		if (className.equals(Constants.MQ_CF_CLASSNAME))
			classObject = mqCfClass;
		else if (className.equals(Constants.MQ_QCF_CLASSNAME))
			classObject = mqQcfClass;
		else if (className.equals(Constants.MQ_TCF_CLASSNAME))
			classObject = mqTcfClass;
		else if (className.equals(Constants.MQ_XA_CF_CLASSNAME))
			classObject = mqXaCfClass;
		else if (className.equals(Constants.MQ_XA_QCF_CLASSNAME))
			classObject = mqXaQcfClass;
		else if (className.equals(Constants.MQ_XA_TCF_CLASSNAME))
			classObject = mqXaTcfClass;
		else if (className.equals(Constants.MQ_QUEUE_CLASSNAME))
			classObject = mqQueueClass;
		else if (className.equals(Constants.MQ_TOPIC_CLASSNAME))
			classObject = mqTopicClass;
		else
			throw new Exception("Class " + className + " is not supported");
		try {
			return classObject.newInstance();
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not create object of class "
					+ className, e);
		}
	}

	/**
	 * get property names for a given class
	 * 
	 * @param cl
	 *            class to introspect
	 * @throws Exception
	 *             If property can not be retreived
	 */
	public static String[] getPropertyNames(Class cl) {
		
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#getPropertyNames(" + cl + ")");
		
		Method[] methods = cl.getMethods();
		ArrayList properties = new ArrayList(methods.length);
		
		/**
		 * Look for methos whose name begins wit "get", and without parameter 
		 */
		for (int i = 0; i < methods.length; i++) {
			String name = methods[i].getName();
			if (name.startsWith("get")
					&& methods[i].getParameterTypes().length == 0) {
				properties.add(name.substring(3));
			}	
		}
		String[] props = null;
		if (properties.size() > 0) {
			props = new String[properties.size()];
			props = (String[]) properties.toArray(props);
		}
		return props;
	}
	
	
	public String[] getPropertyNames(String classFQN)
	throws Exception {
		
		//com.ibm.mq.jms.MQQueue q = new com.ibm.mq.jms.MQQueue();
		//com.ibm.mq.jms.MQXAConnectionFactory cfmq = new com.ibm.mq.jms.MQXAConnectionFactory();
		
		/*
		 * TODO this requires optimization: shhould be done once per class
		 */
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#getPropertyNames(" + classFQN + ")");
		String[] propertyNames = null;
		
		/*
		 * Look at the cache
		 */
		propertyNames = (String[] )propertyNamesCache.get(classFQN);
		if (propertyNames == null) {
			if (Constants.MQ_QUEUE_CLASSNAME.equals(classFQN)) {
				propertyNames = getPropertyNames(mqQueueClass);
			} else if (Constants.MQ_TOPIC_CLASSNAME.equals(classFQN)) {
				propertyNames = getPropertyNames(mqTopicClass);
			} else if (Constants.MQ_XA_CF_CLASSNAME.equals(classFQN)) {
				propertyNames = getPropertyNames(mqXaCfClass);
			} else if (Constants.MQ_XA_QCF_CLASSNAME.equals(classFQN)) {
				propertyNames = getPropertyNames(mqXaQcfClass);
			} else if (Constants.MQ_XA_TCF_CLASSNAME.equals(classFQN)) {
				propertyNames = getPropertyNames(mqXaTcfClass);
			} else if (Constants.MQ_CF_CLASSNAME.equals(classFQN)) {
				propertyNames = getPropertyNames(mqCfClass);
			} else if (Constants.MQ_QCF_CLASSNAME.equals(classFQN)) {
				propertyNames = getPropertyNames(mqQcfClass);
			} else if (Constants.MQ_TCF_CLASSNAME.equals(classFQN)) {
				propertyNames = getPropertyNames(mqTcfClass);
			} else {
				ClassLoader loader = Thread.currentThread()
						.getContextClassLoader();
				try {
					Class cl = loader.loadClass(classFQN);
					propertyNames = getPropertyNames(cl);
				} catch (ClassNotFoundException e) {
					logger.error(
							"JavaBeanBuilder#getPropertyNames : Cannot retrieve property names for "
									+ classFQN, e);
					throw e;
				}
			}
			// update the cache
			if (propertyNames != null) {
				propertyNamesCache.put(classFQN, propertyNames);
			}
		}
		return propertyNames;
	}
}