/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaBeanExecutor.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.util;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.StringTokenizer;

import javax.resource.ResourceException;

/**
 * An util class used mainly for getting and setting properties of JavaBean
 * object
 * 
 * @author Kien THAI
 * 
 */
public class JavaBeanExecutor {

	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * Parse from properties string to properties hastable
	 * 
	 * @param propString
	 *            A string that hold PropertyName-PropertyValue paires
	 * @return A Hashtable object that hold PropertyName-PropertyValue paires
	 */
	public static Hashtable parseToProperties(String propString) {
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#parseToProperties(" + propString + ")");
		Hashtable result = new Hashtable();
		if (propString == null || propString.trim().equals("")) {
			return result;
		}

		StringTokenizer tokenList = new StringTokenizer(propString,
				Constants.SEPARATOR);
		while (tokenList.hasMoreTokens()) {
			String propValuePair = (String) tokenList.nextToken();

			int loc = propValuePair.indexOf(Constants.SETTER);
			String propName = propValuePair.substring(0, loc);
			String propValue = propValuePair.substring(loc
					+ Constants.SETTER.length());

			result.put(propName, propValue);
		}

		return result;
	}

	/**
	 * Setting properties for an object
	 * 
	 * @param obj
	 *            Object to be set
	 * @param properties
	 *            A string that hold PropertyName-PropertyValue paires
	 * @throws Exception
	 *             If one of the properties can not be set
	 */
	public static void setProperties(Object obj, Hashtable properties)
			throws Exception {
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#setProperties(" + obj + "," + properties
				+ ")");
		Method[] methods = obj.getClass().getMethods();

		for (int i = 0; i < methods.length; i++) {
			String name = methods[i].getName();
			if (name.startsWith("set")) {
				String propName = name.substring(3);
				if (properties.containsKey(propName)) {
					String propValue = properties.get(propName).toString();
					runSetter(propValue, methods[i], obj);
				}
			}
		}
	}

	/**
	 * Set property for an object
	 * 
	 * @param obj
	 *            Object to be set property
	 * @param propertyName
	 *            Name of the property
	 * @param propertyValue
	 *            Value of the property
	 * @throws Exception
	 *             If property can not be set for the object
	 */
	public static void setProperty(Object obj, String propertyName,
			String propertyValue) throws Exception {
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#setProperty(" + obj + "," + propertyName
				+ "," + propertyValue + ")");

		Method[] methods = obj.getClass().getMethods();

		for (int i = 0; i < methods.length; i++) {
			String name = methods[i].getName();
			if (name.startsWith("set")) {
				String propName = name.substring(3);
				if (propertyName.equals(propName)) {
					runSetter(propertyValue, methods[i], obj);
					return;
				}
			}
		}
	}

	/**
	 * get property of an object
	 * 
	 * @param obj
	 *            Object to be get
	 * @param String
	 *            Property name
	 * @throws Exception
	 *             If property can not be retreived
	 */
	public static Object getProperty(Object obj, String propertyName)
			throws Exception {
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#getProperty(" + obj + "," + propertyName
				+ ")");
		String methodName = null;
		Class[] parameters = {};
		Object[] args = {};

		if (!StringUtils.isEmpty(propertyName))
			methodName = "get" + propertyName;
		Method method = obj.getClass().getMethod(methodName, parameters);
		return method.invoke(obj, args);
	}

	//
	// public static void set(Object obj, String properties)
	// throws Exception {
	// logger.debug("JavaBeanBuilder#setProperties("+obj+","+properties+")");
	// setProperties (obj, parseToProperties(properties));
	// }
	//	

	/**
	 * Setting a property for an object
	 * 
	 * @param value
	 *            Value of the property
	 * @param method
	 *            Method setter of the property
	 * @param obj
	 *            Object to be set
	 * @throws Exception
	 */
	public static void runSetter(String value, Method method, Object obj)
			throws Exception {
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#runSetter(" + value + "," + method + ","
				+ obj + ")");
		if (value == null || value.trim().equals("")) {
			return;
		}
		try {
			Class[] parameters = method.getParameterTypes();
			if (parameters.length == 1) {
				Object[] values = new Object[1];
				values[0] = convertType(parameters[0], value);
				method.invoke(obj, values);
			}
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not set value: " + value
					+ " by method: " + method + " on object: " + obj, e);
		}
	}

	/**
	 * Converts the type from String to the Class type.
	 * 
	 * @param type
	 *            Class name to which the conversion is required.
	 * @param parameter
	 *            String value to be converted.
	 * @return Converted value.
	 * @throws ResourceException
	 *             in case of the mismatch of parameter values or a security
	 *             violation.
	 */
	private static Object convertType(Class type, String parameter)
			throws Exception {
		if (logger.isDebug()) logger.debug("JavaBeanBuilder#convertType(" + type + "," + parameter
				+ ")");
		String typeName = type.getName();
		if (typeName.equals("java.lang.String")
				|| typeName.equals("java.lang.Object")) {
			return parameter;
		}

		if (typeName.equals("int") || typeName.equals("java.lang.Integer")) {
			return new Integer(parameter);
		}

		if (typeName.equals("short") || typeName.equals("java.lang.Short")) {
			return new Short(parameter);
		}

		if (typeName.equals("byte") || typeName.equals("java.lang.Byte")) {
			return new Byte(parameter);
		}

		if (typeName.equals("long") || typeName.equals("java.lang.Long")) {
			return new Long(parameter);
		}

		if (typeName.equals("float") || typeName.equals("java.lang.Float")) {
			return new Float(parameter);
		}

		if (typeName.equals("double") || typeName.equals("java.lang.Double")) {
			return new Double(parameter);
		}

		if (typeName.equals("java.math.BigDecimal")) {
			return new java.math.BigDecimal(parameter);
		}

		if (typeName.equals("java.math.BigInteger")) {
			return new java.math.BigInteger(parameter);
		}

		if (typeName.equals("boolean") || typeName.equals("java.lang.Boolean")) {
			return new Boolean(parameter);
		}
		return parameter;
	}

}